/**
 * 
 */
package sluceii.agent.decisionmaking;

import java.util.ArrayList;

import sluceii.agent.Developer;
import sluceii.land.development.DeveloperSpecialtyParam;
import sluceii.land.development.LandProfitInfoRecord;
import sluceii.land.development.SubdivisionType;
import sluceii.land.market.SimpleMarketLand;
import sluceii.land.market.decision.DecisionToBuy;
import sluceii.land.market.decision.DecisionToSell;
import sluceii.land.market.decision.LandProfitEstimator;
import sluceii.landscape.LandUnit;
import sluceii.landscape.Parcel;
import sluceii.landscape.Subdivision;

/**
 * @author Shipeng Sun
 *
 */
public class DeveloperDecisionHelper implements DecisionToBuy, DecisionToSell, LandProfitEstimator
{
	private Developer developerAgent;

	private double residentialLandTransportCoefficient; 
	
	public static LandUnit urbanCenterObj;
	
	private int curSubdivisionTypeIndex;

	private double savingInterest;

	private double borrowingInterest;

	
	/**
	 * The maximum stock number. If the number of parcels is beyond this number, developer will stop developing.
	 */
	private int maxStockNum;
	
	/**
	 * @return the developerAgent
	 */
	public Developer getDeveloperAgent() {
		return developerAgent;
	}

	/**
	 * @param developerAgent the developerAgent to set
	 */
	public void setDeveloperAgent(Developer developerAgent) {
		this.developerAgent = developerAgent;
	}

	/**
	 * @return the residentialLandTransportCoefficient
	 */
	public double getResidentialLandTransportCoefficient() {
		return residentialLandTransportCoefficient;
	}

	/**
	 * @param residentialLandTransportCoefficient the residentialLandTransportCoefficient to set
	 */
	public void setResidentialLandTransportCoefficient(
			double residentialLandTransportCoefficient) {
		this.residentialLandTransportCoefficient = residentialLandTransportCoefficient;
	}

	/**
	 * @return the savingInterest
	 */
	public double getSavingInterest() {
		return savingInterest;
	}

	/**
	 * @param savingInterest the savingInterest to set
	 */
	public void setSavingInterest(double savingInterest) {
		this.savingInterest = savingInterest;
	}

	/**
	 * @return the borrowingInterest
	 */
	public double getBorrowingInterest() {
		return borrowingInterest;
	}

	/**
	 * @param borrowingInterest the borrowingInterest to set
	 */
	public void setBorrowingInterest(double borrowingInterest) {
		this.borrowingInterest = borrowingInterest;
	}

	/**
	 * @return the maxStockNum
	 */
	public int getMaxStockNum() {
		return maxStockNum;
	}

	/**
	 * @param maxStockNum the maxStockNum to set
	 */
	public void setMaxStockNum(int maxStockNum) {
		this.maxStockNum = maxStockNum;
	}

	@Override
	public boolean isToSell(SimpleMarketLand landObj) 
	{
		// TODO Auto-generated method stub
		// If the current developer has a parcel, sell it.
		if(landObj instanceof Parcel)
		{
			return true;
		}
		
		return false;
	}

	@Override
	public double calcSellProbability(SimpleMarketLand landObj) 
	{
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public double calcAskPrice(SimpleMarketLand landObj) {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public double calcWTAPrice(SimpleMarketLand landObj) {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public double[] calcUtilityAndPricesForSell(SimpleMarketLand landObj) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean isToBuy() 
	{
		int numParcels = 0;
		for(SimpleMarketLand luObj : developerAgent.getPropertyList())
		{
			// If the current developer has a subdivision, don't buy more.
			// Always keep one subdivision in hand.
			if(luObj instanceof Subdivision)
			{
				return false;
			}
			else if(luObj instanceof Parcel)
			{
				numParcels++;
			}
			
			if(numParcels >= maxStockNum)
			{
				return false;
			}
		}
		
		return true;
	}

	@Override
	public double calcBuyProbability() {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public double calcBidPrice(SimpleMarketLand landObj) {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public double calcUtility(SimpleMarketLand landObj) 
	{
		// TODO Auto-generated method stub
		return 0;
	}
	
	/**
	 * Calculate the profit from the land unit expected by the developer
	 * The equation is 
	 * 
	 * @param landObj
	 * @return Expected profit, total cost, sale price, capital profit
	 */
	public double[] calcExpectedProfit(LandUnit landObj, double landOwerAskPrice, double averageDist)
	{
		if(landObj instanceof Subdivision)
		{
			averageDist = landObj.distanceTo(urbanCenterObj);
		
			double[] rslt = new double[4];
			double totalCost, salePrice, capitalProfit;
			double travelDiscount = this.calcHedonicNonconstantEstimation(landObj, averageDist);
			DeveloperSpecialtyParam curSpecialtyParam = this.developerAgent.getDevelopmentSpecialtyParams().get(this.curSubdivisionTypeIndex);
			
			totalCost = curSpecialtyParam.developCost + curSpecialtyParam.buildCost + landOwerAskPrice;
			salePrice = curSpecialtyParam.averageSalePrice - travelDiscount;
			double capitalAsset = this.developerAgent.getCapitalResource().getAssetQuantity();
			
			if(capitalAsset > totalCost)
				capitalProfit = (this.developerAgent.getCapitalResource().getAssetQuantity() - totalCost)*savingInterest;
			else
				capitalProfit = (this.developerAgent.getCapitalResource().getAssetQuantity() - totalCost)*borrowingInterest;
			
			double expectedProfit = (1-curSpecialtyParam.perceivedRisk)*(salePrice - totalCost)-curSpecialtyParam.perceivedRisk*(totalCost);
			
			expectedProfit *= Math.ceil(landObj.getNumOfCells()/SubdivisionType.subdivisionTypeCollections.get(curSubdivisionTypeIndex).getMeanSize());
			
			rslt[0] = expectedProfit;
			rslt[1] = totalCost;
			rslt[2] = salePrice;
			rslt[3] = capitalProfit;
			
			return rslt;
		}
		else
		{
			return null;
		}
	}

	/**
	 * @param landObj
	 * @param landOwnerWTA
	 * @return expectedTotalProfit, land profit, capital profit, totalCost, land cost, development cost, construction cost, unit sale price
	 */
	public double[] calcExpectedProfit(LandUnit landObj, double landOwnerWTA)
	{
		if(landObj instanceof Subdivision)
		{
			double[] rslt = new double[8];
			double totalCost, salePrice, capitalProfit = 0.0;
			double travelDiscount = this.calcHedonicNonconstantEstimation(landObj, 0.0);
			DeveloperSpecialtyParam curSpecialtyParam = this.developerAgent.getDevelopmentSpecialtyParams().get(curSubdivisionTypeIndex);
			
			if(curSubdivisionTypeIndex >= 2)
			{
				int i = 1;
				i += 1;
			}
			
			double estimatedHUNum = Math.ceil(landObj.getNumOfCells()/SubdivisionType.subdivisionTypeCollections.get(curSubdivisionTypeIndex).getMeanSize());
			
			totalCost = curSpecialtyParam.developCost + curSpecialtyParam.buildCost*estimatedHUNum + landOwnerWTA;
			salePrice = curSpecialtyParam.averageSalePrice - travelDiscount;
			
			/*
			double capitalAsset = this.developerAgent.getCapitalResource().getAssetQuantity();
			if(capitalAsset > totalCost)
				capitalProfit = (this.developerAgent.getCapitalResource().getAssetQuantity() - totalCost)*savingInterest;
			else
				capitalProfit = (this.developerAgent.getCapitalResource().getAssetQuantity() - totalCost)*borrowingInterest;
			*/
			
			double expectedProfit = capitalProfit + (1 - curSpecialtyParam.perceivedRisk)*(salePrice*estimatedHUNum)- totalCost;
			
			rslt[0] = expectedProfit;
			rslt[1] = expectedProfit - capitalProfit;
			rslt[2] = capitalProfit;
			rslt[3] = totalCost;
			rslt[4] = landOwnerWTA;
			rslt[5] = curSpecialtyParam.developCost;
			rslt[6] = curSpecialtyParam.buildCost*estimatedHUNum;
			rslt[7] = salePrice;
			
			return rslt;
		}
		else
		{
			return null;
		}
	}
	
	private double calcHedonicNonconstantEstimation(LandUnit landObj, double averageDist)
	{
		if(landObj instanceof Subdivision)
		{
			double dist = landObj.distanceTo(urbanCenterObj);
			double travelDiscount = (dist-averageDist)*residentialLandTransportCoefficient/(savingInterest + borrowingInterest)*2;
			travelDiscount *= (1+curSubdivisionTypeIndex*curSubdivisionTypeIndex);
			
			return travelDiscount;
		}
		else
		{
			return Double.NaN;
		}
	}
	
	@Override
	public double calcWTPPrice(SimpleMarketLand landObj) {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public ArrayList<Number> calcUtilityAndPricesForPurchase(SimpleMarketLand landObj) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public ArrayList<Number> estimateExpectedLandProfit(SimpleMarketLand luObj, double sellerWTA, ArrayList<Double> landProperties) 
	{
		return null;
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.econestimator.LandProfitEstimator#estimateExpectedLandProfit(sluceii.landscape.LandUnit, double)
	 */
	@Override
	public ArrayList<Number> estimateExpectedLandProfit(SimpleMarketLand luObj, double sellerWTA) 
	{
		// return profit and cost
		int specialtyNum = developerAgent.getDevelopmentSpecialtyParams().size();
		double[] calcRslts, maxRslts = null;
		double maxProfit = -Double.MAX_VALUE;
		int specialtyIndex = 0;
		
		for(int i = 0; i < specialtyNum; i++)
		{
			this.curSubdivisionTypeIndex = i;
			calcRslts = this.calcExpectedProfit((LandUnit) luObj, sellerWTA);
			if(calcRslts == null || calcRslts.length < 8)
			{
				return null;
			}
			
			if(calcRslts[0] > maxProfit)
			{
				maxProfit = calcRslts[0];
				maxRslts = calcRslts;
				specialtyIndex = i;
			}
		}
		
		if(maxRslts == null || maxRslts.length <= 0) return null;
		
		ArrayList<Number> rtnVals = new ArrayList<Number>();
		rtnVals.add(specialtyIndex);
		for(int k = 0; k < maxRslts.length; k++)
		{
			rtnVals.add(maxRslts[k]);
		}
		
		//expectedTotalProfit, land profit, capital profit, totalCost, land cost, development cost, construction cost, unit sale price
		LandProfitInfoRecord piRec = new LandProfitInfoRecord();
		piRec.developTypeIndex = specialtyIndex;
		piRec.landProfit = maxRslts[1];
		piRec.capitalProfit = maxRslts[2];
		piRec.constructionCost = maxRslts[6];
		piRec.developCost = maxRslts[5];
		piRec.landCost = maxRslts[4];
		piRec.salePrice = maxRslts[7];
		
		this.developerAgent.addLandProfitEstimationRecord((LandUnit) luObj, piRec);

		return rtnVals;
	}

}
