/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.agent.decisionmaking;

import repast.simphony.random.RandomHelper;
import sluceii.agent.RuralLandOwner;
import sluceii.params.AppParameters;
import sluceii.land.market.SimpleMarketLand;
import sluceii.land.market.decision.DecisionToSell;
import sluceii.landscape.LandUnit;
import sluceii.landscape.Parcel;
import sluceii.landscape.Subdivision;

/**
 * @author Shipeng Sun
 *
 */
public class LandOwnerDecisionToSellHelper implements DecisionToSell
{
	/**
	 * the land owner to sell land
	 */
	RuralLandOwner landOwner;
	
	private double annualAgProfit;
	private double produceTransportCoefficient;
	private double estimatedInterestRate;
	public static LandUnit urbanCenterObj;
	
	/**
	 * Default constructor
	 */
	public LandOwnerDecisionToSellHelper()
	{
		// TODO Auto-generated constructor stub
	}
	
	/**
	 * Constructor with land owner initialization
	 * @param ruralLandOwner
	 */
	public LandOwnerDecisionToSellHelper(RuralLandOwner ruralLandOwner)
	{
		// TODO Auto-generated constructor stub
		this.setOwner(ruralLandOwner);
	}
	
	/**
	 * Set the property owner, this might influence the decision to sell.
	 * 
	 * @param ruralLandOwner
	 */
	public void setOwner(RuralLandOwner ruralLandOwner)
	{
		// TODO Auto-generated method stub
		this.landOwner = ruralLandOwner;
	}

	/**
	 * @return the annualAgProfit
	 */
	public double getAnnualAgProfit() {
		return annualAgProfit;
	}

	/**
	 * @param d the annualAgProfit to set
	 */
	public void setAnnualAgProfit(double d) {
		this.annualAgProfit = d;
	}

	/**
	 * @return the produceTransportCoefficient
	 */
	public double getProduceTransportCoefficient() 
	{
		return produceTransportCoefficient;
	}

	/**
	 * @param d the produceTransportCoefficient to set
	 */
	public void setProduceTransportCoefficient(double d) 
	{
		this.produceTransportCoefficient = d;
	}

	/**
	 * @return the estimatedInterestRate
	 */
	public double getEstimatedInterestRate() {
		return estimatedInterestRate;
	}

	/**
	 * @param d the estimatedInterestRate to set
	 */
	public void setEstimatedInterestRate(double d) {
		this.estimatedInterestRate = d;
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToSell#calcSellProbability()
	 */
	@Override
	public double calcSellProbability(SimpleMarketLand landObj)
	{
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToSell#isToSell()
	 */
	@Override
	public boolean isToSell(SimpleMarketLand landObj)
	{
		double prob = RandomHelper.nextDouble();
		return (this.landOwner.getPressureToSell() == 1.0f && prob <= AppParameters.sellDecisionUpperBound);
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToSell#calcAskPrice(sluceii.landscape.LandUnit)
	 */
	@Override
	public double calcAskPrice(SimpleMarketLand landObj)
	{
		if(landObj instanceof Parcel)
		{
			// Currently just return the market value
			return ((Parcel)landObj).getMarketValue();
		}
		else if(landObj instanceof Subdivision)
		{
			double dist = ((Subdivision)landObj).distanceTo(urbanCenterObj);

			return (((Subdivision)landObj).getNumOfCells()*(annualAgProfit - dist*produceTransportCoefficient)/estimatedInterestRate);
		}
		else
		{
			return 0;
		}
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToSell#calcWTAPrice(sluceii.landscape.LandUnit)
	 */
	@Override
	public double calcWTAPrice(SimpleMarketLand landObj)
	{
		if(landObj instanceof Parcel)
		{
			// Currently just return the market value
			return ((Parcel)landObj).getMarketValue();
		}
		else if(landObj instanceof Subdivision)
		{
			double dist = ((Subdivision)landObj).distanceTo(urbanCenterObj);

			return (((Subdivision)landObj).getNumOfCells()*(annualAgProfit - dist*produceTransportCoefficient)/estimatedInterestRate);
		}
		else
		{
			return 0;
		}
	}
	
	@Override
	public double[] calcUtilityAndPricesForSell(SimpleMarketLand landObj)
	{
		{
			return null;
		}
	}
}
