/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.agent.decisionmaking;

import java.util.ArrayList;

import repast.simphony.random.RandomHelper;
import sluceii.agent.ResidentialHousehold;
import sluceii.params.AppParameters;
import sluceii.land.market.LandListingRecord;
import sluceii.land.market.SimpleMarketLand;
import sluceii.land.market.decision.DecisionToBuy;
import sluceii.land.market.decision.DecisionToSell;
import sluceii.land.market.decision.LandPriceEstimator;
import sluceii.land.market.utility.ConsumptionBundle;
import sluceii.landscape.LandCoverDataHelper;
import sluceii.landscape.LandUnit;
import sluceii.landscape.Parcel;
import umn.geog.helia.landscape.RasterLayer;

/**
 * @author Shipeng Sun
 *
 */
public class ResidentialHouseholdDecisionHelper implements DecisionToBuy, DecisionToSell
{
	/**
	 * the potential land buyer
	 */
	protected ResidentialHousehold householdAgent;
	
	/**
	 * 
	 */
	protected LandPriceEstimator<Parcel> landAppraiser;
	
	/**
	 * 
	 */
	public ResidentialHouseholdDecisionHelper()
	{
		// TODO Auto-generated constructor stub
	}

	/**
	 * @return the householdAgent
	 */
	public ResidentialHousehold getHouseholdAgent()
	{
		return householdAgent;
	}

	/**
	 * @param householdAgent the householdAgent to set
	 */
	public void setHouseholdAgent(ResidentialHousehold householdAgent)
	{
		this.householdAgent = householdAgent;
	}

	/**
	 * @return the landAppraiser
	 */
	public LandPriceEstimator<Parcel> getLandAppraiser()
	{
		return landAppraiser;
	}

	/**
	 * @param landAppraiser the landAppraiser to set
	 */
	public void setLandAppraiser(LandPriceEstimator<Parcel> landAppraiser)
	{
		this.landAppraiser = landAppraiser;
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToBuy#CalcMoveProbability()
	 */
	@Override
	public double calcBuyProbability()
	{
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToBuy#ToMove()
	 */
	@Override
	public boolean isToBuy()
	{
		// Twenty percent
		if(this.householdAgent.getPropertyList().size() > 0)
			return false;
		else 
			//return true;
		
		return (householdAgent.getPressureToBuy() == 1.0f && RandomHelper.nextDouble() <= AppParameters.buyDecisionUpperBound);
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToBuy#calcBidPrice(sluceii.landscape.LandCover)
	 */
	@Override
	public double calcBidPrice(SimpleMarketLand landObj)
	{
		// TODO Auto-generated method stub
		if(landObj instanceof Parcel)
		{
			RasterLayer<?> distLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.distanceLayerIndex);
			
			double dist = ((Parcel)landObj).getValue(distLayer, LandUnit.LandAggregateMethod.average).doubleValue();
			double transportCost = dist * AppParameters.unitTransportCost;
			
			double Y = householdAgent.getCapitalResource().getAssetQuantity()-transportCost;
			double U = this.calcUtility(landObj);
			double b = AppParameters.bInWTP;
			
			return Y/(1+(b*b)/(U*U));
			
			//return Math.min(landBuyer.getCaptialResource().getAssetQuantity(), ((Parcel)landObj).getMarketValue());
		}
		else
		{
			return 0;
		}
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToBuy#calcWTPPrice(sluceii.landscape.LandCover)
	 */
	@Override
	public double calcWTPPrice(SimpleMarketLand landObj)
	{
		// TODO Auto-generated method stub
		if(landObj instanceof Parcel)
		{
			return this.calcBidPrice(landObj);
			//return Math.min(landBuyer.getCaptialResource().getAssetQuantity(), ((Parcel)landObj).getMarketValue());
		}
		else
		{
			return 0;
		}
	}

	@Override
	public double calcUtility(SimpleMarketLand landObj)
	{
		ConsumptionBundle cBundle = LandCoverDataHelper.getConsumptionBundle((LandUnit)landObj, this.householdAgent);
		return this.householdAgent.getUtilStrategy().getUtility(cBundle);
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToSell#calcSellProbability()
	 */
	@Override
	public double calcSellProbability(SimpleMarketLand landObj)
	{
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToSell#isToSell()
	 */
	@Override
	public boolean isToSell(SimpleMarketLand landObj)
	{
		/*
		 *  
		 */
		int num = this.householdAgent.getPropertyList().size();
		SimpleMarketLand primaryLand = this.householdAgent.getPropertyList().get(this.householdAgent.getPrimaryResidenceIndex());
		
		if(num > 1 && landObj ==  primaryLand) 
		{
			// If there are other properties, don't sell the primary residence
			return false;
		}
		else if(num > 1 && landObj != primaryLand)
		{
			// For non-primary residence, the main reason to sell is to make profit.
			// If the estimated price of this land is higher than the purchase price by
			// a percentage. It will be put on the market.
		}
		else if(num == 1 && landObj == primaryLand)
		{
			// For primary residence, the main driver to increase housing utility. 
			
			// Utility Drop percentage
			double initUtil = this.householdAgent.getInitUtility(landObj);
			double curUtil = this.calcUtility(landObj);
			double estLandPrice = 0.0;
			double expectedResource = 0.0;
			
			if((initUtil-curUtil)/initUtil >= this.householdAgent.getUtilDropPtgRelocationTrigger())
			{
				// See if the estimated price + resource can afford a new, better house.
				estLandPrice = landAppraiser.estimateLandUnitPrice((LandUnit) landObj);
				((Parcel)landObj).setMarketValue((float)estLandPrice);
				expectedResource = estLandPrice + householdAgent.getCapitalResource().getAssetQuantity();
				
				// See if this resource can buy something.
				ArrayList<LandListingRecord<Parcel,?>> searchRslt = householdAgent.searchMarketListing(1000, curUtil, expectedResource, landAppraiser);
				if(searchRslt != null && searchRslt.size() > 10)
				{
					return true;
				}
			}
			else
			{
				return false;
			}
		}
		else
		{
			// num == 1 && landObj != primaryLand
			// For those who have no primary residence, they are pure speculators.
			// They will sell their last property for profit.
		}
	
		double prob = RandomHelper.nextDouble();
		return (this.householdAgent.getPressureToSell() == 1.0f && prob <= AppParameters.sellDecisionUpperBound);
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToSell#calcAskPrice(sluceii.landscape.LandUnit)
	 */
	@Override
	public double calcAskPrice(SimpleMarketLand landObj)
	{
		if(landObj instanceof Parcel)
		{
			// Currently just return the market value
			return ((Parcel)landObj).getMarketValue();
		}
		else
		{
			return 0;
		}
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToSell#calcWTAPrice(sluceii.landscape.LandUnit)
	 */
	@Override
	public double calcWTAPrice(SimpleMarketLand landObj)
	{
		if(landObj instanceof Parcel)
		{
			// Currently just return the market value
			return ((Parcel)landObj).getMarketValue();
		}
		else
		{
			return 0;
		}
	}

	@Override
	public ArrayList<Number> calcUtilityAndPricesForPurchase(SimpleMarketLand landObj)
	{
		if(landObj instanceof Parcel)
		{
			ConsumptionBundle cb = LandCoverDataHelper.getConsumptionBundle((LandUnit) landObj, this.householdAgent);
			
			RasterLayer<?> distLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.distanceLayerIndex);
			
			double dist = ((LandUnit) landObj).getValue(distLayer, LandUnit.LandAggregateMethod.average).doubleValue();
			double transportCost = dist * AppParameters.unitTransportCost;
			
			double Y = householdAgent.getCapitalResource().getAssetQuantity()-transportCost;
			double U = this.calcUtility(landObj);
			double b = AppParameters.bInWTP;
			
			double bidPrice = Y/(1+(b*b)/(U*U));
			
			ArrayList<Number> rslt = new ArrayList<Number>();
			// dist, open space amenity, tranCost, Y, b, U, wtp, bidPrice
			rslt.add(U);
			rslt.add(bidPrice);
			rslt.add(bidPrice);
			
			rslt.add(dist); // distance
			rslt.add(cb.getObservedValue(1)); // open space amenity
			rslt.add(transportCost); // transport cost
			rslt.add(Y); // housing budget
			rslt.add(b); // splitting factor

			return rslt;
		}
		else
		{
			return null;
		}
	}

	@Override
	public double[] calcUtilityAndPricesForSell(SimpleMarketLand landObj)
	{
		if(landObj instanceof Parcel)
		{
			// Currently just return the market value
			double wta = ((Parcel)landObj).getMarketValue();
			double U = this.calcUtility(landObj);
			return new double[]{U, wta, wta};
		}
		else
		{
			return null;
		}
	}
	
	public ArrayList<Number> calcLandProperties(LandUnit landObj)
	{
		if(landObj instanceof Parcel)
		{
			ConsumptionBundle cb = LandCoverDataHelper.getConsumptionBundle(landObj, this.householdAgent);
			
			RasterLayer<?> distLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.distanceLayerIndex);
			
			double dist = landObj.getValue(distLayer, LandUnit.LandAggregateMethod.average).doubleValue();
			double transportCost = dist * AppParameters.unitTransportCost;
			
			double Y = householdAgent.getCapitalResource().getAssetQuantity()-transportCost;
			double U = this.calcUtility(landObj);
			double b = AppParameters.bInWTP;
			
			double bidPrice = Y/(1+(b*b)/(U*U));
			
			ArrayList<Number> rslt = new ArrayList<Number>();
			// dist, open space amenity, tranCost, Y, b, U, wtp, bidPrice
			rslt.add(U);
			rslt.add(bidPrice);
			rslt.add(bidPrice);
			
			rslt.add(dist); // distance
			rslt.add(cb.getObservedValue(1)); // open space amenity
			rslt.add(transportCost); // transport cost
			rslt.add(Y);
			rslt.add(b);

			
			return rslt;
		}
		else
		{
			return null;
		}
	}

}
