/**
 * This implements several simple price estimation strategies.
 * 
 */
package sluceii.agent.econestimator;

import java.util.ArrayList;
import java.util.HashMap;
import sluceii.agent.SimpleAgent;
import sluceii.land.market.LandBidRecord;
import sluceii.land.market.LandBidRecord.BidRecordStatus;
import sluceii.land.market.SimpleMarketLand;
import sluceii.land.market.decision.LandPriceEstimator;
import sluceii.landscape.Parcel;
import sluceii.params.AppParameters;
import umn.geog.helia.landscape.RasterLayer;

/**
 * @author Shipeng Sun
 *
 */
public class SimplePriceEstimator extends LandPriceEstimator<Parcel>
{
	/**
	 * A reference to the transaction price layer
	 */
	protected RasterLayer<?> transactionPriceLayer;
	
	/**
	 * 
	 */
	protected RasterLayer<?> landUnitIDLayer;
	
	/**
	 * @return the transactionPriceLayer
	 */
	public RasterLayer<?> getTransactionPriceLayer()
	{
		return transactionPriceLayer;
	}

	/**
	 * @param transactionPriceLayer the transactionPriceLayer to set
	 */
	public void setTransactionPriceLayer(RasterLayer<?> transactionPriceLayer)
	{
		this.transactionPriceLayer = transactionPriceLayer;
	}
	
	/**
	 * @return the landUnitIDLayer
	 */
	public RasterLayer<?> getLandUnitIDLayer()
	{
		return landUnitIDLayer;
	}

	/**
	 * @param landUnitIDLayer the landUnitIDLayer to set
	 */
	public void setLandUnitIDLayer(RasterLayer<?> landUnitIDLayer)
	{
		this.landUnitIDLayer = landUnitIDLayer;
	}
	
	/**
	 * 
	 */
	public SimplePriceEstimator() {
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param agent
	 */
	public SimplePriceEstimator(SimpleAgent agent) {
		super(agent);
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.priceestimation.LandPriceEstimator#estimatedLandUnitPrice(sluceii.landscape.LandUnit)
	 */
	@Override
	public double estimateLandUnitPrice(SimpleMarketLand luObj) 
	{
		switch(priceSource)
		{
			case finalTransaction:
			{
				// Use the final transaction price layer to calculate the estimated price
				int xCenter = luObj.getXCenterIndex();
				int yCenter = luObj.getYCenterIndex();
				
				Double price = 0.0d;
			
				// Push the price and land unit id into a dictionary to avoid repeated accounting
				price = transactionPriceLayer.getValueAt(xCenter, yCenter).doubleValue();
				
				if(price > AppParameters.agResPrice)
					return (AppParameters.agResPrice + price)*0.5;
				else
					return AppParameters.agResPrice;
				
			}
			case bidRecords: // Use bidding records to get the transaction price and history
			{
				// Use the final transaction price layer to calculate the estimated price
				int xCenter = luObj.getXCenterIndex();
				int yCenter = luObj.getYCenterIndex();
				
				HashMap<Integer,Double> luIDValues = new HashMap<Integer,Double>();
				luIDValues.clear();
				
				double luPrice = 0.0d;
				ArrayList<LandBidRecord<Parcel,?>> bidRecords;
				LandBidRecord<Parcel,?> curRecord;
				
		
				bidRecords = landMarket.getBidRecordsByLandUnitID(landUnitIDLayer.getValueAt(xCenter, yCenter).intValue());
				if(bidRecords == null) return Double.NaN;
				
				for(int k = bidRecords.size()-1; k >=0; k--)
				{
					curRecord = bidRecords.get(k);
					if(curRecord.getRecordStatus() == BidRecordStatus.completed)
					{
						luPrice = curRecord.getBidPrice();
						break;
					}
				}
				
				if(luPrice > AppParameters.agResPrice)
					return (AppParameters.agResPrice + luPrice)*0.5;
				else
					return AppParameters.agResPrice;
			}
			default:
			{
			}
		}
		
		// Didn't find anything
		return Double.NaN;
	}

}
