/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.agent.initialization;

import cern.jet.random.Normal;
import cern.jet.random.Uniform;
import repast.simphony.context.Context;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.grid.Grid;
import sluceii.agent.Developer;
import sluceii.agent.ResidentialHousehold;
import sluceii.agent.RuralLandOwner;
import sluceii.agent.SimpleAgent;
import sluceii.app.lmm.DataPreparation;
import sluceii.params.AppParameters;
import sluceii.land.development.DeveloperSpecialtyParam;
import sluceii.land.development.SubdivisionType;
import sluceii.land.management.LandManagementStrategy;
import sluceii.landscape.LandCoverDataHelper;
import sluceii.landscape.Parcel;
import sluceii.landscape.Subdivision;
import umn.geog.helia.landscape.RasterLayer;
import umn.geog.helia.landscape.data.IntegerMatrix;
import umn.geog.helia.landscape.data.ShortIntMatrix;

/**
 * The random initializer creates and initializes actor agents.
 * This class has not been generalized so far. Change the code for different applications.
 * 
 * @author Shipeng Sun
 *
 */
public class RandomInitializer
{
	/**
	 * A reference context object
	 */
	Context<SimpleAgent> refContext;
	
	protected int numHouseholds;
	protected int numFarmer;
	protected int numDeveloper;
	
	/**
	 * 
	 */
	public RandomInitializer(Context<SimpleAgent> contextObj)
	{
		this.refContext = contextObj;
	}
	

	public Context<SimpleAgent> getRefContext() {
		return refContext;
	}


	public void setRefContext(Context<SimpleAgent> refContext) {
		this.refContext = refContext;
	}


	public int getNumHouseholds() {
		return numHouseholds;
	}


	public void setNumHouseholds(int numHouseholds) {
		this.numHouseholds = numHouseholds;
	}


	public int getNumFarmer() {
		return numFarmer;
	}


	public void setNumFarmer(int numFarmer) {
		this.numFarmer = numFarmer;
	}


	public int getNumDeveloper() {
		return numDeveloper;
	}


	public void setNumDeveloper(int numDeveloper) {
		this.numDeveloper = numDeveloper;
	}

	/**
	 * This method initialize rural land owners, developers, and buyers.
	 * @return
	 */
	@SuppressWarnings({ "unchecked", "unused" })
	public boolean init()
	{
		Grid<SimpleAgent> agentGrid = (Grid<SimpleAgent>) this.refContext.getProjection(sluceii.params.Constants.AGENT2D);
		
		// Agent ID
		int agentID = 1;
		
		///////////////////////////////////////////////////////////////////////
		// Initialize rural land owners using subdivIDLayer
		RasterLayer<IntegerMatrix> subdivIDLayer = (RasterLayer<IntegerMatrix>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.subdivisionIDLayerIndex);
		RasterLayer<IntegerMatrix> parcelIDLayer = (RasterLayer<IntegerMatrix>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.parcelIDLayerIndex);
		RasterLayer<ShortIntMatrix> lutLayer = (RasterLayer<ShortIntMatrix>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.landUseTypeLayerIndex);

		int subdivID, parcelID, cellID;
		for(int y = 0; y < subdivIDLayer.getRows(); y++)
		{
			for(int x = 0 ; x < subdivIDLayer.getCols(); x++)
			{
				// This is how cell ID is generated.
				// There should be a reverse procedure to get (x, y) from the ID.
				cellID = x + y*subdivIDLayer.getCols();
				subdivID = subdivIDLayer.getValueAt(x, y).intValue();
				parcelID = parcelIDLayer.getValueAt(x, y).intValue();
				
				if(LandCoverDataHelper.parcelList.containsKey(parcelID))
				{
					LandCoverDataHelper.parcelList.get(parcelID).addCellID(cellID);
				}
				else
				{
					Parcel pObj = new Parcel();
					pObj.setLandUnitID(parcelID);
					pObj.addCellID(cellID);
					
					LandCoverDataHelper.addToParcelList(pObj);
				}

				if(LandCoverDataHelper.subdivisionList.containsKey(subdivID))
				{
					LandCoverDataHelper.subdivisionList.get(subdivID).addParcelID(parcelID);
				}
				else
				{
					// Create a new subdivision and a rural land owner/farmer
					Subdivision sObj = new Subdivision();
					sObj.setLandUnitID(subdivID);
					sObj.addParcelID(parcelID);
					
					LandCoverDataHelper.addToSubdivisionList(sObj);
					
					// Don't need a rural land owner agent for urban core
					if(lutLayer.getValueAt(x, y).intValue() == 1)
					{
						if(DataPreparation.urbanCenterObj == null)
							DataPreparation.urbanCenterObj = sObj;
					}
					else
					{
						// Create an rural land owner for this subdivision/farm
						RuralLandOwner rloObj = new RuralLandOwner();
						rloObj.setAgentID(agentID++);
						
						rloObj.setPosition(x, y);
						rloObj.getCapitalResource().setAssetQuantity(0.0);
						
						rloObj.addProperty(sObj);
						
						refContext.add(rloObj);
						if(!agentGrid.moveTo(rloObj,rloObj.getX(),rloObj.getY()))
						{
							System.out.println("Error: Failed to place the agent on Grid");
						}
					}
				}
			}
		}
		
		Subdivision.parcelObjects = LandCoverDataHelper.parcelList;
		
		int pNum;
		float emv, ast; // estimated market value, asset
		double pfp = AppParameters.utilityBeta; // preference for proximity
		
		Normal budgetDist = RandomHelper.createNormal(AppParameters.meanBudget, AppParameters.sdBudget);
		Normal preferDist = RandomHelper.createNormal(AppParameters.utilityBeta, AppParameters.sdUtilBeta);
		
		Normal frequencyDistribution = RandomHelper.createNormal(2.0, 1);
		
		Uniform riskDistribution = RandomHelper.createUniform(AppParameters.minDevelopmentRisk, AppParameters.maxDevelopmentRisk);
		
		int centerX = subdivIDLayer.getCols()/2;
		int centerY = subdivIDLayer.getRows()/2;
		
		int subdivisionTypeNum = SubdivisionType.subdivisionTypeCollections.size();
		
		///////////////////////////////////////////////
		// Set up the developers
		// Right now, one single developer with unlimited resources
		for(int i = 0 ; i < this.numDeveloper; i++)
		{
			Developer daObj = new Developer();
			daObj.setAgentID(agentID++);
			daObj.setPosition(centerX, centerY-1);
			daObj.getCapitalResource().setAssetQuantity(AppParameters.initialDeveloperAsset);
			
			refContext.add(daObj);

			if(!agentGrid.moveTo(daObj, daObj.getX(), daObj.getY()))
			{
				System.out.println("Error: Failed to place the agent on Grid");
				return false;
			}
			
			// Initialize subdivision type related parameters
			for(int k = 0; k < subdivisionTypeNum; k++)
			{
				DeveloperSpecialtyParam param = new DeveloperSpecialtyParam();
				
				param.perceivedRisk = riskDistribution.nextDouble();
				
				switch(k)
				{
					case 0:
						param.developCost = 2000.0;
						param.buildCost = Math.pow(SubdivisionType.subdivisionTypeCollections.get(k).getMeanSize(),0.3)*150.0;
						param.averageSalePrice = 4200;
						
						break;
					case 1:
						param.developCost = 1800.0;
						param.buildCost = Math.pow(SubdivisionType.subdivisionTypeCollections.get(k).getMeanSize(),0.3)*150.0;
						param.averageSalePrice = 8000;

						break;
					case 2:
						param.developCost = 1200.0;
						param.buildCost = Math.pow(SubdivisionType.subdivisionTypeCollections.get(k).getMeanSize(),0.3)*150.0;
						param.averageSalePrice = 24000;
						break;
					case 3:
						param.developCost = 1000.0;
						param.buildCost = Math.pow(SubdivisionType.subdivisionTypeCollections.get(k).getMeanSize(),0.3)*150.0;
						param.averageSalePrice = 36000;
						break;
					default:
						param.developCost = 200.0;
						param.buildCost = Math.pow(SubdivisionType.subdivisionTypeCollections.get(k).getMeanSize(),0.3)*150.0;
						param.averageSalePrice = Math.pow(SubdivisionType.subdivisionTypeCollections.get(k).getMeanSize(),0.7)*5000.0;;
						break;
				}
					
				daObj.getDevelopmentSpecialtyParams().add(param);
			}
		}

		Uniform mgmtStrategyDistribution = RandomHelper.createUniform(0.0,4.0);
		
		// set up the buyers. They have no properties.
		for(int i = 0; i < this.numHouseholds; i++)
		{
			emv = 0;
			do
			{
				ast = (float)budgetDist.nextDouble();
			} while(ast <= AppParameters.lowBoundBudget);
			
			// urban parcels
			ResidentialHousehold hhObj = new ResidentialHousehold();
			hhObj.setAgentID(agentID++);
			
			hhObj.setPosition(centerX, centerY);
			hhObj.getCapitalResource().setAssetQuantity(ast);
			
			// Keep trying until a random number falls into the range.
			do
			{
				pfp = (float)preferDist.nextDouble();
				
			} while(pfp < 0.0 || pfp > 1.0 || pfp < (AppParameters.utilityBeta - AppParameters.rangeUtilBeta/2.0)||pfp > (AppParameters.utilityBeta + AppParameters.rangeUtilBeta/2.0));
			
			hhObj.setPreferenceForProximity(pfp);			
			
			// add land management strategies for residential households
			// randomly choose one of the four strategies
			int tmpI = (int)mgmtStrategyDistribution.nextDouble(); tmpI = tmpI < 0?0:tmpI; tmpI = tmpI > 3?3:tmpI;
			hhObj.setLandManagementStrategies(LandManagementStrategy.strategyCollections.get(tmpI));
			
			refContext.add(hhObj);

			if(!agentGrid.moveTo(hhObj,hhObj.getX(),hhObj.getY()))
			{
				System.out.println("Error: Failed to place the agent on Grid");
				return false;
			}
		}
	
		return true;
	}
}
