/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.agent;

import java.util.ArrayList;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.valueLayer.GridValueLayer;
import sluceii.agent.decisionmaking.LandOwnerDecisionToSellHelper;
import sluceii.params.AppParameters;
import sluceii.params.ModelParameters;
import sluceii.land.market.LandBidRecord;
import sluceii.land.market.LandListingRecord;
import sluceii.land.market.LandMarketListing;
import sluceii.land.market.LandBidRecord.BidRecordStatus;
import sluceii.land.market.LandListingRecord.ListingRecordStatus;
import sluceii.land.market.exchange.LandBuyingProcess;
import sluceii.land.market.exchange.LandSellingProcess;
import sluceii.landscape.LandCoverDataHelper;
import sluceii.landscape.Parcel;
import sluceii.params.Constants;
import umn.geog.helia.landscape.RasterLayer;

/**
 * Land brokers wait for "unsold" land parcels and put them on the market. 
 * @author Shipeng Sun
 *
 */
public class LandBroker extends SimpleAgent implements LandSellingProcess<Parcel>, LandBuyingProcess<Parcel>
{
	/**
	 * A Static reference to the land market listing
	 */
	public static LandMarketListing<Parcel> landMarket;
	
	/**
	 * 
	 */
	public LandBroker()
	{
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.SimpleAgent#getAgentTypeCode()
	 */
	@Override
	public int getAgentTypeCode() 
	{
		return AgentTypeCode.landBroker.ordinal();
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.SimpleAgent#step()
	 * 
	 * The logic for land owner
	 * 1. Decision to put land on sale
	 * 2. Put land on the market with WTA
	 * 3. Wait buyers to bid
	 * 4. Check and possibly accept offer from the bidder
	 * 5. Give up the land/house
	 * 6. Update resources
	 */
	@Override
	@ScheduledMethod(start = 1, interval = 1, shuffle = false, priority = 10)
	public void step()
	{
		// TODO Auto-generated method stub
		super.step();
		
		LandOwnerDecisionToSellHelper decisionSell = new LandOwnerDecisionToSellHelper();
		//decisionSell.setOwner(this);
		
		Parcel curParcelObj = null;
		float askPrice, wta;
		
		for(int i = 0; i < this.propertyList.size(); i++)
		{
			curParcelObj = (Parcel)this.propertyList.get(i);
			
			{
				askPrice = (float)decisionSell.calcAskPrice(curParcelObj);
				wta = (float)decisionSell.calcWTAPrice(curParcelObj);
				this.putLandOnMarket(curParcelObj,askPrice, wta);
			}
		}
		
		//this.checkBidOffers();
	}

	/**
	 * @param landCover
	 */
	public void putLandOnMarket(Parcel parcelObj, double askPrice, double wta)
	{
		// TODO Auto-generated method stub

		landMarket.addParcelOnMarket(parcelObj, this, askPrice, wta, AppParameters.maxBidNums, RunEnvironment.getInstance().getCurrentSchedule().getTickCount(), null);
		//System.out.println(parcelObj.getXCenterIndex() + "," + parcelObj.getYCenterIndex());
	}
	
	/**
	 * Check and possibly accept offer from the bidder
	 */
	@ScheduledMethod(start=1 , interval=1, shuffle= true, priority = 6)
	public void checkBidOffers()
	{
		Parcel curParcelObj = null;
		LandBidRecord<Parcel,?> bidRecObj = null;
		
		double maxBidPrice = -10000.0f;
		int maxBidIndex = -1;
		double wta = 0;
		ArrayList<LandBidRecord<Parcel,?>> bidRecordList = null;
		
		for(int i = 0; i < this.propertyList.size(); i++)
		{
			curParcelObj = (Parcel)this.propertyList.get(i);
			bidRecordList = landMarket.getBidRecordsByLandUnitID(curParcelObj.getParcelID());
			
			if(bidRecordList == null) continue;
			
			for(int j = 0; j < bidRecordList.size(); j++)
			{
				bidRecObj = bidRecordList.get(j);
				wta = bidRecObj.getWillingToAccept();
				
				if(bidRecObj.getLandUnitID() == curParcelObj.getParcelID())
				{
					// Check out the highest bid price
					if(bidRecObj.getRecordStatus() == BidRecordStatus.open && bidRecObj.getBidPrice() > maxBidPrice)
					{
						maxBidPrice = bidRecObj.getBidPrice();
						maxBidIndex = j;
					}
				}
			}
			
			// if the maximum bid price is not less than the willingness to accept price, deal
			if(maxBidPrice >= wta)
			{
				this.acceptBidOffer(bidRecordList.get(maxBidIndex));
				
				// Change the status of all other records
				for(int j = 0; j < bidRecordList.size(); j++)
				{
					bidRecObj = bidRecordList.get(j);
					
					if(j == maxBidIndex)
					{
						bidRecObj.setRecordStatus(BidRecordStatus.accepted);
					}
					else
					{
						bidRecObj.setRecordStatus(BidRecordStatus.rejected);
					}
				}

			}
			else
			{
				// There are multiple paths for this.
				// Lower the WTA or withdraw the listing
				// Change the status of all other records
				for(int j = 0; j < bidRecordList.size(); j++)
				{
					bidRecObj = bidRecordList.get(j);
					bidRecObj.setRecordStatus(BidRecordStatus.rejected);
				}
				
			}
		}		
	}

	/**
	 * Remove the parcel from the listing and this agent have to give up the ownership.
	 * The land owner might keep all the bidding records for future price formation.
	 * This owner needs to notify s/he has accepted the offer from one buyer.
	 * @param landBidRecord
	 */
	public void acceptBidOffer(LandBidRecord<Parcel,?> landBidRecord)
	{
		LandBuyingProcess<Parcel> buyerInterface = (LandBuyingProcess<Parcel>)landBidRecord.getBuyer();
		if(buyerInterface == null) return;
		buyerInterface.bidAccepted(landBidRecord);
	}
	
	/* (non-Javadoc)
	 * @see sluceii.land.exchange.LandSellingProcess#rejectBidOffer(sluceii.landmarket.LandBidRecord)
	 */
	public void rejectBidOffer(LandBidRecord<Parcel,?> landBidRecord)
	{
		LandBuyingProcess<Parcel> buyerInterface = (LandBuyingProcess<Parcel>)landBidRecord.getBuyer();
		if(buyerInterface != null)
			buyerInterface.bidRejected(landBidRecord);
	}
	
	/**
	 * The buyer has accepted the transaction.
	 * update the market listing and property list 
	 */
	public void clearTransaction(LandBidRecord<Parcel,?> landBidRecord)
	{
		// TODO: Remove the record or change the status of the record?
		// Remove this parcel from the market listing
		landMarket.removeListing(landBidRecord.getListingRecord());
		landBidRecord.getListingRecord().setRecordStaus(ListingRecordStatus.done);
		landBidRecord.setRecordStatus(BidRecordStatus.completed);
		
		// Remove this parcel from the owner's property list
		this.propertyList.remove(landBidRecord.getLandUnitObj());
		
		// Gain some cash
		this.capitalResource.addAsset(landBidRecord.getBidPrice());
	}

	@Override
	public void bidAccepted(LandBidRecord<Parcel, ?> landBidRecord)
	{
		// This is largely guaranteed.
		// Add this property/land parcel to this agent's property list
		
		this.propertyList.add(landBidRecord.getLandUnitObj());
		landBidRecord.setRecordStatus(BidRecordStatus.accepted);
		
		// Notify seller to clean up.
		// The seller must have implemented LandSellingProcess interface.
		LandSellingProcess<Parcel> sellerInterface = (LandSellingProcess<Parcel>)landBidRecord.getListingRecord().getSeller();
		sellerInterface.clearTransaction(landBidRecord);
		
		this.capitalResource.subtractAsset(landBidRecord.getBidPrice());
		
		this.finalizeBidAction(landBidRecord);
	}
	
	/**
	 * The seller will directly call this method to transfer her land to this broker.
	 * @param listedParcelObj
	 * @param price
	 */
	public void acceptPrivateBid(LandListingRecord<Parcel, ?> listedParcelObj, double price)
	{
		LandBidRecord<Parcel,LandBroker> bidRecord = new LandBidRecord<Parcel,LandBroker>();
		bidRecord.setListingRecObj(listedParcelObj);
		
		//System.out.println(listedParcelObj.getParcelObj().getXCenterIndex() + "," + listedParcelObj.getParcelObj().getYCenterIndex());
		
		bidRecord.setBidPrice(price);
		bidRecord.setWillingToPay(price);
		bidRecord.setRecordStatus(BidRecordStatus.open);
		bidRecord.setBuyer(this);
		
		landMarket.addBidRecord(bidRecord);
		
		if(AppParameters.preInfoAvailable)
		{
			RasterLayer<?> luTypeLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.landUseTypeLayerIndex);
			Parcel pObj = listedParcelObj.getLandUnitObj();
			pObj.putValue(luTypeLayer, 1);
		}
		
		listedParcelObj.getSeller().checkBidOffers();
	}

	private void finalizeBidAction(LandBidRecord<Parcel, ?> landBidRecord)
	{
		// Change the current bid parcel from rural to urban use
		RasterLayer<?> luTypeLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.landUseTypeLayerIndex);
		Parcel pObj = landBidRecord.getListingRecord().getLandUnitObj();
		pObj.putValue(luTypeLayer, 1);
		
		RasterLayer<?> tpcLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.finalPriceLayerIndex);
		pObj.putValue(tpcLayer, landBidRecord.getBidPrice());
		
		GridValueLayer vlLUType = (GridValueLayer)SimpleAgent.context.getValueLayer(Constants.LANDUSETYPE);
		pObj.putValue(vlLUType, tpcLayer);
				
		// Update the location of the agent/ Don't need to move land broker.
		// agentGrid.moveTo(this, pObj.getXCenterIndex(),pObj.getYCenterIndex());
		
		// this.pressureToBuy = 0.0f;
		ModelParameters.activity = true;
	}

	@Override
	public boolean bidOnLand(LandListingRecord<Parcel, ?> listedParcelObj,
			double utility, double bidPrice, double wtp)
	{
		// Brokers don't actively bid on land. They just wait for sellers.
		return false;
	}

	@Override
	public void bidRejected(LandBidRecord<Parcel, ?> landBidRecord)
	{
		// As sellers are willing to sell, this will never happen for brokers for the current version.
	}

	@Override
	public boolean bidOnLand(LandListingRecord<Parcel, ?> listedParcelObj, double utility, double bidPrice, double wtp, ArrayList<Number> extraLandAttri) 
	{
		return false;
	}

	@Override
	public void putLandOnMarket(Parcel landObj, double askPrice, double wta, ArrayList<Number> extraLandAttributes) 
	{
	}

}
