/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.agent;

import java.util.ArrayList;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import sluceii.agent.decisionmaking.LandOwnerDecisionToSellHelper;
import sluceii.app.lmm.ContextCreator;
import sluceii.params.AppParameters;
import sluceii.land.market.LandBidRecord;
import sluceii.land.market.LandMarketListing;
import sluceii.land.market.exchange.LandSellingAction;
import sluceii.land.market.exchange.LandSellingProcess;
import sluceii.landscape.Subdivision;

/**
 * @author Shipeng Sun
 *
 */
public class RuralLandOwner extends SimpleAgent implements LandSellingProcess<Subdivision>
{
	/**
	 * A Static reference to the land market listing
	 */
	public static LandMarketListing<Subdivision> landMarket;
	
	/**
	 * A delegate object for land selling procedure
	 */
	protected LandSellingAction<Subdivision, RuralLandOwner> sellAction;
	
	protected LandOwnerDecisionToSellHelper sellDecisionHelper;
	
	/**
	 * 
	 */
	private float pressureToSell;
	
	/**
	 * Default constructor
	 */
	public RuralLandOwner()
	{
		// Initialize the property list
		super();
		pressureToSell = 1.0f;
		
		this.sellAction = new LandSellingAction<Subdivision, RuralLandOwner>();
		sellDecisionHelper = new LandOwnerDecisionToSellHelper();
	}

	/**
	 * @return the pressureToSell
	 */
	public float getPressureToSell()
	{
		return pressureToSell;
	}

	/**
	 * @param pressureToSell the pressureToSell to set
	 */
	public void setPressureToSell(float pressureToSell)
	{
		this.pressureToSell = pressureToSell;
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.SimpleAgent#step()
	 * 
	 * The logic for land owner
	 * 1. Decision to put land on sale
	 * 2. Put land on the market with WTA
	 * 3. Wait buyers to bid
	 * 4. Check and possibly accept offer from the bidder
	 * 5. Give up the land/house
	 * 6. Update resources
	 */
	@Override
	@ScheduledMethod(start = 1, interval = 1, shuffle = false, priority = ScheduleParameters.FIRST_PRIORITY, duration = -1.0)
	public void step()
	{
		// TODO Auto-generated method stub
		super.step();
		
		sellAction.setLandMarket(RuralLandOwner.landMarket);
		sellAction.setSellerAgent(this);
		
		sellDecisionHelper.setOwner(this);
		sellDecisionHelper.setAnnualAgProfit(30.0);
		sellDecisionHelper.setEstimatedInterestRate(0.05);
		sellDecisionHelper.setProduceTransportCoefficient(AppParameters.ruralLandTransportCostCoefficient);
		
		Subdivision curLandObj = null;
		float askPrice, wta;
		
		for(int i = 0; i < this.propertyList.size(); i++)
		{
			curLandObj = (Subdivision)this.propertyList.get(i);
			
			if(sellDecisionHelper.isToSell(curLandObj))
			{
				askPrice = (float)sellDecisionHelper.calcAskPrice(curLandObj);
				wta = (float)sellDecisionHelper.calcWTAPrice(curLandObj);
				this.putLandOnMarket(curLandObj,askPrice, wta);
			}
		}
		
		this.checkBidOffers();
	}

	/**
	 * @param landCover
	 */
	public void putLandOnMarket(Subdivision landObj, double askPrice, double wta)
	{
		sellAction.putLandOnMarket(landObj, askPrice, wta, AppParameters.maxBidNums, RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
		this.pressureToSell = 0.0f;
		
		ContextCreator.contextAssist.writeStdOutMessage(" Rural land owner " + this.agentID + " offers subdivision " + landObj.getLandUnitID() + " at (" + landObj.getXCenterIndex() + "," + landObj.getYCenterIndex() + ") for sale ...", 3, true);
	}
	
	/**
	 * Check and possibly accept offer from the bidder
	 */
	public void checkBidOffers()
	{
		sellAction.checkBidOffers();
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.SimpleAgent#getAgentTypeCode()
	 */
	@Override
	public int getAgentTypeCode() 
	{
		return AgentTypeCode.ruralLandOwner.ordinal();
	}

	/**
	 * Remove the Subdivision from the listing and this agent have to give up the ownership.
	 * The land owner might keep all the bidding records for future price formation.
	 * This owner needs to notify s/he has accepted the offer from one buyer.
	 * @param landBidRecord
	 */
	public void acceptBidOffer(LandBidRecord<Subdivision,?> landBidRecord)
	{
		sellAction.acceptBidOffer(landBidRecord);
	}
	
	/* (non-Javadoc)
	 * @see sluceii.land.exchange.LandSellingProcess#rejectBidOffer(sluceii.landmarket.LandBidRecord)
	 */
	public void rejectBidOffer(LandBidRecord<Subdivision,?> landBidRecord)
	{
		sellAction.rejectBidOffer(landBidRecord);
	}
	
	/**
	 * The buyer has accepted the transaction.
	 * update the market listing and property list 
	 */
	public void clearTransaction(LandBidRecord<Subdivision,?> landBidRecord)
	{
		sellAction.clearTransaction(landBidRecord);

		Subdivision landObj = landBidRecord.getLandUnitObj();
		agentGrid.moveTo(this, AppParameters.worldXCenter, AppParameters.worldYCenter+1);
		ContextCreator.contextAssist.writeStdOutMessage(" Rural land owner " + this.agentID + " sold subdivision " + landObj.getLandUnitID() + " at (" + landObj.getXCenterIndex() + "," + landObj.getYCenterIndex() + ") for $" + landBidRecord.getBidPrice(), 2, true);
	}

	@Override
	public void putLandOnMarket(Subdivision landObj, double askPrice, double wta, ArrayList<Number> extraLandAttributes) 
	{
	}
}
