/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.app.lmm;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;

import repast.simphony.context.Context;
import repast.simphony.context.space.grid.GridFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ISchedule;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridBuilderParameters;
import repast.simphony.space.grid.SimpleGridAdder;
import repast.simphony.valueLayer.GridValueLayer;
import sluceii.agent.Developer;
import sluceii.agent.ResidentialHousehold;
import sluceii.agent.RuralLandOwner;
import sluceii.agent.SimpleAgent;
import sluceii.agent.decisionmaking.DeveloperDecisionHelper;
import sluceii.agent.decisionmaking.LandOwnerDecisionToSellHelper;
import sluceii.agent.initialization.RandomInitializer;
import sluceii.app.lmm.DataPreparation;
import sluceii.carve.CarveLandscapeSweeper;
import sluceii.carve.biomebgc.BiomeBGCRunner;
import sluceii.carve.iemss.IEMSSCaseBuilder;
import sluceii.land.development.SubdivisionType;
import sluceii.land.market.LandMarketListing;
import sluceii.landscape.LandCoverDataHelper;
import sluceii.landscape.Parcel;
import sluceii.landscape.Subdivision;
import sluceii.landscape.SubdivisionTemplateHelper;
import sluceii.params.AppParameters;
import sluceii.params.Constants;
import sluceii.params.DefaultContextCreatorAssist;
import sluceii.params.ModelParameters;
import sluceii.params.SimpleLogHelper;
import sluceii.params.ToolAndTips;
import umn.geog.helia.landscape.RasterLayer;

/**
 * @author sunsp
 *
 */
public class ContextCreator implements ContextBuilder<SimpleAgent>
{
	int param_xSize;
	int param_ySize;
	double param_endAt;
	
	/**
	 * 
	 */
	private SimpleLogHelper logHelper;

	/** Grid storing all {@code SimpleAgent} */
	private Grid<SimpleAgent> agentGrid;
	
	/** The Parcel Market */
	private LandMarketListing<Parcel> parcelMarket;
	
	/** The SubDivision Market */
	private LandMarketListing<Subdivision> subDivisionMarket;
	
	/**
	 */
	private Context<SimpleAgent> context2;
	
	/**
	 * This object helps handle miscellaneous tasks like retrieving host name, duration, write messages,
	 * update user panel, etc.
	 */
	public static DefaultContextCreatorAssist contextAssist; 
	
	/** Current wait period. Wait several rounds to confirm market clearance. */
	public static int currentWaitPeriod = 0;
	
	/**  */
	public static long totalStepNum = 1;
	
	/** these parcels will be run CARVE model */
	public static Collection<Integer> tmpParcelIDs;
	
	/** A Sweeper used to run Biome-BGC */
	public CarveLandscapeSweeper<IEMSSCaseBuilder> carveSweeper = null;
	
	/**
	 * Default constructor
	 */
	public ContextCreator()
	{
		super();
		contextAssist = new DefaultContextCreatorAssist();

	}

	/**
	 * Builds and returns a context. Building a context consists of filling it
	 * with agents, adding projects and so forth. When this is called for the
	 * master context the system will pass in a created context based on
	 * information given in the model.score file. When called for subcontexts,
	 * each subcontext that was added when the master context was built will be
	 * passed in.
	 * 
	 * @param context
	 * @return the built context.
	 */
	public Context<SimpleAgent> build(Context<SimpleAgent> context)
	{
		ContextCreator.currentWaitPeriod = 0;
		ContextCreator.totalStepNum = 1;
		ContextCreator.tmpParcelIDs = new ArrayList<Integer>(12);
		ModelParameters.resetVals();

		contextAssist.writeStdOutMessage("******************************************************************************", 1, false);
		contextAssist.writeStdOutMessage(" Started the context build method...", 2, true);
		
		contextAssist.writeHostStartTimeInfo();
		contextAssist.createUserPanel();
		contextAssist.setTotalTransactionNum(0);
		
		context2 = context;
		context2.clear();
		
		carveSweeper = null;
		carveSweeper = new CarveLandscapeSweeper<IEMSSCaseBuilder>();		
		carveSweeper.setRefContext(context2);

		SimpleAgent.context = context;

		setupParameters(); // Load parameters from the GUI
		
		setupLogHelper();
		
		// set the seed randomly
		//RandomHelper.setSeed((int)AppParameters.randomSeed);
		// Fix the random seed for debugging
		RandomHelper.setSeed(100);

		// Load the landscape data (external data defined in xml file)
		setupLandscape();

		setupGrids(); // set up the agent grids and the value layers
		setupActorAgents(); // create and initialize agents
		
		setupLandMarket(); // create the land market object
		
		//setupSellerAgents(); // create the seller agents and parameterize them
		//setupBuyerAgents(); // create the buyer agents and parameterize them
		//setupRemainingAgentCharacteristics(); // parameterize agent
		// characteristics that are the
		// same for all agents
		// regardless of if they're
		// buyers or sellers
		
		this.setupSchedule();
		
		contextAssist.writeStdOutMessage(" Finished the context build method", 1, true);
		contextAssist.writeStdOutMessage("******************************************************************************", 1, false);
		
		System.gc(); // useful for those
		return context;
	}

	private void setupLogHelper()
	{
		this.logHelper = new SimpleLogHelper(AppParameters.dataFolderName + AppParameters.logFileName, 1, false);
	}

	/**
	 * Set up the land market
	 */
	protected void setupLandMarket()
	{
		contextAssist.writeStdOutMessage(" Start setting up land markets...", 1, contextAssist.CurrentDateTimeString(2));

 		this.parcelMarket = new LandMarketListing<Parcel>();
 		this.parcelMarket.clear();
 		
 		this.subDivisionMarket = new LandMarketListing<Subdivision>();
 		this.subDivisionMarket.clear();

		ResidentialHousehold.landMarket = this.parcelMarket;
		RuralLandOwner.landMarket = this.subDivisionMarket;
		
		Developer.parcelMarket = this.parcelMarket;
		Developer.subdivisionMarket = this.subDivisionMarket;
		
		parcelMarket.addBidAttributeName(ResidentialHousehold.getExtraBidFieldNames());
		
		this.parcelMarket.setLandUnitList(LandCoverDataHelper.parcelList);
		
		contextAssist.writeStdOutMessage(" Finish setting up land markets.", 1, contextAssist.CurrentDateTimeString(2));
	}
	
	protected void setupSchedule()
	{
		// see http://repast.sourceforge.net/docs/reference/SIM/Working%20with%20the%20scheduler.html
		// for scheduler setup.
		// Schedule ContextCreator.step to run starting at time step x and
		// repeat every y steps
		ISchedule schedule = RunEnvironment.getInstance().getCurrentSchedule();
		// Set its priority as 1 lower than the agent actions.
		ScheduleParameters params = ScheduleParameters.createRepeating(Constants.STARTSTEP, Constants.INTERVALSTEP, 1, -1.0);
		// schedule.schedule(ScheduleParameters.createOneTime(0), this,
		// "setup");
		schedule.schedule(params, this, "step");

		if (RunEnvironment.getInstance().isBatch())
		{ 
			// If running in batch mode, tell the scheduler when to end each run.
			RunEnvironment.getInstance().endAt(AppParameters.maxSteps); // stop the model
		}
		// RunEnvironment.getInstance().pauseAt(param_endAt); // if it is not a
		// batch run then
		// pause the model
		// at the user
		// defined last time
		// step
	}

	/**
	 * The environment parameters contain the user-editable values that appear
	 * in the GUI. Get the parameters p and then specifically the initial
	 * numbers of wolves and sheep.
	 * 
	 * Currently most model parameters will be fixed in the ModelParameters file.
	 * In the later version, an XML file will be used to save all external meta-parameters.
	 * 
	 * @return void
	 */
	protected void setupParameters()
	{
		if (RunEnvironment.getInstance().isBatch())
		{
			Parameters p = RunEnvironment.getInstance().getParameters();
			AppParameters.fillParameters(p);
		}
		else
		{
			Parameters p = RunEnvironment.getInstance().getParameters();
			AppParameters.fillGUIParameters(p);
		}
	}

	/**
	 * Create a new 2D grid to model the discrete patches of residential
	 * households. The inputs to the GridFactory include the grid name, the
	 * context in which to place the grid, and the grid parameters. Grid
	 * parameters include the border specification, random adder for populating
	 * the grid with agents, boolean for multiple occupancy, and the dimensions
	 * of the grid.
	 * 
	 * @return void
	 */
	@SuppressWarnings("unchecked")
	protected void setupGrids()
	{
		GridFactoryFinder.createGridFactory(null).createGrid(
				Constants.AGENT2D,
				context2,
				new GridBuilderParameters<SimpleAgent>(
						new repast.simphony.space.grid.WrapAroundBorders(),
						new SimpleGridAdder<SimpleAgent>(), true, param_xSize,
						param_ySize));

		GridFactoryFinder.createGridFactory(null).createGrid(
				Constants.LANDSCAPE2D,
				context2,
				new GridBuilderParameters<SimpleAgent>(
						new repast.simphony.space.grid.WrapAroundBorders(),
						new SimpleGridAdder<SimpleAgent>(), true, param_xSize,
						param_ySize));

		GridValueLayer vlLUType = new GridValueLayer(Constants.LANDUSETYPE,
				true, new repast.simphony.space.grid.WrapAroundBorders(),
				param_xSize, param_ySize);

		context2.addValueLayer(vlLUType);

		GridValueLayer lmType = new GridValueLayer(Constants.MANAGEMENTTYPE,
				true, new repast.simphony.space.grid.WrapAroundBorders(),
				param_xSize, param_ySize);

		context2.addValueLayer(lmType);
		
		RasterLayer<?> luLayer = (RasterLayer<?>) LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.landUseTypeLayerIndex);
		for(int i = 0; i < param_xSize; i++)
		{
			for(int j = 0; j < param_ySize; j++)
			{
				// Set the land use layer just for display
				vlLUType.set(luLayer.getValueAt(i, j).doubleValue(), i,j);
				lmType.set(0,i,j);
			}
		}
		
		agentGrid = (Grid<SimpleAgent>) context2.getProjection(Constants.AGENT2D);
		SimpleAgent.agentGrid = agentGrid;
	}

	/**
	 * Set up and load landscape/land cover/land use/environment and other external data into
	 * the system. This will be done using LandCoverDataHelper
	 * 
	 * @see LandCoverDataHelper
	 */
	protected void setupLandscape()
	{
		contextAssist.writeStdOutMessage(" Start setting up landscape...", 1, contextAssist.CurrentDateTimeString(2));

		LandCoverDataHelper.clear();
		
		// Load all external data into the group object
		LandCoverDataHelper.loadRootLandscapeLayerGroupInfo(AppParameters.dataFolderName, AppParameters.landscapeGroupXMLFile);
		
		SubdivisionTemplateHelper.clear();
		SubdivisionTemplateHelper.loadTemplatesInfo(AppParameters.dataFolderName + AppParameters.templateXMLFile);
		SubdivisionTemplateHelper.loadTemplateData();
		
		// Set the reference layer for all actor agents
		SimpleAgent.referenceLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.referenceLayerIndex);
		
		DataPreparation.initLandscape(AppParameters.farmNumParam,AppParameters.farmSizeParam, true);

		this.param_xSize = SimpleAgent.referenceLayer.getCols();
		this.param_ySize = SimpleAgent.referenceLayer.getRows();
		
		// Initialize subdivision types
		DataPreparation.initSubdivisionTypes(4, SubdivisionType.subdivisionTypeCollections);

		contextAssist.writeStdOutMessage(" Finish setting up landscape.", 1, contextAssist.CurrentDateTimeString(2));
	}

	
	/**
	 * 
	 */
	@SuppressWarnings("deprecation")
	protected void setupActorAgents()
	{
		DataPreparation.initLandManagementStrategies();

		// int countSellers = 0;
		this.initAgents(AppParameters.dataFolderName + AppParameters.agentInitFile);
		
		// Update the parcel list, each parcel will get IDs of the cells it contains
		// RasterLayer<?> parcelIDLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.parcelIDLayerIndex);
		// LandCoverDataHelper.updateParcelList(parcelIDLayer);
		
		// for every location where land exists, create a seller agent
	}
	
	/**
	 * This method should be promoted to a separate class to initialize agents.
	 * @param fileName The CSV file having agent characteristics
	 */
	protected void initAgents(String fileName)
	{
		/*
		CSVfileLoader agentLoader = new CSVfileLoader(fileName, this.context2);
		
		if(!agentLoader.load())
		{
			System.out.println("Error: Cannot initialize actor agents from " + fileName	+ ". Please check.");
		}
		*/
		RandomInitializer riObj = new RandomInitializer(this.context2);
			
		riObj.setNumHouseholds(AppParameters.numHouseholds);
		riObj.setNumDeveloper(AppParameters.numDevelopers);
				
		if(!riObj.init())
		{
			contextAssist.writeStdOutMessage("Cannot initialize actor agents. Please check.",0,"Error: ");
		}
		
		LandOwnerDecisionToSellHelper.urbanCenterObj = DataPreparation.urbanCenterObj;
		DeveloperDecisionHelper.urbanCenterObj = DataPreparation.urbanCenterObj;
		
		LandCoverDataHelper.updateParcelCoordinates(SimpleAgent.referenceLayer);
		LandCoverDataHelper.updateSubdivisionCoordinates();
	}
	
	/**
	 * For each Model Step, run the following method
	 */
	public void step()
	{
		contextAssist.writeStdOutMessage(" Started Context Step Method...", 3, true);

		if(!ModelParameters.activity)
		{
			//saveLandscapeData();
			
			if(ContextCreator.currentWaitPeriod++ >= AppParameters.waitPeriodBeforeEnd)
			{
				RunEnvironment.getInstance().endRun();
				contextAssist.writeDurationInfo();
				//this.saveTransactionData();
			}
		}
		else
		{
			ContextCreator.currentWaitPeriod = 0;
			if(!AppParameters.onlyKeepFinalStep)
			{
				//saveLandscapeData();
			}
		}
		
		contextAssist.setTotalTransactionNum(ModelParameters.totalTansactionNum);
		
		// Reset these values when 
		ModelParameters.activity = false;
		// Run the carve model
		//runCarveSubModel();
		
		contextAssist.writeStdOutMessage(" Finished Step Method.", 3, true);
	}
	
	/**
	 * 
	 */
	private void saveLandscapeData()
	{
		// Save the intermediate land use type layer
		String totalStepStr = "" + ContextCreator.totalStepNum++;
		
		RasterLayer<?> dataLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.landUseTypeLayerIndex);
		String fileName = LandCoverDataHelper.saveRasterLayer3(dataLayer, true, true, totalStepStr, true);
		
		RasterLayer<?> pIDLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.parcelIDLayerIndex);
		String pIDName = LandCoverDataHelper.saveRasterLayer3(pIDLayer, true, true, totalStepStr, true);

		RasterLayer<?> tpcLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.finalPriceLayerIndex);
		String priceFile = LandCoverDataHelper.saveRasterLayer3(tpcLayer, true, true, totalStepStr, true);
		
		RasterLayer<?> sdTypeLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.subdivisionTypeLyaerIndex);
		String sdTypeFile = LandCoverDataHelper.saveRasterLayer3(sdTypeLayer, true, true, totalStepStr, true);

		String tickStr = String.valueOf(RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
		
		logHelper.writeLogWithoutTime(AppParameters.paramsValueToString(",") + "," + tickStr + "," + fileName + "," + pIDName + "," + sdTypeFile +"," + priceFile, 0);
	}
	
	private void saveTransactionData()
	{
		
		String logFile = AppParameters.dataFolderName + AppParameters.logFileName + "_transc.csv";
		FileOutputStream fileStream;
		try 
		{
			fileStream = new FileOutputStream(logFile, true);
			PrintWriter pwObj = new PrintWriter(fileStream);
			
			ArrayList<String> agentFields = SimpleAgent.getAgentAttributeNamesList();
			Developer.parcelMarket.exportTransactionData(pwObj,agentFields, agentFields);
			
			pwObj.println("**************************");
			Developer.subdivisionMarket.exportTransactionData(pwObj,agentFields, agentFields);
		} 
		catch (FileNotFoundException e) 
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * Run the Carve/Biome-BGC model at the end of the year.<br>
	 * The result is for the previous year.
	 */
	@SuppressWarnings("unused")
	private void runCarveSubModel()
	{
		RasterLayer<?> refLayer = (RasterLayer<?>) LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.parcelIDLayerIndex);
		RasterLayer<?> controlLayer = null;
		
		carveSweeper.setRefLayer(refLayer);
		carveSweeper.setControlLayer(controlLayer);
		carveSweeper.setRefContext(context2);
		int stepNum = (int)RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
		carveSweeper.setStepNum(stepNum);
		
		if(carveSweeper.getCaseBuilder() == null)
		{
			IEMSSCaseBuilder caseBuilderObj = new IEMSSCaseBuilder();
			caseBuilderObj.setPropertyTemplateFile(AppParameters.biomeBGCPropteryTemplateFile);
			int pid = (int)(RandomHelper.nextDouble()*100000000.0);
			caseBuilderObj.setProcessIDStr(ToolAndTips.getCurrentDateTime(2) + pid);
			
			carveSweeper.setCaseBuilder(caseBuilderObj);
		}
		
		if(carveSweeper.getCaseRunner() == null)
		{
			BiomeBGCRunner caseRunnerObj = new BiomeBGCRunner();
			carveSweeper.setCaseRunner(caseRunnerObj);
		}
	
		try
		{
			//carveSweeper.sweepLandscapeCells();
			if(ContextCreator.tmpParcelIDs.size() < 12)
			{
				Enumeration<Integer> pIDs = SimpleAgent.landOwnership.keys();
				int pID; SimpleAgent agentObj;
				while(pIDs.hasMoreElements() && tmpParcelIDs.size() < 12)
				{
					pID = pIDs.nextElement();
					if(LandCoverDataHelper.parcelList.containsKey(pID))
					{
						agentObj = SimpleAgent.landOwnership.get(pID);
						if(agentObj instanceof ResidentialHousehold)
						{
							tmpParcelIDs.add(pID);
						}
					}
				}
			}
			
			carveSweeper.sweepLandUnits(tmpParcelIDs);
		}
		catch (Exception e)
		{
			contextAssist.writeStdOutMessage(e.toString(), 1, "Error:");
			e.printStackTrace();
		}
		finally
		{
			ModelParameters.curModelYear += 1;
		}
	}
}
