/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.app.lmm;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

import repast.simphony.parameter.ParameterTreeSweeper;
import repast.simphony.parameter.Parameters;
import repast.simphony.parameter.xml.ParameterSweepParser;
import repast.simphony.util.collections.Pair;
import sluceii.land.development.SubdivisionType;
import sluceii.land.management.FertilizationStrategy;
import sluceii.land.management.LandManagementStrategy;
import sluceii.land.management.LitterRemovalStrategy;
import sluceii.landscape.LandCoverDataHelper;
import sluceii.landscape.LandUnit;
import sluceii.params.AppParameters;
import umn.geog.helia.landscape.RasterLayer;
import umn.geog.helia.landscape.data.FloatMatrix;
import umn.geog.helia.landscape.data.IntegerMatrix;
import umn.geog.helia.landscape.data.ShortIntMatrix;

/**
 * @author sunsp
 *
 */
public class DataPreparation
{
	public static LandUnit urbanCenterObj;
	
	/**
	 * 
	 */
	public DataPreparation()
	{
	}

	/**
	 * @param args
	 */
	public static void main(String[] args)
	{
		//DataPreparation dp = new DataPreparation();
		//dp.generatePIN();
		URL r;
		try
		{
			r = new File("C:\\Users\\sunsp\\workspace\\SLUCEII20100204\\batch\\aag_batch_params_lm_levels.xml").toURI().toURL();
			ParameterSweepParser psp = new ParameterSweepParser(r);
			
			Pair<Parameters,ParameterTreeSweeper> prams = psp.parse();
			
			Parameters p = prams.getFirst();
			ParameterTreeSweeper pts = prams.getSecond();
			
			AppParameters.fillParameters(p);
			System.out.println(AppParameters.paramsValueToString(","));
			
			pts.next(p);
			
			AppParameters.fillParameters(p);
			System.out.println(AppParameters.paramsValueToString(","));

			
		}
		catch (MalformedURLException e)
		{
			e.printStackTrace();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		
		/*
		
		ArrayList<String> targetArray = new ArrayList<String>(10);
		ArrayList<Double> criteriaArray = new ArrayList<Double>(10);
		
		for(int i = 0; i < 10; i++)
		{
			targetArray.add(i, "element " + i);
			criteriaArray.add(i, RandomHelper.nextDouble());
			System.out.println(criteriaArray.get(i) + "," + targetArray.get(i));
		}
		
		Integer indexArray[] = new Integer[targetArray.size()];
		for(int i = 0; i < indexArray.length; i++)
		{
			indexArray[i] = i;
		}
		
		// This is always in ascending order
		Arrays.sort(indexArray, new LinkedComparator(LinkedComparatorType.index, criteriaArray));
		int num = 8;
		ArrayList<String> rslt = new ArrayList<String>(num);
		
		for(int i = 0; i < num; i++)
		{
			if(true)
			{
				rslt.add(targetArray.get(indexArray[indexArray.length - 1- i]));
			}
			else
			{
				rslt.add(targetArray.get(indexArray[i]));
			}
			
		}
		
		for(int i = 0; i < rslt.size(); i++)
		{
			System.out.println(criteriaArray.get(i) + "," + rslt.get(i));
		}
		*/
	}
	
	/**
	 * Initialize a simple landscape with farms/subdivisions.
	 * 
	 * @param numFarmParam
	 * @param sizeFarmParam
	 * @param withUrbanCenter
	 */
	@SuppressWarnings("unchecked")
	public static void initLandscape(int numFarmParam, int sizeFarmParam, boolean withUrbanCenter)
	{
		RasterLayer<IntegerMatrix> subdivIDLayer = (RasterLayer<IntegerMatrix>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.subdivisionIDLayerIndex);
		RasterLayer<IntegerMatrix> parcelIDLayer = (RasterLayer<IntegerMatrix>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.parcelIDLayerIndex);
		RasterLayer<FloatMatrix> distLayer = (RasterLayer<FloatMatrix>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.distanceLayerIndex);
		RasterLayer<ShortIntMatrix> lutLayer = (RasterLayer<ShortIntMatrix>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.landUseTypeLayerIndex);
		RasterLayer<ShortIntMatrix> lctLayer = (RasterLayer<ShortIntMatrix>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.landCoverTypeLayerIndex);
		RasterLayer<ShortIntMatrix> subdivisionTypeLayer = (RasterLayer<ShortIntMatrix>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.subdivisionTypeLyaerIndex);
		RasterLayer<FloatMatrix> tpcLayer = (RasterLayer<FloatMatrix>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AppParameters.finalPriceLayerIndex);
		
		int sizeFarm = sizeFarmParam;
		int width = (2*numFarmParam+1)*(sizeFarm);
		int height = width;

		AppParameters.worldXSize = width;
		AppParameters.worldYSize = height;
		
		double[] extent = {0, 0, width, height};
		
		IntegerMatrix sudDivPinMatrix = new IntegerMatrix(width,height);
		subdivIDLayer.setDataMatrix(sudDivPinMatrix);
		subdivIDLayer.setExtent(extent);
		
		IntegerMatrix parcelPinMatrix = new IntegerMatrix(width,height);
		parcelIDLayer.setDataMatrix(parcelPinMatrix);
		parcelIDLayer.setExtent(extent);
		
		FloatMatrix disMatrix = new FloatMatrix(width, height);
		distLayer.setDataMatrix(disMatrix);
		distLayer.setExtent(extent);
		
		FloatMatrix tpcMatrix = new FloatMatrix(width, height);
		tpcLayer.setDataMatrix(tpcMatrix);
		tpcLayer.setExtent(extent);
		
		ShortIntMatrix lutMatrix = new ShortIntMatrix(width, height);
		lutLayer.setDataMatrix(lutMatrix);
		lutLayer.setExtent(extent);

		ShortIntMatrix lctMatrix = new ShortIntMatrix(width, height);
		lctLayer.setDataMatrix(lctMatrix);
		lctLayer.setExtent(extent);
		
		ShortIntMatrix sdtMatrix = new ShortIntMatrix(width, height);
		subdivisionTypeLayer.setDataMatrix(sdtMatrix);
		subdivisionTypeLayer.setExtent(extent);
		
		float cX = AppParameters.worldXCenter ;
		float cY = AppParameters.worldYCenter ;
		int subdivX = 0;
		int subdivY = 0;
		
		for(int y = 0; y < subdivIDLayer.getRows(); y++)
		{
			for(int x = 0 ; x < subdivIDLayer.getCols(); x++)
			{
				subdivX = (int) Math.ceil(x/sizeFarm);
				subdivY = (int) Math.ceil(y/sizeFarm);
				
				subdivIDLayer.putValueAt(x, y, subdivX + 1 + subdivY*(2*numFarmParam+1));
				parcelIDLayer.putValueAt(x, y, subdivX + 1 + subdivY*(2*numFarmParam+1));
				
				if(withUrbanCenter && subdivX == numFarmParam && subdivY == numFarmParam)
				{
					// Urban core
					lutLayer.putValueAt(x, y, 1);
					sdtMatrix.putValue(x, y, 8);
				}
				else
				{
					lutLayer.putValueAt(x, y, 0);
					sdtMatrix.putValue(x, y, 0);
				}
				
				distLayer.putValueAt(x, y, Math.sqrt((x-cX)*(x-cX)+(y-cY)*(y-cY)));
				tpcLayer.putValueAt(x, y, 0.0f);
				lctLayer.putValueAt(x, y, 4);
			}
		}
	}

	public static void initSubdivisionTypes(int typeNum, ArrayList<SubdivisionType> subdivisionTypeCollections) 
	{
		if(typeNum < 1 || subdivisionTypeCollections == null) return;
		subdivisionTypeCollections.clear();
		
		SubdivisionType stA = new SubdivisionType((byte) 1,"Country Type",2);
		subdivisionTypeCollections.add(stA);
		
		SubdivisionType stC = new SubdivisionType((byte) 2,"Horticultural Small",4);
		subdivisionTypeCollections.add(stC);
		
		SubdivisionType stB = new SubdivisionType((byte) 3,"Horticultural Large",12);
		subdivisionTypeCollections.add(stB);
		
		SubdivisionType stD = new SubdivisionType((byte) 4,"Remnant with Woods Large",18);
		subdivisionTypeCollections.add(stD);
	}

	public static void initLandManagementStrategies()
	{
		if(LandManagementStrategy.strategyCollections == null)
		{
			LandManagementStrategy.strategyCollections = new ArrayList<ArrayList<LandManagementStrategy>>();
		}
		else
			LandManagementStrategy.strategyCollections.clear();
		
		// Do nothing		
		ArrayList<LandManagementStrategy> s1 = new ArrayList<LandManagementStrategy>();
		LandManagementStrategy.strategyCollections.add(s1);

		// Add fertilizer
		ArrayList<LandManagementStrategy> s2 = new ArrayList<LandManagementStrategy>();
		FertilizationStrategy fS = new FertilizationStrategy();
		fS.type = LandManagementStrategy.ManagementStrategyType.absoluteQuantity;
		fS.quantityChange = 0.0;//0.016;
		s2.add(fS);
		LandManagementStrategy.strategyCollections.add(s2);

		// Removal 
		ArrayList<LandManagementStrategy> s3 = new ArrayList<LandManagementStrategy>();
		LitterRemovalStrategy rS = new LitterRemovalStrategy();
		rS.type = LandManagementStrategy.ManagementStrategyType.percentChange;
		rS.quantityChange = 0.0; //0.5;
		s3.add(rS);
		LandManagementStrategy.strategyCollections.add(s3);

		// Both add and removal
		ArrayList<LandManagementStrategy> s4 = new ArrayList<LandManagementStrategy>();
		s4.add(fS);
		s4.add(rS);
		LandManagementStrategy.strategyCollections.add(s4);
	}
}
