/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.land.market.decision;

import java.util.ArrayList;
import sluceii.land.market.SimpleMarketLand;

/**
 * This interface specifies how a farmer/developer/household/family/resident decide to buy/move.
 * 
 * @author Shipeng Sun
 *
 */
public interface DecisionToBuy
{
	/**
	 * This make yes/no, true/false decision to buy/move. 
	 * This is purely based on the characteristics of the potential buyer.
	 * 
	 * @return if the agent decide to search land 
	 */
	public boolean isToBuy();
	
	/**
	 * This method generate a probability instead of binary decision
	 * 
	 * @return a probability of buying/moving
	 */
	public double calcBuyProbability();
	
	/**
	 * Calculate a bid price for landObj
	 * @param landObj
	 * @return
	 */
	public double calcBidPrice(SimpleMarketLand landObj);
	
	/**
	 * Calculate a utility for the land
	 * @param landObj
	 * @return
	 */
	public double calcUtility(SimpleMarketLand landObj);
	
	/**
	 * Calculate the Willingness to Pay for the land
	 * @param landObj
	 * @return
	 */
	public double calcWTPPrice(SimpleMarketLand landObj);
	
	/**
	 * Calculate the utility, bid price, and Willingness to Pay, and more.
	 * @param landObj
	 * @return
	 */
	public ArrayList<Number> calcUtilityAndPricesForPurchase(SimpleMarketLand landObj);
}
