/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.land.market.decision;

import sluceii.land.market.LandMarketListing;
import sluceii.land.market.SimpleMarketAgent;
import sluceii.land.market.SimpleMarketLand;

/**
 * Land price estimator is used by an agent to estimated the market value of a land unit.
 * This abstract one defines the structure of the class. Subclasses can implement neighborhood 
 * average, weighted neighborhood average, linear regression, or other estimators.
 * 
 * @author Shipeng Sun
 *
 */
public abstract class LandPriceEstimator<T extends SimpleMarketLand>
{
	
	/**
	 * @author Shipeng Sun
	 * There are two price sources. One is the final transaction price without information
	 * on sellers, buyers, and transaction time. The other is bidding records, which have
	 * more information.
	 */
	public enum PriceSourceType {finalTransaction, bidRecords};
	
	/**
	 * The agent who makes the estimation. This agent can buy or sell the land.
	 */
	protected SimpleMarketAgent agent;

	/**
	 * A reference to the land market listing
	 */
	protected LandMarketListing<T> landMarket;	
	
	/**
	 * the price source
	 */
	protected PriceSourceType priceSource;
	
	/**
	 * Default constructor
	 */
	public LandPriceEstimator()
	{
		// TODO Auto-generated constructor stub
	}

	/**
	 * Constructor
	 */
	public LandPriceEstimator(SimpleMarketAgent agent)
	{
		this.agent = agent;
	}

	/**
	 * @return the agent
	 */
	public SimpleMarketAgent getAgent()
	{
		return agent;
	}

	/**
	 * @param agent the agent to set
	 */
	public void setAgent(SimpleMarketAgent agent)
	{
		this.agent = agent;
	}

	/**
	 * @return the landMarket
	 */
	public LandMarketListing<T> getLandMarket()
	{
		return landMarket;
	}

	/**
	 * @param landMarket the landMarket to set
	 */
	public void setLandMarket(LandMarketListing<T> landMarket)
	{
		this.landMarket = landMarket;
	}



	/**
	 * @return the priceSource
	 */
	public PriceSourceType getPriceSource()
	{
		return priceSource;
	}

	/**
	 * @param priceSource the priceSource to set
	 */
	public void setPriceSource(PriceSourceType priceSource)
	{
		this.priceSource = priceSource;
	}

	/**
	 * @param luObj
	 * @return the estimated land unit price
	 */
	public abstract double estimateLandUnitPrice(SimpleMarketLand luObj); 
}
