/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.land.market.decision;

import java.util.ArrayList;
import sluceii.land.market.LandListingRecord;
import sluceii.land.market.SimpleMarketLand;

/**
 * This interface defines how buyers search the records in the market listing.
 * @author Shipeng Sun
 *
 */
public interface LandSearch<L extends SimpleMarketLand>
{
	/**
	 * This method defines how a buyer search the list of available properties.
	 * In principle, the searching process should have at least two inter-related processes.
	 * 
	 * The first one is to find a sub-sample as a result of incomplete information. This can be
	 * done in two ways. One is to define a destination neighborhood. All parcels/properties outside
	 * this neighborhood will be excluded. The other one is define a probability surface. Different
	 * parcels/properties have various probabilities being chose by the buyer. The most common formula
	 * for such probability is the negative exponential probability. 
	 * 
	 * The second one is to assign a utility to each property in the whole list or the sub-sample list.
	 * Several methods have been defined in the utility packages. 
	 * @see sluceii.agent.utility
	 * 
	 * @param targetNum the number of properties/land parcels/houses returned
	 * @param sampleNum the size of the sample
	 * @return A list of land listing records
	 */
	public ArrayList<LandListingRecord<L,?>> searchProperties(int targetNum, int sampleNum);
	
	/**
	 * Those profit-seeking land buyers use this method to find the best land to bid on. 
	 * @param totalSearchNumber
	 * @param minUtil
	 * @param resourceQuantity
	 * @param landAppraiser
	 * @return
	 */
	public ArrayList<LandListingRecord<L, ?>> searchProperties(int targetNum, int totalSearchNumber,double minProfit,double resourceQuantity, LandProfitEstimator landAppraiser);

	/**
	 * Those highest-utility-seeking buyers use this method find the best land/housing to bid on with a given price estimator
	 * @param totalSearchNumber
	 * @param minUtil
	 * @param resourceQuantity
	 * @param landAppraiser
	 * @return
	 */
	public ArrayList<LandListingRecord<L, ?>> searchProperties(int totalSearchNumber,double minUtil,double resourceQuantity,LandPriceEstimator<L> landAppraiser);
}
