/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.land.market.exchange;

import java.util.ArrayList;

import sluceii.land.market.LandBidRecord;
import sluceii.land.market.LandListingRecord;
import sluceii.land.market.SimpleMarketLand;

/**
 * @author Shipeng Sun
 * This is an interface for all land buyers (households, developers).
 * All land buyers on the land market must implement this interface.
 * Without this, the bidding process would not work. 
 */
public interface LandBuyingProcess<L extends SimpleMarketLand>
{
	/**
	 * Bid on the specified parcel object.
	 * Depending on the limit on the number of bids on a parcel, this may or may not succeed.

	 * @param listedParcelObj
	 * @return if the operation is successful
	 */
	public boolean bidOnLand(LandListingRecord<L,?> listedParcelObj, double utility, double bidPrice, double wtp);
	
	/**
	 * Bid on the specified parcel object.
	 * Depending on the limit on the number of bids on a parcel, this may or may not succeed.

	 * @param listedParcelObj
	 * @return if the operation is successful
	 */
	public boolean bidOnLand(LandListingRecord<L,?> listedParcelObj, double utility, double bidPrice, double wtp, ArrayList<Number> extraLandAttri);

	/**
	 * The seller has accepted this buyer/household's bid.
	 */
	public void bidAccepted(LandBidRecord<L,?> landBidRecord);
	
	/**
	 * @param landBidRecord
	 */
	public void bidRejected(LandBidRecord<L,?> landBidRecord);

}
