/**
 * 
 */
package sluceii.land.market;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;

/**
 * 
 * @author Shipeng Sun
 *
 */
public class LandMarketTransactionDataDepot 
{
	/**
	 * if to save all transaction records in transactionRecordValues
	 */
	protected boolean isToStoreTransactionRecords;
	
	/**
	 * All transaction records that happened in this market.
	 * It collects data regarding the buyer, the seller, and the land/housing.
	 */
	protected ArrayList<ArrayList<Number>> transactionRecordValues;
	
	/**
	 * The names of the above properties
	 */
	protected ArrayList<String> landListingAttributeNames;
	
	/**
	 * 
	 */
	protected ArrayList<String> landBidAttributeNames;

	/**
	 * 
	 */
	public LandMarketTransactionDataDepot() 
	{
		super();
		isToStoreTransactionRecords = true;
		transactionRecordValues = new ArrayList<ArrayList<Number>>();
	}
	
	/**
	 * 
	 */
	public void clear()
	{
		if(transactionRecordValues!= null) transactionRecordValues.clear();
		if(landListingAttributeNames != null) landListingAttributeNames.clear();
		if(landBidAttributeNames != null) landBidAttributeNames.clear();
	}

	/**
	 * @param pName
	 * @return
	 */
	public boolean addLandListingAttributeName(String pName)
	{
		if(landListingAttributeNames != null)
		{
			return landListingAttributeNames.add(pName);
		}
		else
		{
			landListingAttributeNames = new ArrayList<String>();
			return landListingAttributeNames.add(pName);
		}		
	}
	
	/**
	 * @param pNames
	 * @return
	 */
	public boolean addLandListingAttributeName(Collection<String> pNames)
	{
		if(landListingAttributeNames != null)
		{
			return landListingAttributeNames.addAll(pNames);
		}
		else
		{
			landListingAttributeNames = new ArrayList<String>();
			return landListingAttributeNames.addAll(pNames);
		}		
	}

	/**
	 * @param pName
	 * @return
	 */
	public boolean addBidAttributeName(String pName)
	{
		if(landBidAttributeNames != null)
		{
			return landBidAttributeNames.add(pName);
		}
		else
		{
			landBidAttributeNames = new ArrayList<String>();
			return landBidAttributeNames.add(pName);
		}		
	}
	
	/**
	 * @param pNames
	 * @return
	 */
	public boolean addBidAttributeName(Collection<String> pNames)
	{
		if(landBidAttributeNames != null)
		{
			return landBidAttributeNames.addAll(pNames);
		}
		else
		{
			landBidAttributeNames = new ArrayList<String>();
			return landBidAttributeNames.addAll(pNames);
		}		
	}

	/**
	 * @return
	 */
	public ArrayList<String> getAllLandListingAttributeNames() 
	{
		ArrayList<String> rslt = new ArrayList<String>();
		rslt.add("listingRecordID");
		rslt.add("landUnitTypeCode");
		rslt.add("maxBidsAllowed");
		rslt.add("askPrice");
		rslt.add("willingToAccept");
		rslt.add("listedTime");
		
		if(landListingAttributeNames != null && landListingAttributeNames.size() > 0) rslt.addAll(landListingAttributeNames);
		
		return rslt;
	}	
	
	/**
	 * @return
	 */
	public ArrayList<String> getAllLandBidAttributeNames()
	{
		ArrayList<String> rslt = new ArrayList<String>();
		
		rslt.add("bidTime");
		rslt.add("bidPrice");
		rslt.add("willingToPay");
		rslt.add("buyerUtility");
		
		if(landBidAttributeNames == null || landBidAttributeNames.size() == 0)
		{
			return rslt;
		}		
		
		rslt.addAll(landBidAttributeNames);
		return rslt;
	}
	
	
	/**
	 * @return
	 */
	public ArrayList<String> getAllLandAttributeNames() 
	{
		ArrayList<String> rslt = new ArrayList<String>();
		rslt.addAll(getAllLandListingAttributeNames());
		rslt.addAll(this.getAllLandBidAttributeNames());
		return rslt;
	}	

	/**
	 * Export all transaction records using the given PrintWriter object.
	 * 
	 * @param pw A PrintWriter object
	 */
	public void exportTransactionRecords(PrintWriter pw)
	{
		boolean firstOne = true;
		for(ArrayList<Number> lineVals : transactionRecordValues)
		{
			firstOne = true;
			for(Number val : lineVals)
			{
				// Its value is a integer type
				if(Math.floor(val.doubleValue()) == val.doubleValue())
				{
					if(firstOne) pw.format("%1d", val.intValue());
					else pw.format(",%1d", val.intValue());					
				}
				else
				{
					if(firstOne) pw.format("%f", val.floatValue());
					else pw.format(",%f", val.floatValue());					
				}
				
				firstOne = false;
			}
			pw.println();
		}
		pw.flush();
	}
	
	/**
	 * Export all transaction records using the given PrintWriter object.
	 * 
	 * @param pw A PrintWriter object
	 */
	public void exportTransactionFieldNames(PrintWriter pw, ArrayList<String> sellerInfoFields, ArrayList<String> buyerInfoFields)
	{
		boolean firstOne = true;
		ArrayList<String> allFields = new ArrayList<String>();
		allFields.addAll(sellerInfoFields);
		allFields.addAll(buyerInfoFields);
		allFields.addAll(this.getAllLandAttributeNames());
		
		for(String val : allFields)
		{
			if(firstOne) pw.format("%s", val);
			else pw.format(",%s", val);
			firstOne = false;
		}
		
		pw.println();
	}
}
