/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.landscape;

import java.util.ArrayList;

import repast.simphony.valueLayer.GridValueLayer;
import umn.geog.helia.landscape.CompoundCell;
import umn.geog.helia.landscape.LandscapeLayerGroup;
import umn.geog.helia.landscape.RasterLayer;

/**
 * @author Shipeng Sun
 *
 */
public class Parcel extends LandUnit
{
	/**
	 * @author Shipeng Sun
	 * Possible Status of parcel development
	 */
	public enum ParcelDevelopStatus {undeveloped, developed, built, inbidding};
	
	/**
	 * The market value of this parcel. This could be estimated or actual sale value.
	 * NOTE: this is different from the WTA or WTP.
	 */
	protected float marketValue;
	
	/**
	 * A parcel has multiple cells.
	 * TODO: see if IDs or objects are stored in the object.
	 * An alternative method is the parcel ID layer.
	 */
	ArrayList<Integer> cellIDList;
	
	/**
	 * The current development status of the parcel
	 */
	ParcelDevelopStatus parcelStatus;
	
	/**
	 * 
	 */
	public Parcel()
	{
		super();
		cellIDList = new ArrayList<Integer>();
		parcelStatus = ParcelDevelopStatus.undeveloped;
	}


	/**
	 * TODO: 
	 * @param autoLandUnitID
	 */
	public Parcel(boolean autoLandUnitID) 
	{
		super();
		cellIDList = new ArrayList<Integer>();
		parcelStatus = ParcelDevelopStatus.undeveloped;
		
		if(autoLandUnitID)
		{
			this.setParcelID(LandUnit.maxLandUnitID + 1);
		}
	}


	/**
	 * @return the parcelID
	 */
	public int getParcelID()
	{
		return this.getLandUnitID();
	}


	/**
	 * @param parcelID the parcelID to set
	 */
	public void setParcelID(int parcelID)
	{
		super.setLandUnitID(parcelID);
	}


	/* (non-Javadoc)
	 * @see sluceii.landscape.LandUnit#setLandUnitID(int)
	 */
	@Override
	public void setLandUnitID(int landUnitID) 
	{
		// TODO Auto-generated method stub
		this.setParcelID(landUnitID);
	}


	/**
	 * @return the marketValue
	 */
	public float getMarketValue()
	{
		return marketValue;
	}


	/**
	 * @param marketValue the marketValue to set
	 */
	public void setMarketValue(float marketValue)
	{
		this.marketValue = marketValue;
	}


	/**
	 * @return the cellIDList
	 */
	public ArrayList<Integer> getCellIDList() {
		return cellIDList;
	}


	/**
	 * @param cellIDList the cellIDList to set
	 */
	public void setCellIDList(ArrayList<Integer> cellIDList) {
		this.cellIDList = cellIDList;
	}


	/**
	 * @return the parcelStatus
	 */
	public ParcelDevelopStatus getParcelStatus() 
	{
		return parcelStatus;
	}


	/**
	 * @param parcelStatus the parcelStatus to set
	 */
	public void setParcelStatus(ParcelDevelopStatus parcelStatus) 
	{
		this.parcelStatus = parcelStatus;
	}


	/* (non-Javadoc)
	 * @see sluceii.landscape.LandUnit#getLandUnitTypeCode()
	 */
	@Override
	public int getLandUnitTypeCode() 
	{
		return LandUnitTypeCode.parcel.ordinal();
	}


	/**
	 * Add a cell ID into the cell ID list
	 * @param cellID
	 */
	public boolean addCellID(int cellID)
	{
		if(this.cellIDList == null) return false;
		
		// No duplicate cell IDs
		if(this.cellIDList.contains(cellID))
		{
			return false;
		}
		else
		{
			return this.cellIDList.add(cellID);
		}
	}
	
	/**
	 * @param cellID
	 * @return
	 */
	public boolean removeCellID(int cellID)
	{
		if(cellIDList == null) return false;
		
		// No duplicate cell IDs
		if(cellIDList.contains(cellID))
		{
			cellIDList.remove(cellIDList.indexOf(cellID));
			return true;
		}
		else
		{
			return false;
		}

	}

	/**
	 * Get the value of the parcel from a raster layer using the given aggregation/statistics method.
	 * @param dataLayer
	 * @param aggregateMethod
	 * @return
	 */
	public Number getValue(RasterLayer<?> dataLayer, LandAggregateMethod aggregateMethod)
	{
		if(dataLayer == null)
		{
			return 0.0f;
		}

		int cellID = 0, x = 0, y = 0;
		if(aggregateMethod == LandAggregateMethod.average)
		{
			double val = 0.0;
			for(int i = 0; i < this.cellIDList.size(); i++)
			{
				cellID = this.cellIDList.get(i);
				y = (int)Math.floor(cellID/dataLayer.getCols());
				x = cellID - (y*dataLayer.getCols());
				
				val += dataLayer.getValueAt(x, y).doubleValue();
			}
			
			return val/cellIDList.size();
		}
		else if(aggregateMethod == LandAggregateMethod.max)
		{
			double maxVal = Double.NEGATIVE_INFINITY;
			for(int i = 0; i < this.cellIDList.size(); i++)
			{
				cellID = this.cellIDList.get(i);
				y = (int)Math.floor(cellID/dataLayer.getCols());
				x = cellID - (y*dataLayer.getCols());
				
				maxVal = Math.max(maxVal, dataLayer.getValueAt(x, y).doubleValue());
			}
			
			return maxVal;
		}
		else if(aggregateMethod == LandAggregateMethod.min)
		{
			double minVal = Double.POSITIVE_INFINITY;
			for(int i = 0; i < this.cellIDList.size(); i++)
			{
				cellID = this.cellIDList.get(i);
				y = (int)Math.floor(cellID/dataLayer.getCols());
				x = cellID - (y*dataLayer.getCols());
				
				minVal = Math.min(minVal, dataLayer.getValueAt(x, y).doubleValue());
			}
			
			return minVal;
		}
	
		return 0.0f;
	}
	
	/**
	 * @param layerGroup
	 * @param aggregateMethod
	 * @return
	 */
	public CompoundCell getValue(LandscapeLayerGroup layerGroup, LandAggregateMethod aggregateMethod)
	{
		layerGroup.buildFastLayersArray();
		int totalLayersNum = layerGroup.getTotalLayersNum();
		
		CompoundCell cell = new CompoundCell((short)totalLayersNum);
		
		for(int i = 0; i < totalLayersNum; i++)
		{
			RasterLayer<?> layerObj = (RasterLayer<?>)layerGroup.getLandscapeLayerByIndex(i);
			if(layerObj != null)
			{
				cell.put(i, this.getValue(layerObj, aggregateMethod));
			}
		}
		
        return cell;
	}
	
	/**
	 * @param layer The layer to be updated
	 * @param val The value to be set
	 * @return
	 */
	public boolean putValue(RasterLayer<?> dataLayer, Number val)
	{
		if(dataLayer == null || Float.isNaN(val.floatValue()))
		{
			return false;
		}

		int cellID = 0, x = 0, y = 0;

		for(int i = 0; i < this.cellIDList.size(); i++)
		{
			cellID = this.cellIDList.get(i);
			y = (int)Math.floor(cellID/dataLayer.getCols());
			x = cellID - (y*dataLayer.getCols());
			
			dataLayer.putValueAt(x, y, val);
		}
			
		return true;
	}
	
	/**
	 * Set the val to the GridValueLayer according to the reference layer and this parcel object
	 * @param vlObj
	 * @param refLayer
	 * @param val
	 * @return
	 */
	public boolean putValue(GridValueLayer vlObj, RasterLayer<?> refLayer, Number val)
	{
		if(vlObj == null || refLayer == null || Float.isNaN(val.floatValue()))
		{
			return false;
		}

		int  x = 0, y = 0;

		for(int cellID : cellIDList)
		{
			y = (int)Math.floor(cellID/refLayer.getCols());
			x = cellID - (y*refLayer.getCols());
			
			vlObj.set(val.doubleValue(), x, y);
		}
			
		return true;
	}
	
	/**
	 * Use the value layer to update the GridValueLayer at the locations of the parcel
	 * @param vlObj
	 * @param valLayer
	 * @return
	 */
	public boolean putValue(GridValueLayer vlObj, RasterLayer<?> valLayer)
	{
		if(vlObj == null || valLayer == null)
		{
			return false;
		}

		int cellID = 0, x = 0, y = 0;

		for(int i = 0; i < this.cellIDList.size(); i++)
		{
			cellID = this.cellIDList.get(i);
			y = (int)Math.floor(cellID/valLayer.getCols());
			x = cellID - (y*valLayer.getCols());
			
			vlObj.set(valLayer.getValueAt(x, y).doubleValue(), x, y);
		}
			
		return true;
	}
	
	/**
	 * Alias of putValue method
	 * @param dataLayer
	 * @param val
	 * @return
	 */
	public boolean setValue(RasterLayer<?> dataLayer, Number val)
	{
		return this.putValue(dataLayer, val);
	}


	@Override
	public String toString()
	{
		// TODO Auto-generated method stub
		return "Parcel: " + this.getLandUnitID();
	}


	@Override
	public int getNumOfCells() 
	{
		// TODO Auto-generated method stub
		return this.cellIDList.size();
	}
	
	/**
	 * @param refLayer
	 */
	public void updateCoordinates(RasterLayer<?> refLayer)
	{
		if(refLayer == null || refLayer.getCols() <= 0)
		{
			throw new ClassCastException("Incorrect raster layer format.");
		}
		
		int cellNum, cellID, x, y, xC, yC;
		double dX, dY;

		xC = yC = x = y = 0;
		dX = dY = 0;
		cellNum = cellIDList.size();
		
		for(int j = 0; j < cellNum; j++)
		{
			cellID = cellIDList.get(j);
			y = (int)Math.floor(cellID/refLayer.getCols());
			x = cellID - (y*refLayer.getCols());
			
			xC += x; 
			yC += y;

			dX += refLayer.getCellCenterX(x);
			dY += refLayer.getCellCenterY(y);
		}
		
		setXCenterIndex(xC/cellNum);
		setYCenterIndex(yC/cellNum);
		
		setXCenterPos(dX/cellNum);
		setYCenterPos(dY/cellNum);	
	}


	public boolean putValue(GridValueLayer vlObj, Number val) 
	{
		if(vlObj == null || Float.isNaN(val.floatValue()))
		{
			return false;
		}

		int  x = 0, y = 0; ArrayList<Integer> xys;

		for(int cellID : cellIDList)
		{
			xys = LandCoverDataHelper.cellID2XY(cellID);
			if(xys != null && xys.size() >= 2)
			{
				y = xys.get(1);
				x = xys.get(0);
				
				vlObj.set(val.doubleValue(), x, y);
			}
			else
				return false;
		}
			
		return true;
	}
}
