/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.landscape;

import java.util.Hashtable;

import umn.geog.helia.landscape.CompoundCell;
import umn.geog.helia.landscape.LandscapeLayerGroup;
import umn.geog.helia.landscape.RasterLayer;

/**
 * @author Shipeng Sun
 *
 */
public class Subdivision extends HierarchicalLandUnit<Parcel>
{
	/**
	 * The market value of this subdivision. This could be estimated or actual sale value.
	 * NOTE: this is different from the WTA or WTP.
	 */
	protected float marketValue;	
	
	/**
	 * A hash table/dictionary of all parcel objects contained by all Subdivisions
	 */
	public static Hashtable<Integer, Parcel> parcelObjects = new Hashtable<Integer, Parcel>();
	
	/**
	 * Default constructor
	 */
	public Subdivision()
	{
		super();
	}

	public int getSubdivisionID()
	{
		return this.getLandUnitID();
	}
	
	public void setSubdivisionID(int id)
	{
		this.setLandUnitID(id);
	}

	public void addParcelID(int parcelID)
	{
		// No Duplicate parcel ID in the subdivision
		super.addUnitID(parcelID);
	}

	@Override
	public Number getValue(RasterLayer<?> dataLayer, LandAggregateMethod aggregateMethod)
	{
		return super.getValue(dataLayer, aggregateMethod);
	}

	@Override
	public CompoundCell getValue(LandscapeLayerGroup layerGroup,
			LandAggregateMethod aggregateMethod)
	{
		return super.getValue(layerGroup, aggregateMethod);
	}

	@Override
	public boolean setValue(RasterLayer<?> dataLayer, Number val)
	{
		return super.setValue(dataLayer, val);
	}
	
	/* (non-Javadoc)
	 * @see sluceii.landscape.LandUnit#getLandUnitTypeCode()
	 */
	@Override
	public int getLandUnitTypeCode() 
	{
		return LandUnitTypeCode.subdivision.ordinal();
	}

	@Override
	public Hashtable<Integer, Parcel> getLowerLevelUnits()
	{
		return Subdivision.parcelObjects;
	}

	@Override
	public void setLowerLevelUnits(Hashtable<Integer, Parcel> llUnits)
	{
		if(llUnits != null)
		{
			Subdivision.parcelObjects = llUnits;
		}
	}

	@Override
	public String toString()
	{
		return "Subdivision: " + this.getLandUnitID();
	}

	public void setMarketValue(float emv) 
	{
		this.marketValue = emv;
	}

	@Override
	public int getNumOfCells() 
	{
		Parcel parcelObject;
		int rtnVal = 0;
		for(int pID : lowerLeverlIDList)
		{
			parcelObject = Subdivision.parcelObjects.get(pID);
			if(parcelObject != null)
			{
				rtnVal += parcelObject.getNumOfCells();
			}
			else
			{
				return -1;
			}
		}
		
		return rtnVal;
	}

}
