
package sluceii.landscape;

import umn.geog.helia.landscape.LandscapeLayerGroup;
import umn.geog.helia.landscape.LandscapeLoader;
import umn.geog.helia.landscape.RasterLayer;
import umn.geog.helia.landscape.data.IntegerMatrix;

/**
 * Subdivision templates are a group of raster layers with integer values as sub-parcel ID.
 * This helper provides an interface to load, query, and manage templates.
 *  
 * @author Shipeng Sun
 *
 */
public class SubdivisionTemplateHelper 
{
	/**
	 * The template group object. Shared among all agents.
	 */
	public static LandscapeLayerGroup templateLayerGroup = new LandscapeLayerGroup();

	/**
	 * 
	 */
	public SubdivisionTemplateHelper() 
	{
	}
	
	/**
	 * @param template XML File
	 * @return
	 */
	public static boolean loadTemplatesInfo(String templateXMLFile)
	{
		LandscapeLoader llObj = new LandscapeLoader(templateXMLFile, templateLayerGroup);
		return llObj.loadLandscapeLayers();
	}
	
	/**
	 * Load the template data to the object.
	 * This would load the actual raster data. Make sure load the meta-data before calling this.
	 * 
	 * @return success or failure
	 */
	public static boolean loadTemplateData()
	{
		return LandCoverDataHelper.heliaObj.loadLandscapeGroupData(templateLayerGroup);
	}
	
	@SuppressWarnings("unchecked")
	public static RasterLayer<IntegerMatrix> getTemplate(String templateName)
	{
		if(templateLayerGroup == null) return null; 
		return (RasterLayer<IntegerMatrix>)(templateLayerGroup.getLandscapeLayer(templateName));
	}

	/**
	 * Return a sub-group of template layers. Developers might use a group of templates, instead of a single template,
	 * to subdivide a large parcel/farm.
	 * 
	 * @param templateSubGroupName
	 * @return
	 */
	public static LandscapeLayerGroup getTemplateSubGroup(String templateSubGroupName)
	{
		if(templateLayerGroup == null) return null;
		return templateLayerGroup.getLandscapeLayerSubGroup(templateSubGroupName);
	}
	
	public static void clear() 
	{
		if(templateLayerGroup != null) templateLayerGroup.deepClear();
	}
}
