/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */

package sluceii.params;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.InvalidPropertiesFormatException;
import java.util.Locale;
import java.util.Properties;

import repast.simphony.parameter.Parameters;


/**
 *
 * @author Shipeng Sun
 *
 */
public class AppParameters
{
	// Properties that control which parameters will be written in the log file.
	public static Properties paramOutControl = null;

	public static int msgOutLevel = 2;
	
	public static double randomSeed = 1d;
	public static int runNumber = 1;
	
	public static String dataFolderName = "C:/Users/sunsp/workspace/SLUCEII_LMM/app/aag/";
	public static String landscapeGroupXMLFile = "config/aag.xml";
	public static String templateXMLFile = "config/template.xml";
	public static String logFileName = "Log/runLog.txt";
	public static String resutlsFolderName = "C:/Users/sunsp/workspace/SLUCEII_LMM/app/aag/";
	public static String biomeBGCPropteryTemplateFile = "C:/Users/sunsp/workspace/biomebgc-4.2/biomebgc-4.2/param/param_sluce2.xml";
	public static String paramOutputControlXML = "C:/Users/sunsp/workspace/SLUCEII_AAG/batch/param_out_control.xml";
	public static boolean exportTransactionRecords = false;
	
	/**
	 * this gives the index of a reference layer used to get the meta data of the raster layers.
	 * As all raster layers have the same set of parameters. Any valid index is fine.
	 */
	public static short referenceLayerIndex = 211;
	public static short subdivisionIDLayerIndex = 210;
	public static short parcelIDLayerIndex = 211;
	public static short landUseTypeLayerIndex = 212;
	public static short distanceLayerIndex = 111;
	public static short finalPriceLayerIndex = 213;
	public static short landCoverTypeLayerIndex = 214;
	public static short subdivisionTypeLyaerIndex = 215;
	
	
	/**
	 * ESRI ArcGIS uses -9999 for NaN values
	 */
	public static float notALayerValue = -9999;	
	
	/**
	 * @deprecated
	 */
	public static String agentInitFile = "data/agents.csv";
	
	/**
	 * the number of farms is (2*farmNumParam + 1)^2
	 */
	public static int farmNumParam = 3;
	
	/**
	 * the size of a farm, 12 by 12
	 */
	public static int farmSizeParam = 12;
	
	public static int worldXSize = 61;
	public static int worldYSize = 61;

	// Starting from 0
	public static int worldXCenter = 41;
	public static int worldYCenter = 41;	
	
	public static int numFarmers = 400;
	public static int numDevelopers = 16;
	public static int numHouseholds = 3721;


	/** When residential households resell their properties, they will lower price after a certain period to make sale possible. */
	public static float priceDropRateHH = 0.10f;
	public static int waitPeriodBeforePriceDrop = 2;

	public static double initialDeveloperAsset = 3000.0;
	public static final int numSearchSubdivision = 400;
	
	/**
	 * The maximum number of bids allowed for a house/parcel/property
	 */
	public static int maxBidNums = 5;
	public static int maxBiddingTries = 2;
	public static int maxBidCapacity = 1;// for buyers, how many parcels a buyer can bid on
	
	/**
	 * In theory, the model need maxBidNums + 1 to clear.
	 * If the maxBidNums is infinity, it should be the number of buyers.
	 */
	public static double maxSteps = AppParameters.maxBidNums + 1;

	public static double unitTransportCost = 1.0;
	
	/**
	 * Any probability not greater than this number will sell parcel.
	 */
	public static double sellDecisionUpperBound = 1.0;
	
	/**
	 * Any probability not greater than this number will buy parcel.
	 */
	public static double buyDecisionUpperBound = 1.0;
	
	/**
	 * Even if there is no more transaction, wait a few periods to end the simulation.
	 * Some sellers might lower the price; some households might decide to move.
	 */
	public static int waitPeriodBeforeEnd = 3;
	
	/**
	 * if true, other agents would know one parcel will be developed when there are bids.
	 * Otherwise, no one knows until the transaction is completed
	 */
	public static boolean preInfoAvailable = true;
	public static boolean onlyKeepFinalStep = true;
	
	/**
	 * This parameter has a significant impact.
	 */
	public static boolean excludeCenterInNeighborhood = true;
	
	public static double maxDistanceTo = Math.sqrt(513*513*2);
	public static short neighborhoodSize = 2;
	
	public static double utilityBeta = 0.4;
	public static double sdUtilBeta = 0.2;
	public static double rangeUtilBeta = 0.4;
	
	public static double bInWTP = 0.4;
	
	public static int numSearchParcel = 100;
	public static float agResPrice = 100;
	public static float meanBudget = 2000.0f;
	public static float sdBudget = 800.0f;
	public static float lowBoundBudget = (float)(AppParameters.unitTransportCost*AppParameters.maxDistanceTo);

	public static double minDevelopmentRisk = 0.2;
	public static double maxDevelopmentRisk = 0.5;

	/** */
	public static double residentialLandTransportCostCoefficient = 10.0;
	public static double ruralLandTransportCostCoefficient = 1.0;
	
	/**
	 * @param p
	 */
	public static void fillGUIParameters(Parameters p)
	{
		AppParameters.randomSeed = Integer.valueOf(p.getValueAsString("randomSeed"));
		AppParameters.dataFolderName = p.getValueAsString("dataFolderName");
		AppParameters.landscapeGroupXMLFile = p.getValueAsString("landscapeGroupXMLFile");
		AppParameters.logFileName = p.getValueAsString("logFileName");
		//AppParameters.resutlsFolderName = p.getValueAsString("resutlsFolderName");
		
		AppParameters.farmNumParam = Integer.valueOf(p.getValueAsString("farmNumParam"));
		AppParameters.farmSizeParam = Integer.valueOf(p.getValueAsString("farmSizeParam"));
		
		AppParameters.worldXSize = AppParameters.farmSizeParam*(2*AppParameters.farmNumParam+1);
		AppParameters.worldYSize = AppParameters.farmSizeParam*(2*AppParameters.farmNumParam+1);

		AppParameters.worldXCenter = Integer.valueOf(p.getValueAsString("worldXCenter"));
		AppParameters.worldYCenter = Integer.valueOf(p.getValueAsString("worldYCenter"));
		
		AppParameters.numHouseholds = Integer.valueOf(p.getValueAsString("numHouseholds"));
		AppParameters.numDevelopers = Integer.valueOf(p.getValueAsString("numDevelopers"));
		AppParameters.numFarmers = (2*farmNumParam + 1)*(2*farmNumParam + 1) - 1;
		
		AppParameters.unitTransportCost = Double.valueOf(p.getValueAsString("unitTransportCost"));
		AppParameters.maxDistanceTo = Math.sqrt( (worldYSize+1)*(worldYSize+1) + (worldXSize+1)*(worldXSize+1) + 1)*0.5;
		AppParameters.maxSteps = Integer.valueOf(p.getValueAsString("maxBidNums")) + 1;
		
		AppParameters.neighborhoodSize = Short.valueOf(p.getValueAsString("neighborhoodSize"));
		AppParameters.agResPrice = Float.valueOf(p.getValueAsString("agResPrice"));
		AppParameters.meanBudget = Float.valueOf(p.getValueAsString("meanBudget"));
		AppParameters.sdBudget = Float.valueOf(p.getValueAsString("sdBudget"));
		AppParameters.lowBoundBudget = (float)(AppParameters.unitTransportCost*AppParameters.maxDistanceTo);
		AppParameters.initialDeveloperAsset = Float.valueOf(p.getValueAsString("initialDeveloperAsset"));
		
		AppParameters.numSearchParcel = Integer.valueOf(p.getValueAsString("numSearchParcel"));
		
		AppParameters.preInfoAvailable = Boolean.valueOf(p.getValueAsString("preInfoAvailable"));
		AppParameters.onlyKeepFinalStep = Boolean.valueOf(p.getValueAsString("onlyKeepFinalStep"));
		
		AppParameters.maxBidNums = Integer.valueOf(p.getValueAsString("maxBidNums"));
		
		AppParameters.bInWTP = Double.valueOf(p.getValueAsString("bInWTP"));
		AppParameters.utilityBeta = Double.valueOf(p.getValueAsString("utilityBeta"));
		AppParameters.sdUtilBeta = Double.valueOf(p.getValueAsString("sdUtilBeta"));
		AppParameters.rangeUtilBeta = Double.valueOf(p.getValueAsString("rangeUtilBeta"));
		
		AppParameters.ruralLandTransportCostCoefficient = Double.valueOf(p.getValueAsString("ruralLandTransportCostCoefficient"));
		AppParameters.residentialLandTransportCostCoefficient = Double.valueOf(p.getValueAsString("residentialLandTransportCostCoefficient"));
	}
	
	/**
	 * @param p
	 */
	public static void fillParameters(Parameters p)
	{
		// TODO Auto-generated method stub
		///////////////////////////////////////////
		AppParameters.randomSeed = Integer.valueOf(p.getValueAsString("randomSeed"));
		AppParameters.runNumber = Integer.valueOf(p.getValueAsString("runNumber"));
		AppParameters.msgOutLevel = Integer.valueOf(p.getValueAsString("msgOutLevel"));
		
		// Near constant
		AppParameters.dataFolderName = p.getValueAsString("dataFolderName");
		AppParameters.landscapeGroupXMLFile = p.getValueAsString("landscapeGroupXMLFile");
		AppParameters.logFileName = p.getValueAsString("logFileName");
		AppParameters.resutlsFolderName = p.getValueAsString("resutlsFolderName");
		
		AppParameters.referenceLayerIndex = Short.valueOf(p.getValueAsString("referenceLayerIndex"));
		AppParameters.parcelIDLayerIndex = Short.valueOf(p.getValueAsString("parcelIDLayerIndex"));
		AppParameters.landUseTypeLayerIndex = Short.valueOf(p.getValueAsString("landUseTypeLayerIndex"));
		AppParameters.distanceLayerIndex = Short.valueOf(p.getValueAsString("distanceLayerIndex"));
		AppParameters.finalPriceLayerIndex = Short.valueOf(p.getValueAsString("finalPriceLayerIndex"));
		
		AppParameters.notALayerValue = Float.valueOf(p.getValueAsString("notALayerValue"));
		
		//////////////////////////////////////////
		// Parameters
		AppParameters.farmNumParam = Integer.valueOf(p.getValueAsString("farmNumParam"));
		AppParameters.farmSizeParam = Integer.valueOf(p.getValueAsString("farmSizeParam"));
		
		AppParameters.worldXSize = AppParameters.farmSizeParam*(2*AppParameters.farmNumParam+1);
		AppParameters.worldYSize = AppParameters.farmSizeParam*(2*AppParameters.farmNumParam+1);
		
		AppParameters.maxDistanceTo = Math.sqrt( (worldYSize+1)*(worldYSize+1) + (worldXSize+1)*(worldXSize+1) + 1)*0.5;
		AppParameters.unitTransportCost = Double.valueOf(p.getValueAsString("unitTransportCost"));
		
		AppParameters.numHouseholds = Integer.valueOf(p.getValueAsString("numBuyer"));

		AppParameters.maxBidNums = Integer.valueOf(p.getValueAsString("maxBidNums"));
		AppParameters.maxBiddingTries = Integer.valueOf(p.getValueAsString("maxBiddingTries"));
		
		AppParameters.preInfoAvailable = Boolean.valueOf(p.getValueAsString("preInfoAvailable"));
		AppParameters.onlyKeepFinalStep = Boolean.valueOf(p.getValueAsString("onlyKeepFinalStep"));
		AppParameters.excludeCenterInNeighborhood = Boolean.valueOf(p.getValueAsString("excludeCenterInNeighborhood"));
		
		// The model will end when there is no more transactions.
		AppParameters.maxSteps = Integer.valueOf(p.getValueAsString("maxBidNums")) + 1;
		
		AppParameters.sellDecisionUpperBound = Float.valueOf(p.getValueAsString("sellDecisionUpperBound"));		
		AppParameters.buyDecisionUpperBound = Float.valueOf(p.getValueAsString("buyDecisionUpperBound"));
		AppParameters.utilityBeta = Double.valueOf(p.getValueAsString("utilityBeta"));
		AppParameters.sdUtilBeta = Double.valueOf(p.getValueAsString("sdUtilBeta"));
		AppParameters.lowBoundBudget = (float)(AppParameters.unitTransportCost*AppParameters.maxDistanceTo);
		AppParameters.rangeUtilBeta = Double.valueOf(p.getValueAsString("rangeUtilBeta"));
		
		AppParameters.bInWTP = Double.valueOf(p.getValueAsString("bInWTP"));

		AppParameters.neighborhoodSize = Short.valueOf(p.getValueAsString("neighborhoodSize"));
		AppParameters.agResPrice = Float.valueOf(p.getValueAsString("agResPrice"));
		AppParameters.meanBudget = Float.valueOf(p.getValueAsString("meanBudget"));
		AppParameters.sdBudget = Float.valueOf(p.getValueAsString("sdBudget"));
		AppParameters.numSearchParcel = Integer.valueOf(p.getValueAsString("numSearchParcel"));
	}
	
	/**
	 * worldSize,numBuyer,numSeller,preInfoAvailable,maxBidNums,utilityBeta,sdUtilBeta,rangeUtilBeta,bInWTP,agResPrice,meanBudget,sdBudget,numSearchParcel,stepNum,luFile,tpFile
	 * @param sep
	 * @return
	 */
	/**
	 * worldSize,numBuyer,numSeller,preInfoAvailable,maxBidNums,utilityBeta,sdUtilBeta,rangeUtilBeta,bInWTP,agResPrice,meanBudget,sdBudget,numSearchParcel,stepNum,luFile,tpFile
	 * @param sep
	 * @return
	 */
	public static String paramsValueToString(String sep)
	{
		String rslt = "";
		try
		{
			if(AppParameters.paramOutControl == null || AppParameters.paramOutControl.size() == 0)
			{
				rslt = paramsValueToStringWithoutControl(sep);
			}
			else
			{
				rslt = paramsValueToStringWithControl(sep);
			}
		}
		catch (InvalidPropertiesFormatException e)
		{
			e.printStackTrace();
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		
		return rslt;
	}
	
	public static String paramsValueToStringWithoutControl(String sep)
	{
		StringBuilder sb = new StringBuilder();
		// Send all output to the Appendable object sb
		java.util.Formatter formatter = new java.util.Formatter(sb, Locale.US);

		String rsltStr = AppParameters.worldXSize + sep;
		//rsltStr += AppParameters.worldYSize + sep;
		//rsltStr += AppParameters.numBuyer + sep;
		//rsltStr += AppParameters.numSeller + sep;
		
		//rsltStr += AppParameters.preInfoAvailable + sep;
		//rsltStr += AppParameters.excludeCenterInNeighborhood + sep;
		
		rsltStr += AppParameters.maxBidNums + sep;
		//rsltStr += AppParameters.maxBiddingTries + sep;
		
		//rsltStr += AppParameters.sellDecisionUpperBound + sep;
		//rsltStr += AppParameters.buyDecisionUpperBound + sep;
		
		//Explicit argument indices may be used to re-order output.
		formatter.format("%1$.2f", AppParameters.randomSeed);
		rsltStr += sb.toString() + sep;
		
		rsltStr += AppParameters.runNumber + sep;
		rsltStr += AppParameters.utilityBeta + sep;
		rsltStr += AppParameters.sdUtilBeta + sep;
		//rsltStr += AppParameters.rangeUtilBeta + sep;
		//rsltStr += AppParameters.bInWTP + sep;
		
		//rsltStr += AppParameters.unitTransportCost + sep;
		
		rsltStr += AppParameters.agResPrice + sep;
		rsltStr += AppParameters.meanBudget + sep;
		rsltStr += AppParameters.sdBudget + sep;
		
		rsltStr += AppParameters.neighborhoodSize + sep;
		rsltStr += AppParameters.numSearchParcel;
	
		return rsltStr;
	}
	
	/**
	 * worldSize,numBuyer,numSeller,preInfoAvailable,maxBidNums,utilityBeta,sdUtilBeta,rangeUtilBeta,bInWTP,agResPrice,meanBudget,sdBudget,numSearchParcel,stepNum,luFile,tpFile
	 * @param sep
	 * @return
	 * @throws IOException 
	 * @throws FileNotFoundException 
	 * @throws InvalidPropertiesFormatException 
	 */
	public static String paramsValueToStringWithControl(String sep) throws InvalidPropertiesFormatException, FileNotFoundException, IOException
	{
		if(paramOutControl == null || paramOutControl.size() <= 0)
		{
			paramOutControl = new Properties();
			paramOutControl.loadFromXML(new FileInputStream(new File(paramOutputControlXML)));
		}
		
		StringBuilder sb = new StringBuilder();
		java.util.Formatter formatter = new java.util.Formatter(sb, Locale.US);

		if(Boolean.valueOf(paramOutControl.getProperty("randomSeed")))
		{
			formatter.format("%1$.2f", AppParameters.randomSeed);
			sb.append(sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("runNumber")))
		{
			sb.append(AppParameters.runNumber + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("msgOutLevel"))) 
		{
			sb.append(AppParameters.msgOutLevel + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("dataFolderName"))) 
		{			
			sb.append(AppParameters.dataFolderName + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("resutlsFolderName"))) 
		{
			sb.append(AppParameters.resutlsFolderName + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("landscapeGroupXMLFile"))) 
		{
			sb.append(AppParameters.landscapeGroupXMLFile + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("logFileName"))) 
		{
			sb.append(AppParameters.logFileName + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("referenceLayerIndex"))) 
		{
			sb.append(AppParameters.referenceLayerIndex + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("parcelIDLayerIndex"))) 
		{
			sb.append(AppParameters.parcelIDLayerIndex + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("landUseTypeLayerIndex"))) 
		{
			sb.append(AppParameters.landUseTypeLayerIndex + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("distanceLayerIndex"))) 
		{
			sb.append(AppParameters.distanceLayerIndex + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("finalPriceLayerIndex"))) 
		{
			sb.append(AppParameters.finalPriceLayerIndex + sep);					
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("worldSize"))) 
		{
			sb.append(AppParameters.worldXSize  + sep);
			sb.append(AppParameters.worldYSize + sep);
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("worldXCenter"))) 
		{
			sb.append(AppParameters.worldXCenter  + sep);
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("worldYCenter"))) 
		{
			sb.append(AppParameters.worldYCenter  + sep);
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("numHouseholds"))) 
		{
			sb.append(AppParameters.numHouseholds + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("numFarmers"))) 
		{
			sb.append(AppParameters.numFarmers + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("sellDecisionUpperBound")))
		{
			sb.append(AppParameters.sellDecisionUpperBound + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("buyDecisionUpperBound")))
		{
			sb.append(AppParameters.buyDecisionUpperBound + sep);				
		}
		if(Boolean.valueOf(paramOutControl.getProperty("maxBidding Tries"))) 
		{
			sb.append(AppParameters.maxBiddingTries + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("onlyKeepFinalStep"))) 
		{
			sb.append(AppParameters.onlyKeepFinalStep + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("unitTransportCost"))) 
		{
			sb.append(AppParameters.unitTransportCost + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("neighborhoodSize"))) 
		{
			sb.append(AppParameters.neighborhoodSize + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("preInfoAvailable"))) 
		{
			sb.append(AppParameters.preInfoAvailable + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("excludeCenterInNeighborhood"))) 
		{
			sb.append(AppParameters.excludeCenterInNeighborhood + sep);

		}
		if(Boolean.valueOf(paramOutControl.getProperty("bInWTP"))) 
		{
			sb.append(AppParameters.bInWTP + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("meanBudget"))) 
		{
			sb.append(AppParameters.meanBudget + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("sdBudget"))) 
		{
			sb.append(AppParameters.sdBudget + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("numSearchParcel"))) 
		{
			sb.append(AppParameters.numSearchParcel + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("maxBidNums"))) 
		{
			sb.append(AppParameters.maxBidNums + sep);					
		}
		if(Boolean.valueOf(paramOutControl.getProperty("agResPrice"))) 
		{
			sb.append(AppParameters.agResPrice + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("utilityBeta"))) 
		{
			sb.append(AppParameters.utilityBeta + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("sdUtilBeta"))) 
		{
			sb.append(AppParameters.sdUtilBeta + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("rangeUtilBeta"))) 
		{
			sb.append(AppParameters.rangeUtilBeta + sep);			
		}
		
		int lastCommaIndex = sb.lastIndexOf(",");
		if(lastCommaIndex >= 0)
		{
			sb.deleteCharAt(lastCommaIndex);
		}
		
		return sb.toString();
	}
	
	/**
	 * worldHeight,worldWidth,numBuyer,numSeller,maxBidNums,maxBiddingTries,sellDecisionUpperBound,buyDecisionUpperBound,
	 * utilityBeta,bInWTP,unitTransportCost,agResPrice,meanBudget,sdBudget,neighborhoodSize,numSearchParcel
	 * stepNum,luFile,tpFile
	 * @param sep
	 * @return
	 * @throws IOException 
	 * @throws FileNotFoundException 
	 * @throws InvalidPropertiesFormatException 
	 */
	public static String paramsNameToString(String sep) throws InvalidPropertiesFormatException, FileNotFoundException, IOException
	{
		if(paramOutControl == null || paramOutControl.size() <= 0)
		{
			paramOutControl = new Properties();
			paramOutControl.loadFromXML(new FileInputStream(new File(paramOutputControlXML)));
		}
		
		StringBuilder sb = new StringBuilder();

		if(Boolean.valueOf(paramOutControl.getProperty("randomSeed")))
		{
			sb.append("randomSeed" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("runNumber")))
		{
			sb.append("runNumber" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("msgOutLevel"))) 
		{
			sb.append("msgOutLevel" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("dataFolderName"))) 
		{			
			sb.append("dataFolderName" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("resutlsFolderName"))) 
		{
			sb.append("resutlsFolderName" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("landscapeGroupXMLFile"))) 
		{
			sb.append("landscapeGroupXMLFile" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("logFileName"))) 
		{
			sb.append("logFileName" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("referenceLayerIndex"))) 
		{
			sb.append("referenceLayerIndex" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("parcelIDLayerIndex"))) 
		{
			sb.append("parcelIDLayerIndex" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("landUseTypeLayerIndex"))) 
		{
			sb.append("landUseTypeLayerIndex" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("distanceLayerIndex"))) 
		{
			sb.append("distanceLayerIndex" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("finalPriceLayerIndex"))) 
		{
			sb.append("finalPriceLayerIndex" + sep);					
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("worldSize"))) 
		{
			sb.append("worldXSize " + sep);
			sb.append("worldYSize" + sep);
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("worldXCenter"))) 
		{
			sb.append("worldXCenter " + sep);
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("worldYCenter"))) 
		{
			sb.append("worldYCenter " + sep);
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("numBuyer"))) 
		{
			sb.append("numBuyer" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("numSeller"))) 
		{
			sb.append("numSeller" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("sellDecisionUpperBound")))
		{
			sb.append("sellDecisionUpperBound" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("buyDecisionUpperBound")))
		{
			sb.append("buyDecisionUpperBound" + sep);				
		}
		if(Boolean.valueOf(paramOutControl.getProperty("maxBidding Tries"))) 
		{
			sb.append("maxBiddingTries" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("onlyKeepFinalStep"))) 
		{
			sb.append("onlyKeepFinalStep" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("unitTransportCost"))) 
		{
			sb.append("unitTransportCost" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("neighborhoodSize"))) 
		{
			sb.append("neighborhoodSize" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("preInfoAvailable"))) 
		{
			sb.append("preInfoAvailable" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("excludeCenterInNeighborhood"))) 
		{
			sb.append("excludeCenterInNeighborhood" + sep);

		}
		if(Boolean.valueOf(paramOutControl.getProperty("bInWTP"))) 
		{
			sb.append("bInWTP" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("meanBudget"))) 
		{
			sb.append("meanBudget" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("sdBudget"))) 
		{
			sb.append("sdBudget" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("numSearchParcel"))) 
		{
			sb.append("numSearchParcel" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("maxBidNums"))) 
		{
			sb.append("maxBidNums" + sep);					
		}
		if(Boolean.valueOf(paramOutControl.getProperty("agResPrice"))) 
		{
			sb.append("agResPrice" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("utilityBeta"))) 
		{
			sb.append("utilityBeta" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("sdUtilBeta"))) 
		{
			sb.append("sdUtilBeta" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("rangeUtilBeta"))) 
		{
			sb.append("rangeUtilBeta" + sep);			
		}
		
		int lastCommaIndex = sb.lastIndexOf(",");
		
		if(lastCommaIndex >= 0)
		{
			sb.deleteCharAt(lastCommaIndex);
		}
		
		return sb.toString();
	}
}
