/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.params;

import java.awt.GridLayout;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import javax.swing.JLabel;
import javax.swing.JPanel;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.ui.RSApplication;

/**
 * This assistant class has many functions that could be useful for ContextCreator class.
 * It basically handle message output levels, user panels, host and time information etc.
 * 
 * @author Shipeng Sun
 *
 */
public class DefaultContextCreatorAssist
{
	/**
	 * The time when the model starts
	 */
	java.util.Date modelStartTime;
	
	/**
	 * The time when the model ends
	 */
	java.util.Date modelEndTime;

	/**
	 * The user panel used to display some critical informaiton.
	 * It only works in GUI model, check this before using the control.
	 */
	public static JPanel userPanelControl;	
	
	/**
	 * 
	 */
	public DefaultContextCreatorAssist()
	{
		// TODO Auto-generated constructor stub
	}
	
	/**
	 * @param msg
	 * @param outLevel
	 * @param preFix
	 */
	public void writeStdOutMessage(String msg, int outLevel, String preFix)
	{
		if(outLevel <= AppParameters.msgOutLevel)
		{
			System.out.println(preFix + msg);
		}
	}

	/**
	 * @param msg
	 * @param outLevel
	 * @param addTimeStringPrefix
	 */
	public void writeStdOutMessage(String msg, int outLevel, boolean addTimeStringPrefix)
	{
		if(outLevel <= AppParameters.msgOutLevel)
		{
			if(addTimeStringPrefix)
				System.out.println(this.CurrentDateTimeString(2) + " " + msg);
			else
				System.out.println(msg);
		}
	}	
	/**
	 * @param pStream
	 * @param msg
	 * @param outLevel
	 */
	public void writeStdOutMessage(java.io.PrintStream pStream, String msg, int outLevel)
	{
		if(outLevel <= AppParameters.msgOutLevel)
		{
			pStream.println(msg);
		}
	}
	
	/**
	 * 1 yyyy_MM_dd_HH_mm_ss
	 * 2 yyyy MM dd HH:mm:ss
	 * @param formatType
	 * @return The current date/time in the format of yyyy_MM_dd_HH_mm_ss
	 */
	public String CurrentDateTimeString(int formatType)
	{
		DateFormat dateFormat;
		switch(formatType)
		{
		case 1:
			dateFormat = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss");
			break;
		case 2:
			dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
			break;
		default:
			dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
			break;
		}
	    java.util.Date date = new java.util.Date();
	    String datetime = dateFormat.format(date);
	    return datetime;
	}
	
	/**
	 * @param elapsedTime Time in milliseconds
	 * @return A time string in the format of xx(xxxx...):xx:xx
	 */
	public String getElapsedTimeHoursMinutesSecondsString(long elapsedTime) 
	{     
	    String format = String.format("%%0%dd", 2);
	    elapsedTime = elapsedTime / 1000;
	    String seconds = String.format(format, elapsedTime % 60);
	    String minutes = String.format(format, (elapsedTime % 3600) / 60);
	    String hours = String.format(format, elapsedTime / 3600);
	    String time =  hours + ":" + minutes + ":" + seconds;
	    return time;
	}
	
	public void writeHostStartTimeInfo()
	{
		String hostName = "";
	    try 
	    {
		    hostName = java.net.InetAddress.getLocalHost().getHostName();
	    } 
	    catch (java.net.UnknownHostException e) 
	    {
	        System.err.printf("\n\nError trying to fetch hostname!\n\n");
	        e.printStackTrace();
	    }
	    
	    hostName = "# Hostname: " + hostName;
	    writeStdOutMessage(hostName, 0, "");
	    writeStdOutMessage(CurrentDateTimeString(2), 0, "# Start time: ");
	    this.modelStartTime = new java.util.Date();
	}
	
	public void writeDurationInfo()
	{
		RunEnvironment.getInstance().endRun();
	    writeStdOutMessage(CurrentDateTimeString(2), 0, "# End time: ");
	    java.util.Date modelEndTime = new java.util.Date();
	    
	    long duration = modelEndTime.getTime() - this.modelStartTime.getTime();
	    writeStdOutMessage(getElapsedTimeHoursMinutesSecondsString(duration), 0, "# Duration : ");
	    writeStdOutMessage("******************************************************************************",1,"");
	}
	
	/**
	 * Create a simple text panel to display information.
	 */
	public void createUserPanel()
	{
		if(userPanelControl == null && !(RunEnvironment.getInstance().isBatch()))
		{
			userPanelControl = new JPanel(new GridLayout(0,1));
			JLabel txtLabel = new JLabel("# of transactions:");
			txtLabel.setVerticalAlignment(JLabel.TOP);
			userPanelControl.add(txtLabel);
		}
		
		if(!RunEnvironment.getInstance().isBatch())
		{
			setTotalTransactionNum(0);
			RSApplication.getRSApplicationInstance().addCustomUserPanel(userPanelControl);
		}
	}

	
	public void setTotalTransactionNum(long num)
	{
		// TODO Auto-generated method stub
		if(!RunEnvironment.getInstance().isBatch())
		{
			((JLabel)userPanelControl.getComponent(0)).setText("Number of Transactions:\n" + num);
		}
	}
}
