/*
 * @(#)Log.java
 * 
 * Created on Jul 31, 2006
 * 
 * Dr. Steve Manson's Human-environment Land Integrated Assessment (HELIA)
 * project
 * 
 * @author: Tom Swanson
 * 
 * Version: 0.1
 */
package sluceii.params;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.nio.channels.FileLock;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

/**
 * A simple Log helper facilitating log writing.
 * 
 * Modified by Shipeng Sun
 * @author sunx0170
 * @version 1.1
 * 
 */
public class SimpleLogHelper
{
    /**
     * The name of the file saving log information
     */
    public String logFileName;
    
    /**
     * if debugLevel is less than or equal to zero, nothing is done.
     */
    public byte debugLevel;
    
    /**
     * File stream
     */
    private PrintStream log;
    
    private FileOutputStream outStream;
    
    public boolean threadSafeMode = true;
    
    /**
     * @param logFile
     * @param debugLevel
     * Default Constructor
     */
    public SimpleLogHelper(String logFile, int debugLevel, boolean addTimeStamp)
    {
        try
        {
        	if(addTimeStamp)
        	{
	    		String fileName = logFile;
	    		String pathName = fileName.substring(0,fileName.length()-4);
	    		
	    		String exntName = fileName.substring(fileName.length()-4,fileName.length());  
				DateFormat dateFormat = new SimpleDateFormat("_yyyy_MM_dd_HH_mm_ss");
	            java.util.Date date = new java.util.Date();
	            String dateTime = dateFormat.format(date);
	            
	            logFile = pathName + dateTime + exntName;
        	}

        	// Create the folder if it does not exist.
        	// This must be explicitly called on Linux/Unix
    		int index = Math.max(logFile.lastIndexOf("/"), logFile.lastIndexOf("\\"));
    		String folder = logFile.substring(0,index);
    		File folderObj = new File(folder);
    		if(!folderObj.exists())
    		{
    			folderObj.mkdir();
    		}
        	
        	File f = new File(logFile);
        	if(!f.exists()) 
        	{
        		f.createNewFile();
        	}
        	
            this.logFileName = logFile;
            this.debugLevel = (byte)debugLevel;
            
            outStream = new FileOutputStream(logFile, true);
            log = new PrintStream(outStream);
            //System.setOut(log);
            //System.setErr(log);
        }
        catch (FileNotFoundException e)
        {
            System.err.println("Failed to create log file (" + logFile + "): using stdout/stderr...");
        }
        catch(IOException ioE)
        {
        	ioE.printStackTrace();
        	System.err.println("Failed to create log file (" + logFile + "): using stdout/stderr...");
        }
    }

    /**
     * @param str
     */
    public void writeLog(String str, int debugLvl)
    {
    	writeLogWithoutTime(ToolAndTips.getCurrentDateTime(4) + " >>> " + str, debugLvl);
    }
    
    /**
     * Write a string to the log file without time stamp
     * @param str
     * @param debugLvl
     */
    public void writeLogWithoutTime(String str, int debugLvl)
    {
        if(this.debugLevel <= 0) return;
		FileLock fLock = null;
		
        try
        {
        	if(threadSafeMode)
        	{
        		fLock = null;
        		while(fLock == null)
        		{
        			fLock = outStream.getChannel().lock();
        		}
        		
        	}
	        
	        if(debugLvl <= this.debugLevel)
	        {
	            log.println(str);
	            log.flush();
	        }
	        
        	if(threadSafeMode)
        	{
        		if(fLock != null)
        		{
        			fLock.release();
        		}
        	}
        }
        catch(Exception e)
        {
        	e.printStackTrace();
        	System.err.println("Failed to lock the file ");
        }    	
    }

}