/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.params;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.channels.ReadableByteChannel;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

/**
 *
 * @author sunsp
 *
 */
public class ToolAndTips
{
	/**
	 * Using URI interface to format file name. <br> No matter the Operating Systems,
	 * formatted file names can be interpreted by Java Virtual Machine without problems.
	 * @param fileName file name to be formatted.
	 * @return file name formatted according to the specific platform
	 * @throws MalformedURLException
	 */
	public static String formatFilePath(String fileName) throws MalformedURLException
	{
		File tmpFile = new File(fileName);
		URL urlName = tmpFile.toURI().toURL();
		return urlName.getFile();
	}
	
	/**
	 * Get the String of current date/time
	 * @param type
	 * <pre>
	 * 0 yyyy_MM_dd_HH_mm_ss      e.g., 2000_01_01_12_00_00
	 * 1 yyyy_MM_dd_HH_mm_ss.SSS  e.g., 2000_01_01_12_00_00.000
	 * 2 yyyy_MM_dd_HH_mm_ss_SSS  e.g., 2000_01_01_12_00_00_000
	 * 3 yyyy_MM_dd               e.g., 2000_01_01	
	 * 4 yyyy MM dd HH:mm:ss      e.g., 2000 01 01 12:00:00
	 * </pre>
	 * @return
	 */
	public static String getCurrentDateTime(int type)
	{
		DateFormat dateFormat = null;
		switch(type)
		{
			case 0:
			{
				dateFormat = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss");
				break;
			}
			case 1:
			{
				dateFormat = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss.SSS");
				break;
			}
			case 2:
			{
				dateFormat = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss_SSS");
				break;
			}
			case 3:
			{
				dateFormat = new SimpleDateFormat("yyyy MM dd HH:mm:ss");
				break;
			}
			case 4:
			{
				dateFormat = new SimpleDateFormat("yyyy_MM_dd");
				break;
			}
			default:
			{
				dateFormat = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss_SSS");
				break;
			}
		}

	    java.util.Date date = new java.util.Date();
	    String datetime = dateFormat.format(date);
		return datetime;
	}
	
	public static boolean parseBinaryFile(String fileName, String outputName, int dataType)
	{
		if(fileName == null || fileName == "") return false;

		try 
		{
			File fH = new File(fileName);
			int num = (int)fH.length()/4;
			ByteBuffer dataBuf = ByteBuffer.allocate(num*4);

			dataBuf.order(ByteOrder.LITTLE_ENDIAN);
			
			ReadableByteChannel channel = new FileInputStream(fileName).getChannel(); 
			
			FileOutputStream out = new FileOutputStream(outputName);
			PrintStream dataOut = new PrintStream(out);

			if(channel.read(dataBuf) >0)
			{						
				dataBuf.rewind();
				for(int i = 0; i < num; i++)
				{
					//System.out.println(dataBuf.getFloat());	
					dataOut.println(dataBuf.getFloat());
				}
			}
		} 
		catch (FileNotFoundException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return true;
	}
}
