#########################################################
# This is a program for the visualization of simulation results.
#
#########################################################

##########################################################################
# Function: To Draw Boxes for the next level categories
##########################################################################
plotNextLevelBox <- function(x, y, totWidth, height, dataVector, plotValues = FALSE, cexT = 0.8, minTextWidth = 20) 
{
	vals <- dataVector[,1]
	fieldName <- names(dataVector)[1]
	
	totNum <- length(vals)
	uVals <- unique(vals)
	uNums <- length(uVals)
	
	#print(paste(dim(dataVector),sep=","))
	
	curLeftX <- x
			
	for(i in 1:uNums)
	{
		uVal <- uVals[i]
		uNum <- length(vals[vals==uVal])
		uWidth <- 0

		if(uNums > 1)
		{
			uNum <- length(vals[vals==uVal])
			uWidth <- totWidth*uNum/totNum
			rect(curLeftX,y-height,curLeftX+uWidth,y)
			
			if(plotValues && (uWidth >= minTextWidth))
			{
				text(curLeftX+uWidth/2.0, y - height/2, paste(uVal,"",sep=""),cex=cexT)
			}	
			
			#print(paste(curLeftX,y-height,curLeftX+uWidth,y,sep=","))
		}
		
		if(dim(dataVector)[2] >= 2)
		{
			plotNextLevelBox(curLeftX,y-height,uWidth,height,as.data.frame(dataVector[,2:dim(dataVector)[2]]),plotValues,cexT,minTextWidth)
		}
		
		curLeftX <- curLeftX + uWidth
	}
}


interpretMarketLevel <- function(marketValStr,sepChar = ",")
{
	marketVals <- as.numeric(strsplit(marketValStr,sepChar)[[1]])
	marketStr <- ""
	# Number of Search, Bidding Number Allowed, and Budget Constraint
	if(marketVals[1] == 1)
	{
		marketStr <- paste(marketStr, "No Preference", sep="")
	} else if(marketVals[1] == 3000)
	{
		marketStr <- paste(marketStr, "With Preference", sep="")
	} else
	{
		
	}
	
	if(marketVals[2] == 1)
	{
		marketStr <- paste(marketStr, "No Bidding", sep=", ")
	} else if(marketVals[2] == 3000)
	{
		marketStr <- paste(marketStr, "Competitive Bidding", sep=", ")
	} else
	{
		
	}
	
	if(marketVals[3] == 160 || marketVals[3] == 180)
	{
		marketStr <- paste(marketStr, "Hard Constraint", sep=", ")
	} else if(marketVals[3] == 200)
	{
		marketStr <- paste(marketStr, "No/Soft/Hard Constraint", sep=", ")
	} else
	{
		
	}
	
	marketStr <- paste(marketStr, " (", marketVals[3], ")", sep="")
	
	return(marketStr)
}

# Set the working directory
setwd("C:\\Users\\sunsp\\workspace\\SLUCEII_AAG\\app\\aag\\log")

dataPath <- "C:\\Users\\sunsp\\workspace\\SLUCEII_AAG\\app\\aag\\data\\0824\\"
logName <- 'runLog_AAG_0824_Fixed_Step_2.csv_output1.csv'
#logName <- 'runLog_AAG_0824_Fixed_Step_2.csv_output1.csv'
logData <- read.csv(file = logName, header = TRUE, sep = ",", quote="\"", dec=".")

logData[,"luFile"] <- NULL
logData[,"tpFile"] <- NULL

outRandomFactor <- rep("",dim(logData)[1])

for(i in 1: dim(logData)[1])
{
	outRandomFactor[i] <- paste(logData[i,3:12],sep='',collapse='p')
}

as.factor(outRandomFactor)

require(nlme)

# all variables/measures are averaged across rondom numbers
logDataGrouped <- as.data.frame(gapply(logData,FUN=mean, groups = outRandomFactor))
logDataGrouped <- t(logDataGrouped)
logDataGrouped <- as.data.frame(logDataGrouped)
row.names(logDataGrouped) <- gsub("p","-",row.names(logDataGrouped))

exploredFieldGroup <- c("numSearchParcel", "maxBidNums", "meanBudget")
 
outMarketFactor <- rep("",dim(logDataGrouped)[1])
for(i in 1: dim(logDataGrouped)[1])
{
	outMarketFactor[i] <- paste(logDataGrouped[i,exploredFieldGroup],sep=',',collapse=',')
}

marketLevel <- rep(1,dim(logDataGrouped)[1])
marketLevelVals <- unique(outMarketFactor)

ltys <- c("longdash", "dotted", "solid", "dotdash", "dashed")
lcol <- c("darkgreen", "darkblue", "purple", "darkred")
ptys <- c(19,21,22,23,24)

# these fields are metrics. 
measuredValues <- c("edge_density", "mean_tc", "max_tc", "sum_sw", "sum_luc", "mean_tp", "max_tp")
mVNames        <- c("Edge Density", "Average Transportation Cost", "Maximum Transportation Cost", "Total Social Welfare", "Total Land Developed", "Average Price", "Maximum Price")

# These are parameters.
exploredFields <- c("utilityBeta", "sdUtilBeta", "sdBudget", "neighborhoodSize")
fieldsNames <- c("b","sd(b)","sd(Y)","Neighborhood Size")

require(rggobi)


for(mi in 1:length(marketLevelVals))
{
	marketDataGrouped <- logDataGrouped[outMarketFactor == marketLevelVals[mi],]
	#write.table(marketDataGrouped, paste(marketLevelVals[mi],".csv"), row.names=F, sep=",")

	g <- ggobi(marketDataGrouped)
	
	# this vector is used to sort the exploredFields, the more important parameter goes before less important ones.
	fieldOrderIndex <- c(1,2,3,4)
	fieldOrderList <- list()

	# We can vary the type, color, size
	varyGlyphVector <- c(1,2,3)
	
	
	for(i in 1:3)
	{
		field_g <- marketDataGrouped[,exploredFields[i]]
		#field_g <- field_g[names(sum_luc_g)]
		
		cols <- rep(0,length(field_g))
		
		fieldVals <- unique(field_g)

		for(j in 1:length(fieldVals))
		{
			cols[field_g == fieldVals[j]] = j
		}

		if(i == 1)
		{
			glyph_type(g[1]) <- cols + 1
		} else if(i==2)
		{
			glyph_colour(g[1]) <- cols
		}else if(i==3)
		{
			glyph_size(g[1]) <- cols*3
		}
	}
		
}

