#########################################################
# This is a program for the visualization of simulation results.
#
#########################################################

##########################################################################
# Function: To Draw Boxes for the next level categories
##########################################################################
plotNextLevelBox <- function(x, y, totWidth, height, dataVector, plotValues = FALSE, cexT = 0.8, minTextWidth = 20) 
{
	vals <- dataVector[,1]
	fieldName <- names(dataVector)[1]
	
	totNum <- length(vals)
	uVals <- unique(vals)
	uNums <- length(uVals)
	
	#print(paste(dim(dataVector),sep=","))
	
	curLeftX <- x
			
	for(i in 1:uNums)
	{
		uVal <- uVals[i]
		uNum <- length(vals[vals==uVal])
		uWidth <- 0

		if(uNums > 1)
		{
			uNum <- length(vals[vals==uVal])
			uWidth <- totWidth*uNum/totNum
			rect(curLeftX,y-height,curLeftX+uWidth,y)
			
			if(plotValues && (uWidth >= minTextWidth))
			{
				text(curLeftX+uWidth/2.0, y - height/2, paste(uVal,"",sep=""),cex=cexT)
			}	
			
			#print(paste(curLeftX,y-height,curLeftX+uWidth,y,sep=","))
		}
		
		if(dim(dataVector)[2] >= 2)
		{
			plotNextLevelBox(curLeftX,y-height,uWidth,height,as.data.frame(dataVector[,2:dim(dataVector)[2]]),plotValues,cexT,minTextWidth)
		}
		
		curLeftX <- curLeftX + uWidth
	}
}


# Set the working directory
setwd("C:\\Users\\sunsp\\workspace\\SLUCEII_AAG\\app\\aag\\log")

dataPath <- "C:\\Users\\sunsp\\workspace\\SLUCEII_AAG\\app\\aag\\data\\0824\\"
logName <- 'runLog_AAG_0824_Fixed_Step_2.csv_output1.csv'
#logName <- 'runLog_AAG_0824_Fixed_Step_2.csv_output1.csv'
logData <- read.csv(file = logName, header = TRUE, sep = ",", quote="\"", dec=".")
logData$outRandomFactor <- rep("",dim(logData)[1])

for(i in 1: dim(logData)[1])
{
	logData$outRandomFactor[i] <- paste(logData[i,3:12],sep='',collapse='')
}

as.factor(logData$outRandomFactor)

logData[,"luFile"] <- NULL
logData[,"tpFile"] <- NULL

#tapply(logData[,1:24], logData$outRandomFactor, mean)

ltys <- c("longdash", "dotted", "solid", "dotdash", "dashed")
lcol <- c("darkgreen", "darkblue", "purple", "darkred")

# these fields are metrics. 
measuredValues <- c("edge_density", "mean_tc", "max_tc", "sum_sw", "sum_luc", "mean_tp", "max_tp")
mVNames        <- c("Edge Density", "Average Transportation Cost", "Maximum Transportation Cost", "Total Social Welfare", "Total Land Developed", "Average Price", "Maximum Price")

# These are parameters.
exploredFields <- c("utilityBeta", "sdUtilBeta", "meanBudget", "sdBudget", "neighborhoodSize", "numSearchParcel", "maxBidNums")
fieldsNames <- c("b","sd(b)","avg(Y)","sd(Y)","Neighborhood Size","Preference","Bidding")

for(mIndex in 3:3)
{
	sum_luc_g <- tapply(logData[,measuredValues[mIndex]], logData$outRandomFactor, mean)
	sum_luc_g <- (sum_luc_g-min(sum_luc_g))*400/(max(sum_luc_g)-min(sum_luc_g))

	# this vector is used to sort the exploredFields, the more important parameter goes before less important ones.
	fieldOrderIndex <- c(6,3,4,1,7,5,2)

	fieldOrderList <- list()

	for(i in 1:length(exploredFields))
	{
		# average value grouped by factor
		field_g <- tapply(logData[,exploredFields[i]], logData$outRandomFactor, mean)
		field_g <- field_g[names(sum_luc_g)]
			
		fields_vals <- cbind(field_g)
		fieldIndexVector <- NULL
		
		for(m in 1:length(fieldOrderIndex))
		{
			if(fieldOrderIndex[m] != i) 
			{
				fieldOrderList[[fieldOrderIndex[m]]] <- tapply(logData[,exploredFields[fieldOrderIndex[m]]], logData$outRandomFactor, mean)
				#col.names(fieldOrderList[[fieldOrderIndex[m]]]) <- fieldsNames[exploredFields[fieldOrderIndex[m]]]
				
				fieldOrderList[[fieldOrderIndex[m]]] <- fieldOrderList[[fieldOrderIndex[m]]][names(sum_luc_g)]
				fields_vals <- cbind(fields_vals,fieldOrderList[[fieldOrderIndex[m]]])
				
				fieldIndexVector <- c(fieldIndexVector, fieldOrderIndex[m])
			}
		}
		
		fields_vals <- as.data.frame(fields_vals)
		names(fields_vals) <- c(exploredFields[i],fieldsNames[fieldIndexVector])
		oIndex <- do.call(order,as.data.frame(fields_vals))
		fields_vals <- fields_vals[oIndex,]
		
		o_filed_g <- field_g[oIndex]
		o_sum_luc_g <- sum_luc_g[oIndex]
		fieldVals <- unique(field_g)

		figFileName <- paste(exploredFields[i],measuredValues[mIndex],sep="_")
		
		#png(paste(figFileName,"png",sep='.'), width=800, height=600, bg='white')
		#print(paste(figFileName,"png",sep='.'))
		
		plot(o_sum_luc_g[o_filed_g == fieldVals[1]],type='n',ylim=c(0,540),xlim=c(0,dim(fields_vals)[1]/length(fieldVals)+20),xlab="",ylab=mVNames[mIndex])

		for(j in 1:length(fieldVals))
		{
			lines(o_sum_luc_g[o_filed_g == fieldVals[j]],lty=ltys[j],col=lcol[j])
		}

		# define a recursive function
		plotNextLevelBox(0,540,dim(fields_vals)[1]/length(fieldVals),20,fields_vals[,2:dim(fields_vals)[2]],T,0.6,5)
		legend(dim(fields_vals)[1]/length(fieldVals),540,fieldsNames[fieldIndexVector],horiz = F,cex=0.6,bty='n',yjust=1,x.intersp = 1, y.intersp = 1,adj=c(0,0))
		
		#text(120,80,exploredFields[i],pos=3,cex=0.8)
		legend(dim(fields_vals)[1]/length(fieldVals),180,paste(fieldVals,"",sep=""),lty=ltys[1:length(fieldVals)],title=fieldsNames[i], col=lcol[1:length(fieldVals)],cex=0.8,bty='n')
		readline()
		#dev.off()
	}
	
}
