###################################################################################
# This is a program for the visualization of simulation results.
# Vis_Marketness.R compares metrics between different parameters. 
# The output figure has only one specific market level.
###################################################################################

##########################################################################
# Function: To Draw Boxes for the next level categories
# It is a recursive function.
##########################################################################
plotNextLevelBox <- function(x, y, totWidth, height, dataVector, plotValues = FALSE, cexT = 0.8, minTextWidth = 20) 
{
	vals <- dataVector[,1]
	fieldName <- names(dataVector)[1]
	
	totNum <- length(vals)
	uVals <- unique(vals)
	uNums <- length(uVals)
	
	#print(paste(dim(dataVector),sep=","))
	
	curLeftX <- x
			
	for(i in 1:uNums)
	{
		uVal <- uVals[i]
		uNum <- length(vals[vals==uVal])
		uWidth <- 0

		if(uNums > 1)
		{
			uNum <- length(vals[vals==uVal])
			uWidth <- totWidth*uNum/totNum
			rect(curLeftX,y-height,curLeftX+uWidth,y)
			
			if(plotValues && (uWidth >= minTextWidth))
			{
				text(curLeftX+uWidth/2.0, y - height/2, paste(uVal,"",sep=""),cex=cexT)
			}	
			
			#print(paste(curLeftX,y-height,curLeftX+uWidth,y,sep=","))
		}
		
		if(dim(dataVector)[2] >= 2)
		{
			plotNextLevelBox(curLeftX,y-height,uWidth,height,as.data.frame(dataVector[,2:dim(dataVector)[2]]),plotValues,cexT,minTextWidth)
		}
		
		curLeftX <- curLeftX + uWidth
	}
}


interpretMarketLevel <- function(marketValStr)
{
	marketVals <- as.numeric(strsplit(marketValStr,",")[[1]])
	marketStr <- ""
	# Number of Search, Bidding Number Allowed, and Budget Constraint
	if(marketVals[1] == 1)
	{
		marketStr <- paste(marketStr, "No Preference", sep="")
	} else if(marketVals[1] == 3000)
	{
		marketStr <- paste(marketStr, "With Preference", sep="")
	} else
	{
		
	}
	
	if(marketVals[2] == 1)
	{
		marketStr <- paste(marketStr, "No Bidding", sep=", ")
	} else if(marketVals[2] == 3000)
	{
		marketStr <- paste(marketStr, "Competitive Bidding", sep=", ")
	} else
	{
		
	}
	
	if(marketVals[3] == 160 || marketVals[3] == 180)
	{
		marketStr <- paste(marketStr, "Hard Constraint", sep=", ")
	} else if(marketVals[3] == 200)
	{
		marketStr <- paste(marketStr, "No/Soft/Hard Constraint", sep=", ")
	} else
	{
		
	}
	
	marketStr <- paste(marketStr, " (budget = ", marketVals[3], ")", sep="")
	
	return(marketStr)
}


interpretMarketVals <- function(marketValStr)
{
	return(as.numeric(strsplit(marketValStr,",")[[1]]))
}

interpretMarketLevelNumber <- function(marketValStr)
{
	marketVals <- as.numeric(strsplit(marketValStr,",")[[1]])
	marketStr <- ""
	# Number of Search, Bidding Number Allowed, and Budget Constraint
	if(marketVals[1] == 1)
	{
		marketStr <- "-1"
	} else if(marketVals[1] == 3000)
	{
		
		if(marketVals[2] == 1)
		{
			marketStr <- "1"
		} else if(marketVals[2] == 3000)
		{
			marketStr <- "2"
		} else
		{
			
		}
	} else
	{
		
	}
	
	return(marketStr)
}


# Set the working directory
setwd("C:\\Users\\sunsp\\workspace\\SLUCEII_AAG\\app\\aag\\log")

dataPath <- "C:\\Users\\sunsp\\workspace\\SLUCEII_AAG\\app\\aag\\data\\0824\\"
logName <- 'runLog_AAG_0824_Fixed_Step_2.csv_output1.csv'
#logName <- 'runLog_AAG_0824_Fixed_Step_2.csv_output1.csv'
logData <- read.csv(file = logName, header = TRUE, sep = ",", quote="\"", dec=".")

logData[,"luFile"] <- NULL
logData[,"tpFile"] <- NULL

outRandomFactor <- rep("",dim(logData)[1])

for(i in 1: dim(logData)[1])
{
	outRandomFactor[i] <- paste(logData[i,3:12],sep='',collapse='')
}

as.factor(outRandomFactor)

require(nlme)

# all variables/measures are averaged across rondom numbers
logDataGrouped <- as.data.frame(gapply(logData,FUN=mean, groups = outRandomFactor))
logDataGrouped <- t(logDataGrouped)
logDataGrouped <- as.data.frame(logDataGrouped)

# kick out the data without preference
logDataGrouped <- logDataGrouped[logDataGrouped[,"numSearchParcel"] == 3000,]

exploredFieldGroup <- c("numSearchParcel", "maxBidNums", "meanBudget")
 
outMarketFactor <- rep("",dim(logDataGrouped)[1])
for(i in 1: dim(logDataGrouped)[1])
{
	outMarketFactor[i] <- paste(logDataGrouped[i,exploredFieldGroup],sep=',',collapse=',')
}

marketLevel <- rep(1,dim(logDataGrouped)[1])
marketLevelVals <- unique(outMarketFactor)

ltys <- c("longdash", "dotted", "solid", "dotdash", "dashed")
lcol <- c("darkgreen", "darkblue", "purple", "darkred")
ptys <- c(19,21,22,23,24)

# these fields are metrics. 
measuredValues <- c("edge_density", "mean_tc", "max_tc", "sum_sw", "sum_luc", "mean_tp", "max_tp")
mVNames        <- c("Edge Density", "Average Transportation Cost", "Maximum Transportation Cost", "Total Social Welfare", "Total Land Developed", "Average Price", "Maximum Price")

# These are parameters.
exploredFields <- c("utilityBeta", "sdUtilBeta", "sdBudget", "neighborhoodSize")
fieldsNames <- c("b","sd(b)","sd(Y)","Neighborhood Size")

for(mi in 1:length(marketLevelVals))
{
	marketDataGrouped <- logDataGrouped[outMarketFactor == marketLevelVals[mi],]
	write.table(marketDataGrouped, paste(marketLevelVals[mi],".csv"), row.names=F, sep=",")

	#for(mIndex in 1:length(measuredValues))
	for(mIndex in c(1,2,3,4,5,6))
	{
		sum_luc_g <- marketDataGrouped[,measuredValues[mIndex]]
		sum_luc_g <- (sum_luc_g-min(sum_luc_g))*400/(max(sum_luc_g)-min(sum_luc_g))

		# this vector is used to sort the exploredFields, the more important parameter goes before less important ones.
		fieldOrderIndex <- c(1,2,3,4)
		fieldOrderList <- list()

		for(i in 1:4)
		{
			# average value grouped by factor
			field_g <- marketDataGrouped[,exploredFields[i]]
			#field_g <- field_g[names(sum_luc_g)]
				
			fields_vals <- cbind(field_g)
			fieldIndexVector <- NULL
			
			for(m in 1:length(fieldOrderIndex))
			{
				if(fieldOrderIndex[m] != i) 
				{
					fieldOrderList[[fieldOrderIndex[m]]] <- marketDataGrouped[,exploredFields[fieldOrderIndex[m]]]
					#col.names(fieldOrderList[[fieldOrderIndex[m]]]) <- fieldsNames[exploredFields[fieldOrderIndex[m]]]
					
					#fieldOrderList[[fieldOrderIndex[m]]] <- fieldOrderList[[fieldOrderIndex[m]]][names(sum_luc_g)]
					fields_vals <- cbind(fields_vals,fieldOrderList[[fieldOrderIndex[m]]])
					fieldIndexVector <- c(fieldIndexVector, fieldOrderIndex[m])
				}
			}
			
			fields_vals <- as.data.frame(fields_vals)
			names(fields_vals) <- c(exploredFields[i],fieldsNames[fieldIndexVector])
			oIndex <- do.call(order,as.data.frame(fields_vals))
			fields_vals <- fields_vals[oIndex,]
			
			o_filed_g <- field_g[oIndex]
			o_sum_luc_g <- sum_luc_g[oIndex]
			fieldVals <- unique(field_g)

			figFileName <- paste("Level", interpretMarketLevelNumber(marketLevelVals[mi]), interpretMarketVals(marketLevelVals[mi])[3],exploredFields[i],measuredValues[mIndex],sep="_")
			
			png(paste(figFileName,"png",sep='.'), width=1024, height=640, bg='white')
			print(paste(figFileName,"png",sep='.'))
			titleStr <- paste("Market Level ", interpretMarketLevelNumber(marketLevelVals[mi]),sep="")
			plot(o_sum_luc_g[o_filed_g == fieldVals[1]],type='n',ylim=c(-50,540),xlim=c(0,(dim(fields_vals)[1]/length(fieldVals))*1.2),xaxt='n',yaxt='n',xlab="",ylab=mVNames[mIndex],main=titleStr,sep="\t")
			axis(side=1,labels=F)
			
			xVals = seq(1,dim(fields_vals)[1]/length(fieldVals),by=1)
			abline(v=xVals,col=gray(0.94))
			
			cur_field <- marketDataGrouped[,measuredValues[mIndex]]
			labelVals <- pretty(min(cur_field):max(cur_field),n=5)
			labelPoss <- (labelVals - min(cur_field))*400/(max(cur_field)-min(cur_field))
			axis(side=2,at=labelPoss[labelPoss>=0 & labelPoss <=400],labels=paste(labelVals[labelPoss>=0 & labelPoss <=400],sep=""))
				
			for(j in 1:length(fieldVals))
			{
				lines(o_sum_luc_g[o_filed_g == fieldVals[j]],lty=ltys[j],pch=ptys[j],col=lcol[j],type='o')
			}
			
			text(mean(xVals),-45,paste("Market :",interpretMarketLevel(marketLevelVals[mi])),cex=1.0)

			# define a recursive function
			plotNextLevelBox(0.5,540,dim(fields_vals)[1]/length(fieldVals),20,fields_vals[,2:dim(fields_vals)[2]],T,cexT=0.99,1)
			legend(0.5+dim(fields_vals)[1]/length(fieldVals),540,fieldsNames[fieldIndexVector],horiz = F,cex=1.1,bty='n',yjust=1,x.intersp = 1, y.intersp = 1,adj=c(0,0))
			
			#text(120,80,exploredFields[i],pos=3,cex=0.8)
			legend(0.5+dim(fields_vals)[1]/length(fieldVals),380,paste(fieldVals,"",sep=""),lty=ltys[1:length(fieldVals)],pch=ptys[1:length(fieldVals)],title=fieldsNames[i], col=lcol[1:length(fieldVals)],cex=1.1,bty='n')
			#legend(dim(fields_vals)[1]/length(fieldVals),380,paste(marketLevelVals,"",sep=""),lty=ltys[1:length(fieldVals)],title=fieldsNames[i], col=lcol[1:length(fieldVals)],cex=0.8,bty='n')
			dev.off()
			
			#readline()
		}
		
	}
}

