rm(list=ls(all=TRUE))

#############################################################################################
## Redefine the read.asciigrid function to handle the possible missing of NODATA_VALUE.
## That line is optional.
##  Modifed from read.asciigrid method in sp package
##  Author: Shipeng Sun
##  Date: March 26, 2010
##################################################################################################
sluce.read.asciigrid <- function (fname, as.image = FALSE, plot.image = FALSE, colname = fname, proj4string = CRS(as.character(NA)),nodata = -9999) 
{
    t = file(fname, "r")
    l5 = readLines(t, n = 6)
    l5s = strsplit(l5, "\\s+", perl = T)
	if (length(grep("nodata_value", tolower(l5s[[6]][1]))) == 0) {
		lineNum = 5
		close(t)
		t = file(fname, "r")
		l5 = readLines(t, n = lineNum)
		l5s = strsplit(l5, "\\s+", perl = T)
		nodata.value = nodata
	} else
	{
		lineNum = 6
	}
	
    xllcenter = yllcenter = xllcorner = yllcorner = as.numeric(NA)
    for (i in 1:lineNum) {
        fieldname = casefold(l5s[[i]][1])
        if (length(grep("ncols", fieldname))) 
            ncols = as.numeric(l5s[[i]][2])
        if (length(grep("nrows", fieldname))) 
            nrows = as.numeric(l5s[[i]][2])
        if (length(grep("xllcorner", fieldname))) 
            xllcorner = as.numeric(l5s[[i]][2])
        if (length(grep("yllcorner", fieldname))) 
            yllcorner = as.numeric(l5s[[i]][2])
        if (length(grep("xllcenter", fieldname))) 
            xllcenter = as.numeric(l5s[[i]][2])
        if (length(grep("yllcenter", fieldname))) 
            yllcenter = as.numeric(l5s[[i]][2])
        if (length(grep("cellsize", fieldname))) 
            cellsize = as.numeric(l5s[[i]][2])
        if (length(grep("nodata_value", fieldname)))
            nodata.value = as.numeric(l5s[[i]][2])
    }
	
    if (is.na(xllcorner) && !is.na(xllcenter)) {
        xllcorner = xllcenter - 0.5 * cellsize
	} else 
	{
		xllcenter = xllcorner + 0.5 * cellsize
	}
    if (is.na(yllcorner) && !is.na(yllcenter)) {
        yllcorner = yllcenter - 0.5 * cellsize
	} else
	{
		yllcenter = yllcorner + 0.5 * cellsize
	}
	
    map = scan(t, as.numeric(0), quiet = TRUE)
    close(t)
    if (length(as.vector(map)) != nrows * ncols) 
        stop("dimensions of map do not match that of header")
    map[map == nodata.value] = NA
	map[map == nodata] = NA
	
    if (as.image) {
        img = matrix(map, ncols, nrows)[, nrows:1]
        img = list(z = img, x = xllcorner + cellsize * ((1:ncols) - 
            0.5), y = yllcorner + cellsize * ((1:nrows) - 0.5))
        if (plot.image) {
            image(img, asp = 1)
            return(invisible(img))
        }
        else return(img)
    }
    df = data.frame(map)
    names(df) = colname
    grid = GridTopology(c(xllcenter, yllcenter), rep(cellsize, 
        2), c(ncols, nrows))
    SpatialGridDataFrame(grid, data = df, proj4string = proj4string)
}

#######################################################################################
# 
#######################################################################################
sluce.rainbow <- function (n, s = 1, v = 1, start = 0, end = max(1, n - 1)/n, gamma = 1, alpha = 1) 
{
    if ((n <- as.integer(n[1L])) > 0) {
        if (start == end || any(c(start, end) < 0) || any(c(start, end) > 1)) 
            stop("'start' and 'end' must be distinct and in [0, 1].")
			
        #hsv(h = seq.int(start, ifelse(start > end, 1, 0) + end, length.out = n)%%1, s, v, gamma, alpha)
		hsv(h = seq.int(start, end, length.out = n), s, v, gamma, alpha)
		
    } else {
		character(0L)
	}
}

#######################################################################################
# 
#######################################################################################
image.legend <- function(x,y, zlim, at.z = NULL, col = heat.colors(12), legnd=NULL,lwd = max(3,32/length(col)), bg = NA, bty = "", ...)
  ## * kein y.i -- Benutzer soll rein ueber lwd steuern; sollte reichen.
  ## * legnd koennte interessant sein, falls Text geschrieben werden soll
  ## (weiss mal wieder nicht, wie man aus legnd legend als option
  ## macht)
  ## * lwd wird per default in Abh. von col gewaehlt.
{
    ## Purpose:
    ## Authors: Martin Maechler, 9 Jul 2001
    ## Martin Schlather, 24 Jul 2001

  if (!is.null(legnd) && is.null(at.z))
      stop("at.z must be given if legnd is") ## falls legnd darf at.z
    ## nicht automatisch gewaehlt werden

    if(!is.numeric(zlim) || zlim[1] > zlim[2])
        stop("`zlim' must be numeric; zlim[1] <= zlim[2]")
    if(is.null(at.z)) {
        ## hier ein Versuch in Abhaengigkeit von n
        ## die Anzahl der labels zu bestimmen:
        n <- min(7, max(1,length(col)/10))
        at.z <- pretty(zlim,n=n,min.n=max(n %/% 3,1))

        ## es sieht nicht schoen aus, wenn pretty die letzte oder
        ## erste zahl weit ausserhalb des zlim legt.
        ## heuristisch nur 25% (oder so) ueberschreitung bzgl
        ## intervalllaenge zulassen:
        tol <- diff(at.z)[1] / 4
        at.z <- at.z[(at.z>=zlim[1]-tol) & (at.z<=zlim[2]+tol)]
      }
    if(!is.numeric(at.z) || is.unsorted(at.z))
        stop("`at.z' must be numeric non-decreasing")
    n.at <- length(at.z)
    nc <- length(col)
    if(n.at >= nc)
        stop("length(at.z) must be (much) smaller than length(col)")
    dz <- diff(zlim)
    ## The colors must run equidistantly from zlim[1] to zlim[2];
    ## col[i] is for z-interval zlim[1] + [i-1, i) * dz/nc ; i = 1:nc
    ## i.e., an at.z[] value z0 is color i0 = floor(nc * (z0 - zlim[1])/dz)
    at.i <- floor(nc * (at.z - zlim[1])/dz )
    ## Possibly extend colors by `background' to the left and right
    bgC <- if(is.null(bg)) NA else bg
    if((xtra.l <- 1 - at.i[1]) > 0) {
        at.i <- at.i + xtra.l
        col <- c(rep(bgC, xtra.l), col)
    }
    if((xtra.r <- at.i[n.at] - nc) > 0)
        col <- c(col, rep(bgC, xtra.r))
    lgd <- character(length(col))

    ## folgende if-Anweisung ist neu:
    if (is.null(legnd)) lgd[at.i] <-format(at.z, dig = 3)
    else {
      if (length(legnd)!=length(at.z))
        stop("at.z and legnd must have the same length")
      lgd[at.i] <- legnd
    }
    if((V <- R.version)$major <= 1 && V$minor <= 3.0 && V$status == "")
{
        ## stop-gap fix around the bug that "NA" is not a valid color:
        if(is.na(bgC)) {
            lgd <- lgd[!is.na(col)]
            col <- col[!is.na(col)]
        }
    }
    legend(x,y, legend = rev(lgd), col = rev(col),y.i = lwd/16, bty = bty, lwd = lwd, bg = bg, ...)
} 


# Set the working directory
setwd("C:\\Users\\sunsp\\workspace\\SLUCEII20100204\\app\\aag\\log")
#setwd("G:\\Realtor_L2")
#setwd("E:\\SUN_BACKUP\\Projects\\GIScientists_Network\\R")
#setwd("D:\\Courses\\Working_Papers\\SNA-GIS\\R")
#setwd("D:\\Courses\\Parcel\\Modeling\\Clustering\\Realtor_L2")
dataPath <- "C:\\Users\\sunsp\\workspace\\SLUCEII20100204\\app\\aag\\data\\"
logName <- 'runLog_L0_QNTY_04_16.csv'#'runLog_L12_04_16.csv'#'RunLog_04_15.csv' #'runLog_L0_QNTY_04_13.csv'
logData <- read.csv(file = logName, header = TRUE, sep = ",", quote="\"", dec=".")

# quantity, max/price, min price, average price
analysisRslt <- matrix(0,nrow=nrow(logData),ncol=4)
colnames(analysisRslt) <- c("cnt","minp",'maxp','totp')

library(sp)
n <- dim(logData)[1]
totalSteps <- 0
fileType <- "png"

tmp <- "tmp\\"
tmpPathName <- paste(getwd(),"\\",tmp,sep="")

batchStart = 1
batchEnd = 1

stepNum = 1

staticImageFileNames <- matrix(nrow=n,ncol=2)

for(i in 1:n)
{
	fname <- logData[i,"luFile"]
	fname <- as.vector(fname)
	subStrs <- strsplit(fname,"_")
	if(subStrs[[1]][1] == "lu")
	{
		#stepNum <- strsplit(subStrs[[1]][9],"\\.")[[1]][1]
		stepNum <- subStrs[[1]][9]

		stepNum <- subStrs[[1]][9]
		totalTickNum <- strsplit(subStrs[[1]][10],"\\.")[[1]][1]
	}

	if((stepNum == 1 && totalSteps > 0))
	{
		#system...
		totalSteps <- 0
		# Use the ImageMajick command to create a gif animation
		batchEnd = i
		
		# Put the files names
		cmdLine <- paste("convert -delay 50 -loop 0 -path ", getwd(), " ", sep="")
		cmdLine <- paste(cmdLine,tmpPathName,"*.",fileType,sep="")
		# for(fIndex in batchStart:batchEnd)
		# {
			# cmdLine <- paste(cmdLine,staticImageFileNames[fIndex,k],sep=" ")
		# }
		
		#cmdLine <- paste(cmdLine,staticImageFileNames[batchStart,j],sep =" ")
		
		#cmdLine <- paste(cmdLine,".gif",sep="")
		
		for(j in 1:2)
		{
			gifName <- gsub(fileType,"gif",staticImageFileNames[batchStart,j])
			
			if(j == 1) {
				print(paste('"c:/Program Files/ImageMagick-6.6.0-Q16/convert.exe"', '-delay 50 -loop 0 tmp\\lu*.png', gifName))
				system(paste('"c:/Program Files/ImageMagick-6.6.0-Q16/convert.exe"', '-delay 50 -loop 0 tmp\\lu*.png', gifName), wait = FALSE)
			} else if(j == 2)
			{
				print(paste('"c:/Program Files/ImageMagick-6.6.0-Q16/convert.exe"', '-delay 50 -loop 0 tmp\\tp*.png', gifName))
				system(paste('"c:/Program Files/ImageMagick-6.6.0-Q16/convert.exe"', '-delay 50 -loop 0 tmp\\tp*.png', gifName), wait = FALSE)
			}
					
			Sys.sleep(10)
		}
		
		curWD <- getwd()
		setwd(paste(curWD,tmp,sep='/'))
		cmdStr <- "ren *.png *.old"
		#cmdStr <- "del tmp\\*.png"
		shell(cmdStr)
		setwd(curWD)
		
		Sys.sleep(10)

		#print(cmdLine)
		
		batchStart = i
	}
	
		
	for(j in 1:2)
	{
		if(j == 1) {
			fname <- logData[i,"luFile"]
			fFname <- paste(dataPath,fname,sep="")
			img <- sluce.read.asciigrid(fFname)
			
			analysisRslt[i,1] <- sum(img@data,na.rm=T)
			
		} else if(j == 2)
		{
			fname <- logData[i,"tpFile"]
			fFname <- paste(dataPath,fname,sep="")
			img <- sluce.read.asciigrid(fFname,nodata=0)
			
			analysisRslt[i,2] <- min(img@data,na.rm=T)
			analysisRslt[i,3] <- max(img@data,na.rm=T)
			analysisRslt[i,4] <- sum(img@data,na.rm=T)			
		}
	
		fname <- as.vector(fname)
		subStrs <- strsplit(fname,"_")
		
		if(subStrs[[1]][1] == "lu")
		{
			stepNum <- subStrs[[1]][9]
			totalTickNum <- subStrs[[1]][10]
			#stepNum <- strsplit(subStrs[[1]][9],"\\.")[[1]][1]
			fExt <- strsplit(fname,"\\.")
			staticFileName <- paste(fExt[[1]][1],fileType,sep=".")
			staticFileName <- paste(tmp,staticFileName,sep="")
			
		} else if (subStrs[[1]][1] == "tprice")
		{
			stepNum <- subStrs[[1]][8]
			totalTickNum <- totalTickNum <- subStrs[[1]][9]
			#stepNum <- strsplit(subStrs[[1]][8],"\\.")[[1]][1]
			fExt <- strsplit(fname,"\\.")
			staticFileName <- paste(fExt[[1]][1],fileType,sep=".")
			staticFileName <- paste(tmp,staticFileName,sep="")
		}

		print(staticFileName)
		
		if(fileType == "jpg") {
			jpeg(filename = staticFileName, width = 4, height = 4, units = "in", pointsize = 12, quality = 75, bg = "white",res = 600, restoreConsole = TRUE)
		} else if (fileType == "png")
		{
			png(filename = staticFileName, width = 4, height = 4, units = "in", pointsize = 12, bg = "white",res = 600, restoreConsole = TRUE)		
		}
		
		if(j == 1)
		{	
			cols <- c(rgb(77, 175, 74, maxColorValue =255), rgb(152, 78, 163, maxColorValue =255))
			image(img,col=cols,add=F)
		}
		else {
			#terrain.colors(32), heat.colors(32)
			cols <- sluce.rainbow(64, start=0.3, end=0.0)

			par(mai=c(0.1,0.1,0.1,0.1))
			plot(x=c(1,1),xlim=c(0,35),y=c(0,35),type="n",xlab="",ylab="",axes=F,frame.plot=F)
			image(img,col=cols,add=T,xlim=c(0,31),ylim=c(0,31),frame.plot=T)
			rect(0,0,31,31,border=rgb(0.8,0.8,0.8),lwd=0.5)
			image.legend(30.8,25,range(as.numeric(img@data[,]),na.rm=T,finite=T),col=cols,bty="n",cex=0.60)
		}

		dev.off()
		
		staticImageFileNames[i,j] <- staticFileName
	}

	print(paste(stepNum,totalSteps,sep=","))
	
	totalSteps <- totalSteps + 1
	
	if(i == n)
	{
		batchEnd = i
		
		# Put the files names
		for(j in 1:2)
		{
			gifName <- gsub(fileType,"gif",staticImageFileNames[batchStart,j])
			
			if(j == 1) {
				print(paste('"c:/Program Files/ImageMagick-6.6.0-Q16/convert.exe"', '-delay 50 -loop 0 tmp\\lu*.png', gifName))
				system(paste('"c:/Program Files/ImageMagick-6.6.0-Q16/convert.exe"', '-delay 50 -loop 0 tmp\\lu*.png', gifName), wait = FALSE)
			} else if(j == 2)
			{
				print(paste('"c:/Program Files/ImageMagick-6.6.0-Q16/convert.exe"', '-delay 50 -loop 0 tmp\\tp*.png', gifName))
				system(paste('"c:/Program Files/ImageMagick-6.6.0-Q16/convert.exe"', '-delay 50 -loop 0 tmp\\tp*.png', gifName), wait = FALSE)
			}
					
			Sys.sleep(10)
		}
		
		curWD <- getwd()
		setwd(paste(curWD,tmp,sep='/'))
		cmdStr <- "ren *.png *.old"
		#cmdStr <- "del tmp\\*.png"
		shell(cmdStr)
		setwd(curWD)
		
		Sys.sleep(10)
	
		batchStart = i
	}	
}


analysisFileName <- paste(logName, format(Sys.time(), "%Y_%m_%d_%H_%M_%S"),sep="_")
write.table(analysisRslt,file=paste(analysisFileName,"csv",sep="."),quote=F,sep=',',row.names=F)

