/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.agent;

import java.util.ArrayList;

import repast.simphony.engine.schedule.ScheduledMethod;
import sluceii.agent.decisionmaking.LandOwnerDecisionToSellHelper;
import sluceii.app.aag.AAGAppParameters;
import sluceii.land.exchange.LandBuyingProcess;
import sluceii.land.exchange.LandSellingProcess;
import sluceii.landmarket.LandBidRecord;
import sluceii.landmarket.LandMarketListing;
import sluceii.landmarket.LandBidRecord.BidRecordStatus;
import sluceii.landmarket.LandListingRecord.ListingRecordStatus;
import sluceii.landscape.Parcel;

/**
 * @author Shipeng Sun
 *
 */
public class RuralLandOwner extends SimpleAgent implements LandSellingProcess
{
	/**
	 * A Static reference to the land market listing
	 */
	public static LandMarketListing landMarket;
	
	/**
	 * 
	 */
	private float pressureToSell;
	
	/**
	 * Default constructor
	 */
	public RuralLandOwner()
	{
		// Initialize the property list
		super();
		pressureToSell = 1.0f;
	}

	/**
	 * @return the pressureToSell
	 */
	public float getPressureToSell()
	{
		return pressureToSell;
	}

	/**
	 * @param pressureToSell the pressureToSell to set
	 */
	public void setPressureToSell(float pressureToSell)
	{
		this.pressureToSell = pressureToSell;
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.SimpleAgent#step()
	 * 
	 * The logic for land owner
	 * 1. Decision to put land on sale
	 * 2. Put land on the market with WTA
	 * 3. Wait buyers to bid
	 * 4. Check and possibly accept offer from the bidder
	 * 5. Give up the land/house
	 * 6. Update resources
	 */
	@Override
	@ScheduledMethod(start = 1, interval = 1, shuffle = true, priority = 10.0d)
	public void step()
	{
		try
		{
			// TODO Auto-generated method stub
			super.step();
			
			LandOwnerDecisionToSellHelper decisionSell = new LandOwnerDecisionToSellHelper();
			decisionSell.setOwner(this);
			
			Parcel curParcelObj = null;
			float askPrice, wta;
			
			for(int i = 0; i < this.propertyList.size(); i++)
			{
				curParcelObj = (Parcel)this.propertyList.get(i);
				
				if(decisionSell.isToSell(curParcelObj))
				{
					askPrice = (float)decisionSell.calcAskPrice(curParcelObj);
					wta = (float)decisionSell.calcWTAPrice(curParcelObj);
					this.putParcelOnMarket(curParcelObj,askPrice, wta);
				}
			}
			
			//this.checkBidOffers();
		}
		catch(Exception e)
		{
			System.err.println("# Error in RuralLandOwner:step - " + e.toString());
			e.printStackTrace(System.err);
		}
	}

	/**
	 * @param landCover
	 */
	public void putParcelOnMarket(Parcel parcelObj, float askPrice, float wta)
	{
		// TODO Auto-generated method stub
		landMarket.addParcelOnMarket(parcelObj, this, askPrice, wta, AAGAppParameters.maxBidNums);
		//System.out.println(parcelObj.getXCenterIndex() + "," + parcelObj.getYCenterIndex());
		this.pressureToSell = 0.0f;
	}
	
	/**
	 * Check and possibly accept offer from the bidder
	 * 
	 * TECH NOTE:
	 * It is necessary to wrap up a ScheduledMethod with a general Exception.
	 * The invoker will not handle and analyze the runtime exception but throw an InvocationTargetException.
	 * So, it is the best practice to intercept the exception here.
	 */
	@ScheduledMethod(start = 1, interval = 1, shuffle = true, priority = 6.0d)
	public void checkBidOffers()
	{
		try
		{
			Parcel curParcelObj = null;
			LandBidRecord bidRecObj = null;
			
			float maxBidPrice = -10000.0f;
			int maxBidIndex = -1;
			float wta = 0;
			ArrayList<LandBidRecord> bidRecordList = null;
			
			for(int i = 0; i < this.propertyList.size(); i++)
			{
				curParcelObj = (Parcel)this.propertyList.get(i);
				bidRecordList = landMarket.getBidRecordsByParcelID(curParcelObj.getParcelID());
				
				if(bidRecordList == null) continue;
				
				for(int j = 0; j < bidRecordList.size(); j++)
				{
					bidRecObj = bidRecordList.get(j);
					wta = bidRecObj.getWillingToAccept();
					
					if(bidRecObj.getParcelID() == curParcelObj.getParcelID())
					{
						// Check out the highest bid price
						if(bidRecObj.getRecordStatus() == BidRecordStatus.open && bidRecObj.getBidPrice() > maxBidPrice)
						{
							maxBidPrice = bidRecObj.getBidPrice();
							maxBidIndex = j;
						}
					}
				}
				
				// if the maximum bid price is not less than the willingness to accept price, deal
				if(maxBidPrice >= wta)
				{
					this.acceptBidOffer(bidRecordList.get(maxBidIndex));
					
					// Change the status of all other records
					for(int j = 0; j < bidRecordList.size(); j++)
					{
						bidRecObj = bidRecordList.get(j);
						
						if(j == maxBidIndex)
						{
							bidRecObj.setRecordStatus(BidRecordStatus.accepted);
						}
						else
						{
							bidRecObj.setRecordStatus(BidRecordStatus.rejected);
						}
					}
	
				}
				else
				{
					// There are multiple paths for this.
					// Lower the WTA or withdraw the listing
					// Change the status of all other records
					for(int j = 0; j < bidRecordList.size(); j++)
					{
						bidRecObj = bidRecordList.get(j);
						bidRecObj.setRecordStatus(BidRecordStatus.rejected);
					}
					
				}
			}		
		}
		catch(Exception e)
		{
			System.err.println("# Error in RuralLandOwner: checkBidOffers - " + e.toString());
			e.printStackTrace(System.err);
		}
	}

	/**
	 * Remove the parcel from the listing and this agent have to give up the ownership.
	 * The land owner might keep all the bidding records for future price formation.
	 * This owner needs to notify s/he has accepted the offer from one buyer.
	 * @param landBidRecord
	 */
	public void acceptBidOffer(LandBidRecord landBidRecord)
	{
		LandBuyingProcess buyerInterface = (LandBuyingProcess)landBidRecord.getBuyer();
		if(buyerInterface == null) return;
		buyerInterface.bidAccepted(landBidRecord);
	}
	
	/* (non-Javadoc)
	 * @see sluceii.land.exchange.LandSellingProcess#rejectBidOffer(sluceii.landmarket.LandBidRecord)
	 */
	public void rejectBidOffer(LandBidRecord landBidRecord)
	{
		LandBuyingProcess buyerInterface = (LandBuyingProcess)landBidRecord.getBuyer();
		if(buyerInterface != null)
			buyerInterface.bidRejected(landBidRecord);
	}
	
	/**
	 * The buyer has accepted the transaction.
	 * update the market listing and property list 
	 */
	public void afterBidAction(LandBidRecord landBidRecord)
	{
		// TODO: Remove the record or change the status of the record?
		// Remove this parcel from the market listing
		landMarket.removeListing(landBidRecord.getListingRecord());
		landBidRecord.getListingRecord().setRecordStaus(ListingRecordStatus.done);
		landBidRecord.setRecordStatus(BidRecordStatus.completed);
		
		// Remove this parcel from the owner's property list
		this.propertyList.remove(landBidRecord.getParcelObj());
		
		// Gain some cash
		this.captialResource.addAsset(landBidRecord.getBidPrice());
	}

}
