/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.agent.decisionmaking;

import java.util.ArrayList;

import repast.simphony.random.RandomHelper;
import sluceii.agent.Household;
import sluceii.agent.utility.ConsumptionBundle;
import sluceii.app.aag.AAGAppParameters;
import sluceii.app.aag.AAGMiscFunctions;
import sluceii.landscape.LandCover;
import sluceii.landscape.LandCoverDataHelper;
import sluceii.landscape.Parcel;
import sluceii.landscape.LandCover.LandAggregateMethod;
import umn.geog.helia.landscape.RasterLayer;

/**
 * @author Shipeng Sun
 *
 */
public class HouseholdDecisionToBuyHelper implements DecisionToBuy
{
	/**
	 * the potential land buyer
	 */
	protected Household landBuyer;
	
	
	public HouseholdDecisionToBuyHelper()
	{
		super();
		// TODO Auto-generated constructor stub
	}

	/**
	 * @return the landBuyer
	 */
	public Household getLandBuyer()
	{
		return landBuyer;
	}

	/**
	 * @param landBuyer the landBuyer to set
	 */
	public void setLandBuyer(Household landBuyer)
	{
		this.landBuyer = landBuyer;
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToBuy#CalcMoveProbability()
	 */
	@Override
	public double calcBuyProbability()
	{
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToBuy#ToMove()
	 */
	@Override
	public boolean isToBuy()
	{
		// Twenty percent
		return (this.landBuyer.getPressureToBuy() == 1.0f && RandomHelper.nextDouble() <= AAGAppParameters.buyDecisionUpperBound);
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToBuy#calcBidPrice(sluceii.landscape.LandCover)
	 */
	@Override
	public double calcBidPrice(LandCover landObj)
	{
		// TODO Auto-generated method stub
		if(landObj instanceof Parcel)
		{
			RasterLayer<?> distLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AAGAppParameters.distanceLayerIndex);
			
			double dist = landObj.getValue(distLayer, LandAggregateMethod.average).doubleValue();
			double transportCost = dist * AAGAppParameters.unitTransportCost;
			
			double Y = landBuyer.getCaptialResource().getAssetQuantity()-transportCost;
			double U = this.calcUtility(landObj);
			double b = AAGAppParameters.bInWTP;
			
			return Y/(1+(b*b)/(U*U));
			
			//return Math.min(landBuyer.getCaptialResource().getAssetQuantity(), ((Parcel)landObj).getMarketValue());
		}
		else
		{
			return 0;
		}
	}

	/* (non-Javadoc)
	 * @see sluceii.agent.decisionmaking.DecisionToBuy#calcWTPPrice(sluceii.landscape.LandCover)
	 */
	@Override
	public double calcWTPPrice(LandCover landObj)
	{
		// TODO Auto-generated method stub
		if(landObj instanceof Parcel)
		{
			return this.calcBidPrice(landObj);
			//return Math.min(landBuyer.getCaptialResource().getAssetQuantity(), ((Parcel)landObj).getMarketValue());
		}
		else
		{
			return 0;
		}
	}

	@Override
	public double calcUtility(LandCover landObj)
	{
		ConsumptionBundle cBundle = AAGMiscFunctions.getConsumptionBundle(landObj, this.landBuyer);
		return this.landBuyer.getUtilStrategy().getUtility(cBundle);
	}
	
	public ArrayList<Number> calcLandProperties(LandCover landObj)
	{
		// TODO Auto-generated method stub
		if(landObj instanceof Parcel)
		{
			ConsumptionBundle cb = AAGMiscFunctions.getConsumptionBundle(landObj, this.landBuyer);
			
			RasterLayer<?> distLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AAGAppParameters.distanceLayerIndex);
			
			double dist = landObj.getValue(distLayer, LandAggregateMethod.average).doubleValue();
			double transportCost = dist * AAGAppParameters.unitTransportCost;
			
			double Y = landBuyer.getCaptialResource().getAssetQuantity()-transportCost;
			double U = this.calcUtility(landObj);
			double b = AAGAppParameters.bInWTP;
			
			double bidPrice = Y/(1+(b*b)/(U*U));
			
			ArrayList<Number> rslt = new ArrayList<Number>();
			// dist, open space amenity, tranCost, Y, b, U, wtp, bidPrice
			rslt.add(dist); // distance
			rslt.add(cb.getObservedValue(1)); // open space amenity
			rslt.add(transportCost); // transport cost
			rslt.add(Y);
			rslt.add(b);
			rslt.add(U);
			rslt.add(bidPrice);
			rslt.add(bidPrice);
			
			return rslt;
		}
		else
		{
			return null;
		}
	}

}
