/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.agent.decisionmaking;

import java.util.ArrayList;
import sluceii.landmarket.LandListingRecord;

/**
 * This interface defines how buyers search the records in the market listing.
 * @author Shipeng Sun
 *
 */
public interface HousingSearch
{
	/**
	 * This method defines how a buyer search the list of available properties.
	 * In principle, the searching process should have at least two inter-related processes.
	 * 
	 * The first one is to find a sub-sample as a result of incomplete information. This can be
	 * done in two ways. One is to define a destination neighborhood. All parcels/properties outside
	 * this neighborhood will be excluded. The other one is define a probability surface. Different
	 * parcels/properties have various probabilities being chose by the buyer. The most common formula
	 * for such probability is the negative exponential probability. 
	 * 
	 * The second one is to assign a utility to each property in the whole list or the sub-sample list.
	 * Several methods have been defined in the utility packages. 
	 * @see sluceii.agent.utility
	 * 
	 * @param num the number of properties/land parcels/houses returned
	 * @return A list of land listing records
	 */
	public ArrayList<LandListingRecord> searchProperties(int num);
}
