/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.agent.initialization;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;

import repast.simphony.context.Context;
import repast.simphony.space.grid.Grid;
import sluceii.agent.Household;
import sluceii.agent.RuralLandOwner;
import sluceii.agent.SimpleAgent;
import sluceii.landscape.LandCoverDataHelper;
import sluceii.landscape.Parcel;
import sluceii.params.Constants;

/**
 * 
 * This class mimics the repast.simphony.space.gis.ShapefileLoader<T>.
 * It creates agents from a csv file.
 * 
 * The current version has no intelligent "sensor" for field names. It uses fixed
 * file structure.
 * 
 * TODO: Make it more flexible by calling get methods in the class.
 * 
 * @author Shipeng Sun
 *
 */
public class CSVfileLoader
{
	/**
	 * The csv file name
	 */
	private String filePathName;
	
	/**
	 * 
	 */
	Context<SimpleAgent> refContext;
	
	/**
	 * 
	 */
	public CSVfileLoader(String fileName, Context<SimpleAgent> context)
	{
		// TODO Auto-generated constructor stub
		this.filePathName = fileName;
		this.refContext = context;
	}
	
	/**
	 * @return the filePathName
	 */
	public String getFilePathName()
	{
		return filePathName;
	}

	/**
	 * @param filePathName the filePathName to set
	 */
	public void setFilePathName(String filePathName)
	{
		this.filePathName = filePathName;
	}

	/**
	 * Suppose we know all agents are actually SimpleAgent, either RuralLandOwner or Household
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public boolean load()
	{
		Grid<SimpleAgent> agentGrid = (Grid<SimpleAgent>) this.refContext.getProjection(Constants.AGENT2D);
		
		// Agent ID
		int agentID = 1;
		
		try
		{
		    FileInputStream fstream = new FileInputStream(filePathName);
		    // Get the object of DataInputStream
		    DataInputStream in = new DataInputStream(fstream);
		    
		    BufferedReader bufRdr = new BufferedReader(new InputStreamReader(in));
			
			java.lang.String line = null;
			
			// Bypass the header line, the structure of the head line is
			// PIN,EMV_TOTAL,X,Y,Asset,AgentType
			line = bufRdr.readLine();
			String[] values = null;
			
			int pNum;
			float emv, x, y, ast;
			int type;
			
			//read each line of text file
			while((line = bufRdr.readLine()) != null)
			{
				values = line.split(",");
				
				if(values.length == 6)
				{
					pNum = Integer.valueOf(values[0]);
					emv = Float.valueOf(values[1]);
					x   = Float.valueOf(values[2]);
					y   = Float.valueOf(values[3]);
					ast = Float.valueOf(values[4]);
					type = Integer.valueOf(values[5]);
					
					Parcel pObj = new Parcel();
					pObj.setParcelID(pNum);
					pObj.setMarketValue(emv);
					
					if(!LandCoverDataHelper.addToParcelList(pObj))
					{
						bufRdr.close();
						return false;
					}
					
					if(type == 0)
					{
						// Rural parcels
						// Context
						RuralLandOwner rloObj = new RuralLandOwner();
						rloObj.setAgentID(agentID++);
						
						rloObj.setPosition(x, y);
						rloObj.getCaptialResource().setAssetQuantity(ast);
						
						rloObj.addProperty(pObj);
						refContext.add(rloObj);
						if(!agentGrid.moveTo(rloObj,rloObj.getX(),rloObj.getY()))
						{
							System.out.println("Error: Failed to place the agent on Grid");
						}
						
					}
					else if (type == 1)
					{
						// urban parcels
						Household hhObj = new Household();
						hhObj.setAgentID(agentID++);
						
						hhObj.setPosition(x, y);
						hhObj.getCaptialResource().setAssetQuantity(ast);
						
						hhObj.addProperty(pObj);
						
						refContext.add(hhObj);
						agentGrid.moveTo(hhObj,hhObj.getX(),hhObj.getY());
					}
					else
					{
						
					}
				}
				else
				{
					
				}
			}
		
			//close the file
			bufRdr.close();

		}
		catch (FileNotFoundException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		catch(IOException ioE)
		{
			ioE.printStackTrace();
			return false;
		}
		
		return true;
	}

}
