/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.agent.initialization;

import cern.jet.random.Normal;
import repast.simphony.context.Context;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.grid.Grid;
import sluceii.agent.Household;
import sluceii.agent.RuralLandOwner;
import sluceii.agent.SimpleAgent;
import sluceii.app.aag.AAGAppParameters;
import sluceii.landscape.LandCoverDataHelper;
import sluceii.landscape.Parcel;
import sluceii.params.Constants;

/**
 * @author sunsp
 *
 */
public class RandomInitializer
{
	/**
	 * 
	 */
	Context<SimpleAgent> refContext;
	
	protected int numBuyer;
	
	protected int numSeller;
	
	protected int cellCenterX;
	protected int cellCenterY;
	
	/**
	 * 
	 */
	public RandomInitializer(Context<SimpleAgent> contextObj)
	{
		this.refContext = contextObj;
	}
	
	/**
	 * @return the numBuyer
	 */
	public int getNumBuyer()
	{
		return numBuyer;
	}

	/**
	 * @param numBuyer the numBuyer to set
	 */
	public void setNumBuyer(int numBuyer)
	{
		this.numBuyer = numBuyer;
	}

	/**
	 * @return the numSeller
	 */
	public int getNumSeller()
	{
		return numSeller;
	}

	/**
	 * @param numSeller the numSeller to set
	 */
	public void setNumSeller(int numSeller)
	{
		this.numSeller = numSeller;
	}

	/**
	 * @return the cellCenterX
	 */
	public int getCellCenterX()
	{
		return cellCenterX;
	}

	/**
	 * @param cellCenterX the cellCenterX to set
	 */
	public void setCellCenterX(int cellCenterX)
	{
		this.cellCenterX = cellCenterX;
	}

	/**
	 * @return the cellCenterY
	 */
	public int getCellCenterY()
	{
		return cellCenterY;
	}

	/**
	 * @param cellCenterY the cellCenterY to set
	 */
	public void setCellCenterY(int cellCenterY)
	{
		this.cellCenterY = cellCenterY;
	}

	/**
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public boolean init()
	{
		Grid<SimpleAgent> agentGrid = (Grid<SimpleAgent>) this.refContext.getProjection(Constants.AGENT2D);
		
		// Agent ID
		int agentID = 1;
		int range = (int)Math.ceil(Math.sqrt(numSeller));

		int pNum;
		float emv, ast; // estimated market value, asset
		double pfp = AAGAppParameters.utilityBeta; // preference for proximity
		int x, y;
		
		int numAgents = 0;
		
		int rangeSize = (range/2);
		
		Normal budgetDist = RandomHelper.createNormal(AAGAppParameters.meanBudget, AAGAppParameters.sdBudget);
		Normal preferDist = RandomHelper.createNormal(AAGAppParameters.utilityBeta, AAGAppParameters.sdUtilBeta);
		
		int centerX = cellCenterX;
		int centerY = cellCenterY;
		
		int minX = 0, maxX = AAGAppParameters.worldXSize - 1, minY = 0, maxY = AAGAppParameters.worldYSize - 1; // the range of seller agents
		
		if(centerX - rangeSize < 0)
		{
			minX = 0;
			maxX = Math.min(range-1 , AAGAppParameters.worldXSize - 1);
		}
		else if(centerX+rangeSize >= AAGAppParameters.worldXSize)
		{
			maxX = AAGAppParameters.worldXSize - 1;
			minX = Math.max(0, AAGAppParameters.worldXSize - 2 - range);
		}
		else
		{
			minX = centerX - rangeSize;
			maxX = centerX + rangeSize;
		}
		
		if(centerY - rangeSize < 0)
		{
			minY = 0;
			maxY = Math.min(range-1, AAGAppParameters.worldYSize - 1);
		}
		else if(centerY + rangeSize >= AAGAppParameters.worldYSize)
		{
			maxY = AAGAppParameters.worldYSize - 1;
			minY = Math.max(0, AAGAppParameters.worldYSize - 2 - range);
		}
		else
		{
			minY = centerY - rangeSize;
			maxY = centerY + rangeSize;
		}
			
		for(y = minY; y <= maxY; y++)
		{
			for(x = minX; x <= maxX; x++)
			{
				pNum = y * agentGrid.getDimensions().getWidth() + x + 1;
				
				emv = AAGAppParameters.agResPrice;
				ast = 0;
				
				Parcel pObj = new Parcel();
				pObj.setParcelID(pNum);
				pObj.setMarketValue(emv);
				
				if(!LandCoverDataHelper.addToParcelList(pObj))
				{
					return false;
				}
					
				// Rural parcels
				// Context
				RuralLandOwner rloObj = new RuralLandOwner();
				rloObj.setAgentID(agentID++);
				
				rloObj.setPosition(x, y);
				rloObj.getCaptialResource().setAssetQuantity(ast);
				
				rloObj.addProperty(pObj);
				
				/*
				// Sellers do not need this parameter.
				do
				{
					pfp = (float)preferDist.nextDouble();
					
				} while(pfp < (AAGAppParameters.utilityBeta - AAGAppParameters.rangeUtilBeta/2.0)||pfp > (AAGAppParameters.utilityBeta + AAGAppParameters.rangeUtilBeta/2.0));
				
				rloObj.setPreferenceForProximity(pfp);
				*/
				
				refContext.add(rloObj);
				if(!agentGrid.moveTo(rloObj,rloObj.getX(),rloObj.getY()))
				{
					System.out.println("Error: Failed to place the agent on Grid");
				}
				
				numAgents++;
				
				if(numAgents == this.numSeller) break;
			}
			
			if(numAgents == this.numSeller) break;
		}
		
		centerX = cellCenterX;
		centerY = cellCenterY;
		
		if(centerX < 0) centerX = 0;
		if(centerX >= AAGAppParameters.worldXSize) centerX = AAGAppParameters.worldXSize - 1;
		
		if(centerY < 0) centerY = 0;
		if(centerY >= AAGAppParameters.worldYSize) centerY = AAGAppParameters.worldYSize - 1;
		
		
		// set up the buyers. They have no properties.
		for(int i = 0; i < this.numBuyer; i++)
		{
			emv = 0;
			x   = centerX;
			y   = centerY;

			do
			{
				ast = (float)budgetDist.nextDouble();
			} while(ast <= AAGAppParameters.lowBoundBudget);
			
			// urban parcels
			Household hhObj = new Household();
			hhObj.setAgentID(agentID++);
			
			hhObj.setPosition(x, y);
			hhObj.getCaptialResource().setAssetQuantity(ast);
			
			// Keep trying until a random number falls into the range.
			do
			{
				pfp = (float)preferDist.nextDouble();
				
			} while(pfp < 0.0 || pfp > 1.0 || pfp < (AAGAppParameters.utilityBeta - AAGAppParameters.rangeUtilBeta/2.0)||pfp > (AAGAppParameters.utilityBeta + AAGAppParameters.rangeUtilBeta/2.0));
			
			hhObj.setPreferenceForProximity(pfp);			
			
			refContext.add(hhObj);

			if(!agentGrid.moveTo(hhObj,hhObj.getX(),hhObj.getY()))
			{
				System.out.println("Error: Failed to place the agent on Grid");
				return false;
			}
		}

	
		return true;
	}
}
