package sluceii.agent.utility;

import java.util.ArrayList;

/**
 * @author Derek T. Robinson
 * @version 1.0
 **/

public class ConsumptionBundle
{
	/**
	 * An arraylist of observed values found in each grid cell.
	 * 
	 * @uml.property name="observedValue"
	 * @uml.associationEnd multiplicity="(0 -1)" elementType="java.lang.Double"
	 */
	ArrayList<Double> observedValue;

	/**
	 * @uml.property name="preferenceWeight"
	 * @uml.associationEnd multiplicity="(0 -1)" elementType="java.lang.Double"
	 */
	ArrayList<Double> preferenceWeight;

	/**
	 * @uml.property name="preferredValue"
	 * @uml.associationEnd multiplicity="(0 -1)" elementType="java.lang.Double"
	 */
	ArrayList<Double> preferredValue;

	/**
	 * @uml.property name="rho"
	 */
	double rho;

	public ConsumptionBundle()
	{
		observedValue = new ArrayList<Double>();
		preferenceWeight = new ArrayList<Double>();
		preferredValue = new ArrayList<Double>();
		rho = 0.0;
	}

	/**
	 * This method adds a value to the list of observed variables.
	 **/
	public void addValueToObservedList(double value)
	{
		observedValue.add(value);
	}

	public void addValueToPreferenceWeightList(double value)
	{
		preferenceWeight.add(value);
	}

	public void addValueToPreferredValueList(double value)
	{
		preferredValue.add(value);
	}

	/**
	 * @param value
	 * @uml.property name="rho"
	 */
	public void setRho(double value)
	{
		rho = value;
	}

	public double getObservedValue(int location)
	{
		return observedValue.get(location);
	}

	public double getPreferenceWeight(int location)
	{
		return preferenceWeight.get(location);
	}

	public double getPreferredValue(int location)
	{
		return preferredValue.get(location);
	}

	/**
	 * Rho variable.
	 * 
	 * @return double
	 * @uml.property name="rho"
	 */
	public double getRho()
	{
		return rho;
	}

	public int getSize()
	{
		return observedValue.size();
	}
}
