/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.app.aag;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.InvalidPropertiesFormatException;
import java.util.Locale;
import java.util.Properties;

import repast.simphony.parameter.Parameters;

/**
 * @author Shipeng Sun
 *
 */
public final class AAGAppParameters
{
	public static String programVersion = "SLUCE2 LMM V 2.0";

	// Properties that control which parameters will be written in the log file.
	public static Properties paramOutControl = null;
	
	public static int msgOutLevel = 0;
	
	public static double randomSeed = 1d;
	public static int runNumber = 1;
	
	public static String dataFolderName = "C:/Users/sunsp/workspace/SLUCEII_AAG/app/aag/";
	//public static String landscapeGroupXMLFile = "C:/Users/sunsp/workspace/sluce_data/config/carver_mn.xml";
	public static String landscapeGroupXMLFile = "config/aag.xml";
	public static String logFileName = "Log/runLog.txt";
	public static String resutlsFolderName = "C:/Users/sunsp/workspace/SLUCEII_AAG/app/aag/";
	public static String paramOutputControlXML = "config/param_out_control.xml";
	public static boolean exportTransactionRecords = false;
	/**
	 * this gives the index of a reference layer used to get the meta data of the raster layers.
	 * As all raster layers have the same set of parameters. Any valid index is fine.
	 */
	public static short referenceLayerIndex = 211;
	public static short parcelIDLayerIndex = 211;
	public static short landUseTypeLayerIndex = 212;
	public static short distanceLayerIndex = 111;
	public static short finalPriceLayerIndex = 213;
	
	/**
	 * ESRI ArcGIS uses -9999 for NaN values
	 */
	public static float notALayerValue = -9999;	
	
	/**
	 * @deprecated
	 */
	public static String agentInitFile = "data/agents.csv";
	
	public static int worldXSize = 61;
	public static int worldYSize = 61;
	
	public static int worldXCenter = 31;
	public static int worldYCenter = 31;
	
	public static int numBuyer = 400;
	public static int numSeller = 3721;
	
	/**
	 * The maximum number of bids allowed for a house/parcel/property
	 */
	public static int maxBidNums = 5;
	public static int maxBiddingTries = 2;
	public static int maxBidCapacity = 1;// for buyers, how many parcels a buyer can bid on
	
	/**
	 * In theory, the model need maxBidNums + 1 to clear.
	 * If the maxBidNums is infinity, it should be the number of buyers.
	 */
	public static double maxSteps = AAGAppParameters.maxBidNums + 1;

	public static double unitTransportCost = 1.0;
	
	/**
	 * Any probability not greater than this number will sell parcel.
	 */
	public static double sellDecisionUpperBound = 1.0;
	
	/**
	 * Any probability not greater than this number will buy parcel.
	 */
	public static double buyDecisionUpperBound = 1.0;
	
	/**
	 * if true, other agents would know one parcel will be developed when there are bids.
	 * Otherwise, no one knows until the transaction is completed
	 */
	public static boolean preInfoAvailable = true;
	public static boolean onlyKeepFinalStep = true;
	
	/**
	 * This parameter has a significant impact.
	 */
	public static boolean excludeCenterInNeighborhood = false;
	
	public static double maxDistanceTo = Math.sqrt(513*513*2);
	public static short neighborhoodSize = 2;
	
	public static double utilityBeta = 0.4;
	public static double sdUtilBeta = 0.2;
	public static double rangeUtilBeta = 0.4;
	
	public static double bInWTP = 0.4;
	
	public static int numSearchParcel = 100;
	public static float agResPrice = 100;
	public static float meanBudget = 220.0f;
	public static float sdBudget = 100.0f;
	public static float lowBoundBudget = (float)(AAGAppParameters.unitTransportCost*AAGAppParameters.maxDistanceTo);

	public static int colorSetting = 1;
	
	/**
	 * @param p
	 */
	public static void fillGUIParameters(Parameters p)
	{
		
		AAGAppParameters.dataFolderName = p.getValueAsString("dataFolderName");
		AAGAppParameters.landscapeGroupXMLFile = p.getValueAsString("landscapeGroupXMLFile");
		AAGAppParameters.logFileName = p.getValueAsString("logFileName");
		//AAGAppParameters.resutlsFolderName = p.getValueAsString("resutlsFolderName");
		AAGAppParameters.paramOutputControlXML = p.getValueAsString("paramOutputControlXML");
		AAGAppParameters.exportTransactionRecords = true;
		
		AAGAppParameters.colorSetting = Integer.valueOf(p.getValueAsString("colorSetting"));
		
		AAGAppParameters.worldYSize = Integer.valueOf(p.getValueAsString("worldSize"));
		AAGAppParameters.worldXSize = Integer.valueOf(p.getValueAsString("worldSize"));
		
		AAGAppParameters.worldXCenter = Integer.valueOf(p.getValueAsString("worldXCenter"));
		AAGAppParameters.worldYCenter = Integer.valueOf(p.getValueAsString("worldYCenter"));
		
		AAGAppParameters.numBuyer = Integer.valueOf(p.getValueAsString("numBuyer"));
		AAGAppParameters.numSeller = AAGAppParameters.worldXSize*AAGAppParameters.worldYSize;
		
		AAGAppParameters.unitTransportCost = Double.valueOf(p.getValueAsString("unitTransportCost"));
		AAGAppParameters.maxDistanceTo = Math.sqrt( (worldYSize+1)*(worldYSize+1) + (worldXSize+1)*(worldXSize+1) + 1)*0.5;
		AAGAppParameters.maxSteps = Integer.valueOf(p.getValueAsString("maxBidNums")) + 1;
		
		AAGAppParameters.neighborhoodSize = Short.valueOf(p.getValueAsString("neighborhoodSize"));
		AAGAppParameters.agResPrice = Float.valueOf(p.getValueAsString("agResPrice"));
		AAGAppParameters.meanBudget = Float.valueOf(p.getValueAsString("meanBudget"));
		AAGAppParameters.sdBudget = Float.valueOf(p.getValueAsString("sdBudget"));
		AAGAppParameters.lowBoundBudget = (float)(AAGAppParameters.unitTransportCost*AAGAppParameters.maxDistanceTo);
		
		AAGAppParameters.numSearchParcel = Integer.valueOf(p.getValueAsString("numSearchParcel"));
		
		AAGAppParameters.preInfoAvailable = Boolean.valueOf(p.getValueAsString("preInfoAvailable"));
		AAGAppParameters.onlyKeepFinalStep = Boolean.valueOf(p.getValueAsString("onlyKeepFinalStep"));
		
		AAGAppParameters.maxBidNums = Integer.valueOf(p.getValueAsString("maxBidNums"));
		
		AAGAppParameters.bInWTP = Double.valueOf(p.getValueAsString("bInWTP"));
		AAGAppParameters.utilityBeta = Double.valueOf(p.getValueAsString("utilityBeta"));
		AAGAppParameters.sdUtilBeta = Double.valueOf(p.getValueAsString("sdUtilBeta"));
		AAGAppParameters.rangeUtilBeta = Double.valueOf(p.getValueAsString("rangeUtilBeta"));
	}
	
	/**
	 * @param p
	 */
	public static void fillParameters(Parameters p)
	{
		// TODO Auto-generated method stub
		///////////////////////////////////////////
		AAGAppParameters.randomSeed = Integer.valueOf(p.getValueAsString("randomSeed"));
		AAGAppParameters.runNumber = Integer.valueOf(p.getValueAsString("runNumber"));
		AAGAppParameters.msgOutLevel = Integer.valueOf(p.getValueAsString("msgOutLevel"));
		
		// Near constant
		AAGAppParameters.dataFolderName = p.getValueAsString("dataFolderName");
		AAGAppParameters.landscapeGroupXMLFile = p.getValueAsString("landscapeGroupXMLFile");
		AAGAppParameters.logFileName = p.getValueAsString("logFileName");
		AAGAppParameters.resutlsFolderName = p.getValueAsString("resutlsFolderName");
		AAGAppParameters.paramOutputControlXML = p.getValueAsString("paramOutputControlXML");
		AAGAppParameters.exportTransactionRecords = Boolean.valueOf(p.getValueAsString("exportTransactionRecords"));
		
		AAGAppParameters.referenceLayerIndex = Short.valueOf(p.getValueAsString("referenceLayerIndex"));
		AAGAppParameters.parcelIDLayerIndex = Short.valueOf(p.getValueAsString("parcelIDLayerIndex"));
		AAGAppParameters.landUseTypeLayerIndex = Short.valueOf(p.getValueAsString("landUseTypeLayerIndex"));
		AAGAppParameters.distanceLayerIndex = Short.valueOf(p.getValueAsString("distanceLayerIndex"));
		AAGAppParameters.finalPriceLayerIndex = Short.valueOf(p.getValueAsString("finalPriceLayerIndex"));
		
		AAGAppParameters.notALayerValue = Float.valueOf(p.getValueAsString("notALayerValue"));
		
		//////////////////////////////////////////
		// Parameters
		AAGAppParameters.worldYSize = Integer.valueOf(p.getValueAsString("worldSize"));
		AAGAppParameters.worldXSize = Integer.valueOf(p.getValueAsString("worldSize"));
		
		AAGAppParameters.worldXCenter = Integer.valueOf(p.getValueAsString("worldXCenter"));
		AAGAppParameters.worldYCenter = Integer.valueOf(p.getValueAsString("worldYCenter"));
		
		AAGAppParameters.maxDistanceTo = Math.sqrt( (worldYSize+1)*(worldYSize+1) + (worldXSize+1)*(worldXSize+1) + 1)*0.5;
		AAGAppParameters.unitTransportCost = Double.valueOf(p.getValueAsString("unitTransportCost"));
		
		AAGAppParameters.numBuyer = Integer.valueOf(p.getValueAsString("numBuyer"));
		AAGAppParameters.numSeller = Integer.valueOf(p.getValueAsString("numSeller"));
		AAGAppParameters.maxBidNums = Integer.valueOf(p.getValueAsString("maxBidNums"));
		AAGAppParameters.maxBiddingTries = Integer.valueOf(p.getValueAsString("maxBiddingTries"));
		
		AAGAppParameters.preInfoAvailable = Boolean.valueOf(p.getValueAsString("preInfoAvailable"));
		AAGAppParameters.onlyKeepFinalStep = Boolean.valueOf(p.getValueAsString("onlyKeepFinalStep"));
		AAGAppParameters.excludeCenterInNeighborhood = Boolean.valueOf(p.getValueAsString("excludeCenterInNeighborhood"));
		
		// The model will end when there is no more transactions.
		AAGAppParameters.maxSteps = Integer.valueOf(p.getValueAsString("maxBidNums")) + 1;
		
		AAGAppParameters.sellDecisionUpperBound = Float.valueOf(p.getValueAsString("sellDecisionUpperBound"));		
		AAGAppParameters.buyDecisionUpperBound = Float.valueOf(p.getValueAsString("buyDecisionUpperBound"));
		AAGAppParameters.utilityBeta = Double.valueOf(p.getValueAsString("utilityBeta"));
		AAGAppParameters.sdUtilBeta = Double.valueOf(p.getValueAsString("sdUtilBeta"));
		AAGAppParameters.lowBoundBudget = (float)(AAGAppParameters.unitTransportCost*AAGAppParameters.maxDistanceTo);
		AAGAppParameters.rangeUtilBeta = Double.valueOf(p.getValueAsString("rangeUtilBeta"));
		
		AAGAppParameters.bInWTP = Double.valueOf(p.getValueAsString("bInWTP"));

		AAGAppParameters.neighborhoodSize = Short.valueOf(p.getValueAsString("neighborhoodSize"));
		AAGAppParameters.agResPrice = Float.valueOf(p.getValueAsString("agResPrice"));
		AAGAppParameters.meanBudget = Float.valueOf(p.getValueAsString("meanBudget"));
		AAGAppParameters.sdBudget = Float.valueOf(p.getValueAsString("sdBudget"));
		AAGAppParameters.numSearchParcel = Integer.valueOf(p.getValueAsString("numSearchParcel"));
	}
	
	/**
	 * worldSize,numBuyer,numSeller,preInfoAvailable,maxBidNums,utilityBeta,sdUtilBeta,rangeUtilBeta,bInWTP,agResPrice,meanBudget,sdBudget,numSearchParcel,stepNum,luFile,tpFile
	 * @param sep
	 * @return
	 */
	public static String paramsValueToString(String sep)
	{
		return paramsValueToString(sep,true);
	}
	
	public static String paramsValueToString(String sep, boolean outControl)
	{
		String rslt = "";
		try
		{
			if(outControl)
			{
				rslt = paramsValueToStringWithControl(sep);
			}
			else
			{
				rslt = paramsValueToStringWithoutControl(sep);
			}
		}
		catch (InvalidPropertiesFormatException e)
		{
			e.printStackTrace();
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		
		return rslt;
	}
	
	public static String paramsValueToStringWithoutControl(String sep)
	{
		StringBuilder sb = new StringBuilder();
		// Send all output to the Appendable object sb
		java.util.Formatter formatter = new java.util.Formatter(sb, Locale.US);
		//Explicit argument indices may be used to re-order output.
		formatter.format("%1$.2f", AAGAppParameters.randomSeed + sep);

		sb.append(AAGAppParameters.worldYSize + sep);
		//sb.append(AAGAppParameters.worldXSize + sep);
		sb.append(AAGAppParameters.numBuyer + sep);
		sb.append(AAGAppParameters.numSeller + sep);
		
		//sb.append(AAGAppParameters.preInfoAvailable + sep);
		//sb.append(AAGAppParameters.excludeCenterInNeighborhood + sep);
		
		sb.append(AAGAppParameters.maxBidNums + sep);
		//sb.append(AAGAppParameters.maxBiddingTries + sep);
		
		//sb.append(AAGAppParameters.sellDecisionUpperBound + sep);
		//sb.append(AAGAppParameters.buyDecisionUpperBound + sep);
		
		
		sb.append(AAGAppParameters.runNumber + sep);
		sb.append(AAGAppParameters.utilityBeta + sep);
		sb.append(AAGAppParameters.sdUtilBeta + sep);
		//sb.append(AAGAppParameters.rangeUtilBeta + sep);
		//sb.append(AAGAppParameters.bInWTP + sep);
		
		//sb.append(AAGAppParameters.unitTransportCost + sep);
		
		sb.append(AAGAppParameters.agResPrice + sep);
		sb.append(AAGAppParameters.meanBudget + sep);
		sb.append(AAGAppParameters.sdBudget + sep);
		
		sb.append(AAGAppParameters.neighborhoodSize + sep);
		sb.append(AAGAppParameters.numSearchParcel);
	
		return sb.toString();
	}
	
	/**
	 * worldYSize,worldXSize,numBuyer,numSeller,maxBidNums,maxBiddingTries,sellDecisionUpperBound,buyDecisionUpperBound,
	 * utilityBeta,bInWTP,unitTransportCost,agResPrice,meanBudget,sdBudget,neighborhoodSize,numSearchParcel
	 * stepNum,luFile,tpFile
	 * @param sep
	 * @return
	 * @throws IOException 
	 * @throws FileNotFoundException 
	 * @throws InvalidPropertiesFormatException 
	 */
	public static String paramsNameToString(String sep) throws InvalidPropertiesFormatException, FileNotFoundException, IOException
	{
		if(paramOutControl == null || paramOutControl.size() <= 0)
		{
			paramOutControl = new Properties();
			paramOutControl.loadFromXML(new FileInputStream(new File(dataFolderName + paramOutputControlXML)));
		}
		
		StringBuilder sb = new StringBuilder();

		if(Boolean.valueOf(paramOutControl.getProperty("randomSeed")))
		{
			sb.append("randomSeed" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("runNumber")))
		{
			sb.append("runNumber" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("msgOutLevel"))) 
		{
			sb.append("msgOutLevel" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("dataFolderName"))) 
		{			
			sb.append("dataFolderName" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("resutlsFolderName"))) 
		{
			sb.append("resutlsFolderName" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("landscapeGroupXMLFile"))) 
		{
			sb.append("landscapeGroupXMLFile" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("logFileName"))) 
		{
			sb.append("logFileName" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("referenceLayerIndex"))) 
		{
			sb.append("referenceLayerIndex" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("parcelIDLayerIndex"))) 
		{
			sb.append("parcelIDLayerIndex" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("landUseTypeLayerIndex"))) 
		{
			sb.append("landUseTypeLayerIndex" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("distanceLayerIndex"))) 
		{
			sb.append("distanceLayerIndex" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("finalPriceLayerIndex"))) 
		{
			sb.append("finalPriceLayerIndex" + sep);					
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("worldSize"))) 
		{
			sb.append("worldXSize " + sep);
			sb.append("worldYSize" + sep);
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("worldXCenter"))) 
		{
			sb.append("worldXCenter " + sep);
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("worldYCenter"))) 
		{
			sb.append("worldYCenter " + sep);
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("numBuyer"))) 
		{
			sb.append("numBuyer" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("numSeller"))) 
		{
			sb.append("numSeller" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("sellDecisionUpperBound")))
		{
			sb.append("sellDecisionUpperBound" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("buyDecisionUpperBound")))
		{
			sb.append("buyDecisionUpperBound" + sep);				
		}
		if(Boolean.valueOf(paramOutControl.getProperty("maxBidding Tries"))) 
		{
			sb.append("maxBiddingTries" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("onlyKeepFinalStep"))) 
		{
			sb.append("onlyKeepFinalStep" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("unitTransportCost"))) 
		{
			sb.append("unitTransportCost" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("neighborhoodSize"))) 
		{
			sb.append("neighborhoodSize" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("preInfoAvailable"))) 
		{
			sb.append("preInfoAvailable" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("excludeCenterInNeighborhood"))) 
		{
			sb.append("excludeCenterInNeighborhood" + sep);

		}
		if(Boolean.valueOf(paramOutControl.getProperty("bInWTP"))) 
		{
			sb.append("bInWTP" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("meanBudget"))) 
		{
			sb.append("meanBudget" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("sdBudget"))) 
		{
			sb.append("sdBudget" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("numSearchParcel"))) 
		{
			sb.append("numSearchParcel" + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("maxBidNums"))) 
		{
			sb.append("maxBidNums" + sep);					
		}
		if(Boolean.valueOf(paramOutControl.getProperty("agResPrice"))) 
		{
			sb.append("agResPrice" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("utilityBeta"))) 
		{
			sb.append("utilityBeta" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("sdUtilBeta"))) 
		{
			sb.append("sdUtilBeta" + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("rangeUtilBeta"))) 
		{
			sb.append("rangeUtilBeta" + sep);			
		}
		
		int lastCommaIndex = sb.lastIndexOf(",");
		if(lastCommaIndex >= 0)
		{
			sb.deleteCharAt(lastCommaIndex);
		}
		
		return sb.toString();
	}
	
	/**
	 * worldSize,numBuyer,numSeller,preInfoAvailable,maxBidNums,utilityBeta,sdUtilBeta,rangeUtilBeta,bInWTP,agResPrice,meanBudget,sdBudget,numSearchParcel,stepNum,luFile,tpFile
	 * @param sep
	 * @return
	 * @throws IOException 
	 * @throws FileNotFoundException 
	 * @throws InvalidPropertiesFormatException 
	 */
	public static String paramsValueToStringWithControl(String sep) throws InvalidPropertiesFormatException, FileNotFoundException, IOException
	{
		if(paramOutControl == null || paramOutControl.size() <= 0)
		{
			paramOutControl = new Properties();
			paramOutControl.loadFromXML(new FileInputStream(new File(dataFolderName + paramOutputControlXML)));
		}
		
		StringBuilder sb = new StringBuilder();
		java.util.Formatter formatter = new java.util.Formatter(sb, Locale.US);

		if(Boolean.valueOf(paramOutControl.getProperty("programVersion")))
		{
			sb.append(AAGAppParameters.programVersion + sep);
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("randomSeed")))
		{
			formatter.format("%1$.2f", AAGAppParameters.randomSeed);
			sb.append(sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("runNumber")))
		{
			sb.append(AAGAppParameters.runNumber + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("msgOutLevel"))) 
		{
			sb.append(AAGAppParameters.msgOutLevel + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("dataFolderName"))) 
		{			
			sb.append(AAGAppParameters.dataFolderName + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("resutlsFolderName"))) 
		{
			sb.append(AAGAppParameters.resutlsFolderName + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("landscapeGroupXMLFile"))) 
		{
			sb.append(AAGAppParameters.landscapeGroupXMLFile + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("logFileName"))) 
		{
			sb.append(AAGAppParameters.logFileName + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("referenceLayerIndex"))) 
		{
			sb.append(AAGAppParameters.referenceLayerIndex + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("parcelIDLayerIndex"))) 
		{
			sb.append(AAGAppParameters.parcelIDLayerIndex + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("landUseTypeLayerIndex"))) 
		{
			sb.append(AAGAppParameters.landUseTypeLayerIndex + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("distanceLayerIndex"))) 
		{
			sb.append(AAGAppParameters.distanceLayerIndex + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("finalPriceLayerIndex"))) 
		{
			sb.append(AAGAppParameters.finalPriceLayerIndex + sep);					
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("worldSize"))) 
		{
			sb.append(AAGAppParameters.worldXSize  + sep);
			sb.append(AAGAppParameters.worldYSize + sep);
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("worldXCenter"))) 
		{
			sb.append(AAGAppParameters.worldXCenter  + sep);
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("worldYCenter"))) 
		{
			sb.append(AAGAppParameters.worldYCenter  + sep);
		}
		
		if(Boolean.valueOf(paramOutControl.getProperty("numBuyer"))) 
		{
			sb.append(AAGAppParameters.numBuyer + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("numSeller"))) 
		{
			sb.append(AAGAppParameters.numSeller + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("sellDecisionUpperBound")))
		{
			sb.append(AAGAppParameters.sellDecisionUpperBound + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("buyDecisionUpperBound")))
		{
			sb.append(AAGAppParameters.buyDecisionUpperBound + sep);				
		}
		if(Boolean.valueOf(paramOutControl.getProperty("maxBidding Tries"))) 
		{
			sb.append(AAGAppParameters.maxBiddingTries + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("onlyKeepFinalStep"))) 
		{
			sb.append(AAGAppParameters.onlyKeepFinalStep + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("unitTransportCost"))) 
		{
			sb.append(AAGAppParameters.unitTransportCost + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("neighborhoodSize"))) 
		{
			sb.append(AAGAppParameters.neighborhoodSize + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("preInfoAvailable"))) 
		{
			sb.append(AAGAppParameters.preInfoAvailable + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("excludeCenterInNeighborhood"))) 
		{
			sb.append(AAGAppParameters.excludeCenterInNeighborhood + sep);

		}
		if(Boolean.valueOf(paramOutControl.getProperty("bInWTP"))) 
		{
			sb.append(AAGAppParameters.bInWTP + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("meanBudget"))) 
		{
			sb.append(AAGAppParameters.meanBudget + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("sdBudget"))) 
		{
			sb.append(AAGAppParameters.sdBudget + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("numSearchParcel"))) 
		{
			sb.append(AAGAppParameters.numSearchParcel + sep);
		}
		if(Boolean.valueOf(paramOutControl.getProperty("maxBidNums"))) 
		{
			sb.append(AAGAppParameters.maxBidNums + sep);					
		}
		if(Boolean.valueOf(paramOutControl.getProperty("agResPrice"))) 
		{
			sb.append(AAGAppParameters.agResPrice + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("utilityBeta"))) 
		{
			sb.append(AAGAppParameters.utilityBeta + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("sdUtilBeta"))) 
		{
			sb.append(AAGAppParameters.sdUtilBeta + sep);			
		}
		if(Boolean.valueOf(paramOutControl.getProperty("rangeUtilBeta"))) 
		{
			sb.append(AAGAppParameters.rangeUtilBeta + sep);			
		}
		
		int lastCommaIndex = sb.lastIndexOf(",");
		if(lastCommaIndex >= 0)
		{
			sb.deleteCharAt(lastCommaIndex);
		}
		
		return sb.toString();
	}
}
