package sluceii.app.aag;

import java.awt.GridLayout;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import javax.swing.JLabel;
import javax.swing.JPanel;

import repast.simphony.context.Context;
import repast.simphony.context.space.grid.GridFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ISchedule;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridBuilderParameters;
import repast.simphony.space.grid.SimpleGridAdder;
import repast.simphony.ui.RSApplication;
import repast.simphony.valueLayer.GridValueLayer;
import sluceii.agent.Household;
import sluceii.agent.RuralLandOwner;
import sluceii.agent.SimpleAgent;
import sluceii.agent.initialization.RandomInitializer;
import sluceii.landmarket.LandMarketListing;
import sluceii.landscape.LandCoverDataHelper;
import sluceii.params.Constants;
import sluceii.params.LogHelper;
import sluceii.params.LogHelperFactory;
import sluceii.params.MySQLDatabaseLogger;
import sluceii.params.SimpleLogHelper;
import sluceii.params.ToolAndTips;
import umn.geog.helia.landscape.RasterLayer;

/**
 * This context creator is the main entry point for modeling.
 * Its core method is build method.
 * 
 * 
 * See
 * http://repast.sourceforge.net/docs/tutorial/SIM/index.html
 * 3-98 and 3-99 for reference.
 * 
 * Except context, all other elements, including projections (geography, network, grid, etc), agents (all actual agents)
 * must be created in score file. Their names and labels must be correct, which means class definition is consistent with
 * score file. However, they don't need to be parameterized. Leave that part to build method of ContextBuilder.
 * 
 * In the GUI of Model, create some display and save it by clicking on the Floppy Disk Icon. Repast Simphony will automatically
 * update the scenario.xml file and add several display configuration xml files.
 * 
 * @author Shipeng Sun
 *
 */
public class ContextCreator implements ContextBuilder<SimpleAgent>
{
	/**
	 * @uml.property name="xSize"
	 */
	int param_xSize;
	/**
	 * @uml.property name="ySize"
	 */
	int param_ySize;
	/**
	 * @uml.property name="randomSeed"
	 */
	int param_seed;
	
	
	/**
	 * The run number passed from GridSweeper
	 */
	int param_runNum;

	/**
	 */
	double param_endAt;
	
	/**
	 * 
	 */
	private LogHelper logHelper;
	
	public static String transactionFileName ="";

	/**
	 */
	private Grid<SimpleAgent> agentGrid;
	
	/**
	 * 
	 */
	private LandMarketListing landMarket;
	
	/**
	 * @uml.property name="context2"
	 * @uml.associationEnd multiplicity="(0 -1)" elementType="sluceii.Agent"
	 */
	private Context<SimpleAgent> context2;
	
	public static boolean activity = false;
	
	public static long totalStepNum = 1;
	
	public static long totalTansactionNum = 0;

	private java.util.Date modelStartTime;
	
	public static JPanel userPanelControl;
	
	/**
	 * 
	 */
	public ContextCreator()
	{
		super();
		// TODO Auto-generated constructor stub
	}

	/**
	 * Builds and returns a context. Building a context consists of filling it
	 * with agents, adding projects and so forth. When this is called for the
	 * master context the system will pass in a created context based on
	 * information given in the model.score file. When called for subcontexts,
	 * each subcontext that was added when the master context was built will be
	 * passed in.
	 * 
	 * @param context
	 * @return the built context.
	 */
	public Context<SimpleAgent> build(Context<SimpleAgent> context)
	{
		Household.transactionLogHelper = null;
		
		String hostName = "";
	    try 
	    {
		    hostName = java.net.InetAddress.getLocalHost().getHostName();
	    } 
	    catch (java.net.UnknownHostException e) 
	    {
	        System.err.printf("\n\nError trying to fetch hostname!\n\n");
	        e.printStackTrace();
	    }
	    
	    hostName = "# Hostname: " + hostName;
	    ContextCreator.writeStdOutMessage(hostName, 0, "");
	    ContextCreator.writeStdOutMessage(ContextCreator.CurrentDateTimeString(2), 0, "# Start time: ");
	    modelStartTime = new java.util.Date();
	    ContextCreator.writeStdOutMessage("Started the context build method...",1,"# ");
	    
		context2 = context;
		context2.clear();
		ContextCreator.activity = false;
		totalTansactionNum = 0;
		totalStepNum = 1;
		
		SimpleAgent.context = context;		
		
		if(ContextCreator.userPanelControl == null && !(RunEnvironment.getInstance().isBatch()))
		{
			ContextCreator.userPanelControl = new JPanel(new GridLayout(0,1));
			JLabel txtLabel = new JLabel("# of transactions:");
			txtLabel.setVerticalAlignment(JLabel.TOP);
			userPanelControl.add(txtLabel);
		}
		else if(!(RunEnvironment.getInstance().isBatch()))
		{
			((JLabel)ContextCreator.userPanelControl.getComponent(0)).setText("Number of Transactions:\n" + ContextCreator.totalTansactionNum);
		}
		
		if(!RunEnvironment.getInstance().isBatch())
		{
			RSApplication.getRSApplicationInstance().addCustomUserPanel(ContextCreator.userPanelControl);
		}
		
		setupParameters(); // Load parameters from the GUI
		
		setupLogHelper();
		
		if (param_seed <= 0)
		{
			// set the seed randomly
		}
		else
		{
			RandomHelper.setSeed(param_seed);
		}

		// Load the landscape data (external data defined in xml file)
		setupLandscape();

		setupGrids(); // set up the agent grids and the value layers
		setupActorAgents(); // create and initialize agents
		
		setupLandMarket(); // create the land market object
		
		//setupSellerAgents(); // create the seller agents and parameterize them
		//setupBuyerAgents(); // create the buyer agents and parameterize them
		//setupRemainingAgentCharacteristics(); // parameterize agent
		// characteristics that are the
		// same for all agents
		// regardless of if they're
		// buyers or sellers
		
		this.setupSchedule();

		ContextCreator.writeStdOutMessage("Finished the context build method.",1,"# ");
		
		return context;
	}

	private void setupLogHelper()
	{
		ContextCreator.writeStdOutMessage("Start setting up the log files.",2,"# ");

		// TODO Auto-generated method stub
		// this.logHelper = new SimpleLogHelper(AAGAppParameters.dataFolderName + AAGAppParameters.logFileName, 1, false);
		this.logHelper = LogHelperFactory.getLogger(1);
		this.logHelper.logRun();
		
		// TODO: create a new transaction file
		ContextCreator.transactionFileName = "";
		if(AAGAppParameters.exportTransactionRecords)
		{
			ContextCreator.transactionFileName = "trans_" + ToolAndTips.getCurrentDateTime(2) + ".csv";
			Household.transactionLogHelper = LogHelperFactory.createTransactionLogger(2, AAGAppParameters.dataFolderName + "TransRecords/" + ContextCreator.transactionFileName, false);
		}
		

		ContextCreator.writeStdOutMessage("Finished setting up the log files.",2,"# ");
	}

	/**
	 * Set up the land market
	 */
	protected void setupLandMarket()
	{
		ContextCreator.writeStdOutMessage("Start setting up the land marketd.",2,"# ");

		// TODO Auto-generated method stub
 		this.landMarket = new LandMarketListing();
 		this.landMarket.clear();

		Household.landMarket = this.landMarket;
		RuralLandOwner.landMarket = this.landMarket;
		this.landMarket.setParcelList(LandCoverDataHelper.parcelList);
		
		ContextCreator.writeStdOutMessage("Finished setting up the market.",2,"# ");
	}
	
	protected void setupSchedule()
	{
		// see http://repast.sourceforge.net/docs/reference/SIM/Working%20with%20the%20scheduler.html
		// for scheduler setup.
		// Schedule ContextCreator.step to run starting at time step x and
		// repeat every y steps
		ISchedule schedule = RunEnvironment.getInstance().getCurrentSchedule();
		// Set its priority as 1 lower than the agent actions.
		ScheduleParameters params = ScheduleParameters.createRepeating(Constants.STARTSTEP, Constants.INTERVALSTEP, 1);
		// schedule.schedule(ScheduleParameters.createOneTime(0), this,
		// "setup");
		schedule.schedule(params, this, "step");

		if (RunEnvironment.getInstance().isBatch())
		{ 
			// If running in batch mode, tell the scheduler when to end each run.
			// RunEnvironment.getInstance().endAt(AAGAppParameters.maxSteps); // stop the model
		}
		// RunEnvironment.getInstance().pauseAt(param_endAt); // if it is not a
		// batch run then
		// pause the model
		// at the user
		// defined last time
		// step
	}

	/**
	 * The environment parameters contain the user-editable values that appear
	 * in the GUI. Get the parameters p and then specifically the initial
	 * numbers of wolves and sheep.
	 * 
	 * TODO: it is kind of tedious for users to type in many parameters.
	 * Currently most model parameters will be fixed in the ModelParameters file.
	 * In the later version, an XML file will be used to save all external meta-parameters.
	 * 
	 * @return void
	 */
	protected void setupParameters()
	{
		ContextCreator.writeStdOutMessage("Started setupParameters Method...",2,"# ");
		
		if (RunEnvironment.getInstance().isBatch())
		{
			Parameters p = RunEnvironment.getInstance().getParameters();
			AAGAppParameters.fillParameters(p);
			
			param_seed = Integer.valueOf(p.getValueAsString("randomSeed"));
			param_runNum = Integer.valueOf(p.getValueAsString("runNumber"));
		}
		else
		{
			Parameters p = RunEnvironment.getInstance().getParameters();
			AAGAppParameters.fillGUIParameters(p);
			
			param_seed = Integer.valueOf(p.getValueAsString("randomSeed"));
			param_runNum = 1;//Integer.valueOf(p.getValueAsString("runNumber"));
		}

		ContextCreator.writeStdOutMessage("Finished setupParameters Method.",2, "# ");
	}

	/**
	 * Create a new 2D grid to model the discrete patches of residential
	 * households. The inputs to the GridFactory include the grid name, the
	 * context in which to place the grid, and the grid parameters. Grid
	 * parameters include the border specification, random adder for populating
	 * the grid with agents, boolean for multiple occupancy, and the dimensions
	 * of the grid.
	 * 
	 * @return void
	 */
	@SuppressWarnings("unchecked")
	protected void setupGrids()
	{
		ContextCreator.writeStdOutMessage("Started setupGrids Method...", 2, "# ");

		GridFactoryFinder.createGridFactory(null).createGrid(
				Constants.AGENT2D,
				context2,
				new GridBuilderParameters<SimpleAgent>(
						new repast.simphony.space.grid.WrapAroundBorders(),
						new SimpleGridAdder<SimpleAgent>(), true, param_xSize,
						param_ySize));

		GridFactoryFinder.createGridFactory(null).createGrid(
				Constants.LANDSCAPE2D,
				context2,
				new GridBuilderParameters<SimpleAgent>(
						new repast.simphony.space.grid.WrapAroundBorders(),
						new SimpleGridAdder<SimpleAgent>(), true, param_xSize,
						param_ySize));

		GridValueLayer vlLUType = new GridValueLayer(Constants.LANDUSETYPE,
				true, new repast.simphony.space.grid.WrapAroundBorders(),
				param_xSize, param_ySize);

		context2.addValueLayer(vlLUType);

		RasterLayer luLayer = (RasterLayer) LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AAGAppParameters.landUseTypeLayerIndex);
		for(int i = 0; i < param_xSize; i++)
		{
			for(int j = 0; j < param_ySize; j++)
			{
				// Set the land use layer just for display
				vlLUType.set(luLayer.getValueAt(i, j).doubleValue(), i,j);
			}
		}
		
		agentGrid = (Grid<SimpleAgent>) context2.getProjection(Constants.AGENT2D);
		SimpleAgent.agentGrid = agentGrid;
		
		ContextCreator.writeStdOutMessage("Finished setupGrids Method.", 2 , "# ");
	}

	/**
	 * Set up and load landscape/land cover/land use/environment and other external data into
	 * the system. This will be done using LandCoverDataHelper
	 * 
	 * @see LandCoverDataHelper
	 */
	protected void setupLandscape()
	{
		ContextCreator.writeStdOutMessage("Started setting up the landscape...", 2, "# ");

		LandCoverDataHelper.clear();
		
		// Load all external data into the group object
		LandCoverDataHelper.loadRootLandscapeLayerGroupInfo(AAGAppParameters.dataFolderName, AAGAppParameters.landscapeGroupXMLFile);
		
		/*
		LandCoverDataHelper.loadRootLandscapeLayerGroup(AAGAppParameters.dataFolderName, AAGAppParameters.landscapeGroupXMLFile);
		
		// Set the reference layer for all actor agents
		SimpleAgent.referenceLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AAGAppParameters.referenceLayerIndex);
		
		this.param_xSize = SimpleAgent.referenceLayer.getCols();
		this.param_ySize = SimpleAgent.referenceLayer.getRows();
		*/
		
		param_xSize = AAGAppParameters.worldXSize;
		param_ySize = AAGAppParameters.worldYSize;
		
		DataPreparation.resizeLayers(param_xSize, param_ySize, AAGAppParameters.worldXCenter, AAGAppParameters.worldYCenter);
		
		//RasterLayer<FloatMatrix> disLayer = (RasterLayer<FloatMatrix>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AAGAppParameters.distanceLayerIndex);
		//LandCoverDataHelper.saveRasterLayer(disLayer, true);
		
		ContextCreator.writeStdOutMessage("Finished loading landscape/land cover/land use/environment and other external data", 2, "# ");
	}

	
	/**
	 * 
	 */
	@SuppressWarnings("deprecation")
	protected void setupActorAgents()
	{
		ContextCreator.writeStdOutMessage("Started to setup sellerAgents...",2 , "# ");
		int countSellers = 0;

		this.initAgents(AAGAppParameters.dataFolderName + AAGAppParameters.agentInitFile);
		
		// Update the parcel list, each parcel will get IDs of the cells it contains
		RasterLayer<?> parcelIDLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AAGAppParameters.parcelIDLayerIndex);
		LandCoverDataHelper.updateParcelList(parcelIDLayer);
		
		// for every location where land exists, create a seller agent
		ContextCreator.writeStdOutMessage("Finished setting up sellerAgents. There is " + countSellers + " in total.",2 , "# ");
	}
	
	/**
	 * This method should be promoted to a separate class to initialize agents.
	 * @param fileName The CSV file having agent characteristics
	 */
	protected void initAgents(String fileName)
	{
		ContextCreator.writeStdOutMessage("Started to setup agents...",2 , "# ");

		/*
		CSVfileLoader agentLoader = new CSVfileLoader(fileName, this.context2);
		
		if(!agentLoader.load())
		{
			System.out.println("Error: Cannot initialize actor agents from " + fileName	+ ". Please check.");
		}
		*/
		RandomInitializer riObj = new RandomInitializer(this.context2);
		riObj.setCellCenterX(AAGAppParameters.worldXCenter);
		riObj.setCellCenterY(AAGAppParameters.worldYCenter);
		
		riObj.setNumBuyer(AAGAppParameters.numBuyer);
		riObj.setNumSeller(AAGAppParameters.numSeller);
				
		if(!riObj.init())
		{
			System.err.println("Error: Cannot initialize actor agents. Please check.");
		}
		
		ContextCreator.writeStdOutMessage("Finished setting up agents...",2 , "# ");
	}
	
	/**
	 * For each Model Step, run the following method
	 */
	public void step()
	{
		ContextCreator.writeStdOutMessage("Started Step Method...",2 , "# ");

		if(!ContextCreator.activity)
		{
			if(AAGAppParameters.onlyKeepFinalStep)
			{
				saveLandscapeData();
			}
			
			RunEnvironment.getInstance().endRun();
			
		    ContextCreator.writeStdOutMessage(ContextCreator.CurrentDateTimeString(2), 0, "# End time: ");
		    java.util.Date modelEndTime = new java.util.Date();
		    
		    long duration = modelEndTime.getTime() - this.modelStartTime.getTime();
		    ContextCreator.writeStdOutMessage(ContextCreator.getElapsedTimeHoursMinutesSecondsString(duration), 0, "# Duration : ");
		    ContextCreator.writeStdOutMessage("******************************************************************************",1,"");
		}
		else
		{
			if(!AAGAppParameters.onlyKeepFinalStep)
			{
				saveLandscapeData();
			}
		}
		
		ContextCreator.activity = false;
		
		ContextCreator.writeStdOutMessage("Finished Step Method.",2 , "# ");
	}
	
	/**
	 * 
	 */
	private void saveLandscapeData()
	{
		ContextCreator.writeStdOutMessage("Saving landscape data ...",1 , "# ");
		
		// Save the intermediate land use type layer
		String totalStepStr = "" + ContextCreator.totalStepNum++;
		
		RasterLayer<?> dataLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AAGAppParameters.landUseTypeLayerIndex);
		String fileName = LandCoverDataHelper.saveRasterLayer3(dataLayer, true, true, totalStepStr, true);
		String fileNameB = LandCoverDataHelper.saveRasterLayer4(dataLayer, AAGAppParameters.resutlsFolderName, true, true, "" + param_runNum, true);
		
		ContextCreator.writeStdOutMessage("		Saving " + dataLayer.getStoragePath() + " ... ",1 , "# ");

		
		RasterLayer<?> tpcLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AAGAppParameters.finalPriceLayerIndex);
		String priceFile = LandCoverDataHelper.saveRasterLayer3(tpcLayer, true, true, totalStepStr, true);
		String priceFileB = LandCoverDataHelper.saveRasterLayer4(tpcLayer, AAGAppParameters.resutlsFolderName, true, true, "" + param_runNum, true);

		ContextCreator.writeStdOutMessage("		Saving " + tpcLayer.getStoragePath() + " ... ",1 , "# ");
		
		String tickStr = String.valueOf(RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
		
		if(logHelper instanceof SimpleLogHelper)
		{
			if(AAGAppParameters.exportTransactionRecords && transactionFileName != "")
			{
				((SimpleLogHelper)logHelper).writeLogWithoutTime(AAGAppParameters.paramsValueToString(",") + "," + tickStr + "," + fileName + "," + priceFile + "," + fileNameB + "," + priceFileB + "," + ContextCreator.transactionFileName , 0);
			}
			else
			{
				((SimpleLogHelper)logHelper).writeLogWithoutTime(AAGAppParameters.paramsValueToString(",") + "," + tickStr + "," + fileName + "," + priceFile + "," + fileNameB + "," + priceFileB, 0);
			}

		}
		else if(logHelper instanceof MySQLDatabaseLogger)
		{
			logHelper.logLandscapeData(0, RunEnvironment.getInstance().getCurrentSchedule().getTickCount(), fileName, priceFile, fileNameB, priceFileB);
		}
	}
	
	/**
	 * @param msg
	 * @param outLevel
	 * @param preFix
	 */
	public static void writeStdOutMessage(String msg, int outLevel, String preFix)
	{
		if(outLevel <= AAGAppParameters.msgOutLevel)
		{
			System.out.println(preFix + msg);
		}
	}
	
	/**
	 * @param pStream
	 * @param msg
	 * @param outLevel
	 */
	public static void writeStdOutMessage(java.io.PrintStream pStream, String msg, int outLevel)
	{
		if(outLevel <= AAGAppParameters.msgOutLevel)
		{
			pStream.println(msg);
		}
	}
	
	/**
	 * 1 yyyy_MM_dd_HH_mm_ss
	 * 2 yyyy MM dd HH:mm:ss
	 * @param formatType
	 * @return The current date/time in the format of yyyy_MM_dd_HH_mm_ss
	 */
	public static String CurrentDateTimeString(int formatType)
	{
		DateFormat dateFormat;
		switch(formatType)
		{
		case 1:
			dateFormat = new SimpleDateFormat("yyyy_MM_dd_HH_mm_ss");
			break;
		case 2:
			dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
			break;
		default:
			dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
			break;
		}
	    java.util.Date date = new java.util.Date();
	    String datetime = dateFormat.format(date);
	    return datetime;
	}
	
	/**
	 * @param elapsedTime Time in milliseconds
	 * @return A time string in the format of xx(xxxx...):xx:xx
	 */
	public static String getElapsedTimeHoursMinutesSecondsString(long elapsedTime) 
	{     
	    String format = String.format("%%0%dd", 2);
	    elapsedTime = elapsedTime / 1000;
	    String seconds = String.format(format, elapsedTime % 60);
	    String minutes = String.format(format, (elapsedTime % 3600) / 60);
	    String hours = String.format(format, elapsedTime / 3600);
	    String time =  hours + ":" + minutes + ":" + seconds;
	    return time;
	}

	public static void setTotalTransactionNum(long num)
	{
		// TODO Auto-generated method stub
		if(!RunEnvironment.getInstance().isBatch())
		{
			((JLabel)ContextCreator.userPanelControl.getComponent(0)).setText("Number of Transactions:\n" + ContextCreator.totalTansactionNum);
		}
	}
}
