package sluceii.app.mn.carver;

import repast.simphony.context.Context;
import repast.simphony.context.space.grid.GridFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ISchedule;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridBuilderParameters;
import repast.simphony.space.grid.SimpleGridAdder;
import repast.simphony.valueLayer.GridValueLayer;
import sluceii.agent.Household;
import sluceii.agent.RuralLandOwner;
import sluceii.agent.SimpleAgent;
import sluceii.agent.initialization.CSVfileLoader;
import sluceii.landmarket.LandMarketListing;
import sluceii.landscape.LandCoverDataHelper;
import sluceii.params.Constants;
import sluceii.app.aag.AAGAppParameters;
import umn.geog.helia.landscape.RasterLayer;

/**
 * This context creator is the main entry point for modeling.
 * Its core method is build method.
 * 
 * 
 * See
 * http://repast.sourceforge.net/docs/tutorial/SIM/index.html
 * 3-98 and 3-99 for reference.
 * 
 * Except context, all other elements, including projections (geography, network, grid, etc), agents (all actual agents)
 * must be created in score file. Their names and labels must be correct, which means class definition is consistent with
 * score file. However, they don't need to be parameterized. Leave that part to build method of ContextBuilder.
 * 
 * In the GUI of Model, create some display and save it by clicking on the Floppy Disk Icon. Repast Simphony will automatically
 * update the scenario.xml file and add several display configuration xml files.
 * 
 * @author Shipeng Sun
 *
 */
public class ContextCreator implements ContextBuilder<SimpleAgent>
{
	/**
	 * @uml.property name="xSize"
	 */
	int param_xSize;
	/**
	 * @uml.property name="ySize"
	 */
	int param_ySize;
	/**
	 * @uml.property name="seed"
	 */
	int param_seed;

	/**

	 */
	double param_endAt;

	/**
	 */
	private Grid<SimpleAgent> agentGrid;
	
	/**
	 * 
	 */
	private LandMarketListing landMarket;
	
	/**
	 * @uml.property name="context2"
	 * @uml.associationEnd multiplicity="(0 -1)" elementType="sluceii.Agent"
	 */
	private Context<SimpleAgent> context2;


	/**
	 * 
	 */
	public ContextCreator()
	{
		super();
		// TODO Auto-generated constructor stub
	}

	/**
	 * Builds and returns a context. Building a context consists of filling it
	 * with agents, adding projects and so forth. When this is called for the
	 * master context the system will pass in a created context based on
	 * information given in the model.score file. When called for subcontexts,
	 * each subcontext that was added when the master context was built will be
	 * passed in.
	 * 
	 * @param context
	 * @return the built context.
	 */
	public Context<SimpleAgent> build(Context<SimpleAgent> context)
	{
		System.out.println("Started the context build method...");
		context2 = context;
		SimpleAgent.context = context;
		
		setupParameters(); // Load parameters from the GUI
		
		if (param_seed <= 0)
		{
			// set the seed randomly
		}
		else
		{
			RandomHelper.setSeed(param_seed);
		}

		// Load the landscape data (external data defined in xml file)
		setupLandscape();

		setupGrids(); // set up the agent grids and the value layers
		setupActorAgents(); // create and initialize agents
		
		setupLandMarket(); // create the land market object
		
		//setupSellerAgents(); // create the seller agents and parameterize them
		//setupBuyerAgents(); // create the buyer agents and parameterize them
		//setupRemainingAgentCharacteristics(); // parameterize agent
		// characteristics that are the
		// same for all agents
		// regardless of if they're
		// buyers or sellers
		
		this.setupSchedule();

		System.out.println("Finished the context build method.");
		System.out
				.println("******************************************************************************");
		
		return context;
	}

	/**
	 * Set up the land market
	 */
	protected void setupLandMarket()
	{
		System.out.println("Start setting up the land marketd.");

		// TODO Auto-generated method stub
 		this.landMarket = new LandMarketListing();

		Household.landMarket = this.landMarket;
		RuralLandOwner.landMarket = this.landMarket;
		System.out.println("Finished setting up the market.");
	}
	
	protected void setupSchedule()
	{
		// see http://repast.sourceforge.net/docs/reference/SIM/Working%20with%20the%20scheduler.html
		// for scheduler setup.
		// Schedule ContextCreator.step to run starting at time step x and
		// repeat every y steps
		ISchedule schedule = RunEnvironment.getInstance().getCurrentSchedule();
		// Set its priority as 1 lower than the agent actions.
		ScheduleParameters params = ScheduleParameters.createRepeating(Constants.STARTSTEP, Constants.INTERVALSTEP, 1);
		// schedule.schedule(ScheduleParameters.createOneTime(0), this,
		// "setup");
		schedule.schedule(params, this, "step");

		if (RunEnvironment.getInstance().isBatch())
		{ // If running in batch mode, tell the scheduler when to end each run.
			RunEnvironment.getInstance().endAt(param_endAt); // stop the model
			// at the
			// parameter
			// defined last
			// time step
		}
		// RunEnvironment.getInstance().pauseAt(param_endAt); // if it is not a
		// batch run then
		// pause the model
		// at the user
		// defined last time
		// step
	}

	/**
	 * The environment parameters contain the user-editable values that appear
	 * in the GUI. Get the parameters p and then specifically the initial
	 * numbers of wolves and sheep.
	 * 
	 * TODO: it is kind of tedious for users to type in many parameters.
	 * Currently most model parameters will be fixed in the AAGAppParameters file.
	 * In the later version, an XML file will be used to save all external meta-parameters.
	 * 
	 * @return void
	 */
	protected void setupParameters()
	{
		System.out.println("Started setupParameters Method...");
		@SuppressWarnings("unused")
		Parameters p = RunEnvironment.getInstance().getParameters();

		// This value is fixed right now.
		//AAGAppParameters.landscapeGroupXMLFile = (String) p.getValue("rootXML");

		System.out.println("Finished setupParameters Method.");
	}

	/**
	 * Create a new 2D grid to model the discrete patches of residential
	 * households. The inputs to the GridFactory include the grid name, the
	 * context in which to place the grid, and the grid parameters. Grid
	 * parameters include the border specification, random adder for populating
	 * the grid with agents, boolean for multiple occupancy, and the dimensions
	 * of the grid.
	 * 
	 * @return void
	 */
	@SuppressWarnings("unchecked")
	protected void setupGrids()
	{
		System.out.println("Started setupGrids Method...");

		GridFactoryFinder.createGridFactory(null).createGrid(
				Constants.AGENT2D,
				context2,
				new GridBuilderParameters<SimpleAgent>(
						new repast.simphony.space.grid.WrapAroundBorders(),
						new SimpleGridAdder<SimpleAgent>(), true, param_xSize,
						param_ySize));

		GridFactoryFinder.createGridFactory(null).createGrid(
				Constants.LANDSCAPE2D,
				context2,
				new GridBuilderParameters<SimpleAgent>(
						new repast.simphony.space.grid.WrapAroundBorders(),
						new SimpleGridAdder<SimpleAgent>(), true, param_xSize,
						param_ySize));

		GridValueLayer vlLUType = new GridValueLayer(Constants.LANDUSETYPE,
				true, new repast.simphony.space.grid.WrapAroundBorders(),
				param_xSize, param_ySize);

		context2.addValueLayer(vlLUType);

		RasterLayer luLayer = (RasterLayer) LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AAGAppParameters.landUseTypeLayerIndex);
		for(int i = 0; i < param_xSize; i++)
		{
			for(int j = 0; j < param_ySize; j++)
			{
				// Set the land use layer just for display
				vlLUType.set(luLayer.getValueAt(i, j).doubleValue(), i,j);
			}
		}
		
		agentGrid = (Grid<SimpleAgent>) context2.getProjection(Constants.AGENT2D);
		SimpleAgent.agentGrid = agentGrid;
		
		System.out.println("Finished setupGrids Method.");
	}

	/**
	 * Set up and load landscape/land cover/land use/environment and other external data into
	 * the system. This will be done using LandCoverDataHelper
	 * 
	 * @see LandCoverDataHelper
	 */
	protected void setupLandscape()
	{
		System.out.println("Started setting up the landscape...");

		// Load all external data into the group object
		LandCoverDataHelper.loadRootLandscapeLayerGroup();
		
		// Set the reference layer for all actor agents
		SimpleAgent.referenceLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AAGAppParameters.referenceLayerIndex);
		
		this.param_xSize = SimpleAgent.referenceLayer.getCols();
		this.param_ySize = SimpleAgent.referenceLayer.getRows();

		System.out.println("Finished loading landscape/land cover/land use/environment and other external data");
	}

	
	/**
	 * 
	 */
	protected void setupActorAgents()
	{
		System.out.println("Started to setup sellerAgents...");
		int countSellers = 0;

		this.initAgents(AAGAppParameters.dataFolderName + AAGAppParameters.agentInitFile);
		
		// for every location where land exists, create a seller agent
		System.out.println("Finished setting up sellerAgents. There is " + countSellers + " in total.");
	}
	
	/**
	 * This method should be promoted to a separate class to initialize agents.
	 * @param fileName The CSV file having agent characteristics
	 */
	@SuppressWarnings("unchecked")
	protected void initAgents(String fileName)
	{
		CSVfileLoader agentLoader = new CSVfileLoader(fileName, this.context2);
		
		if(!agentLoader.load())
		{
			System.out.println("Error: Cannot initialize actor agents from " + fileName	+ ". Please check.");
		}
		
		// Update the parcel list, each parcel will get IDs of the cells it contains
		RasterLayer parcelIDLayer = (RasterLayer)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AAGAppParameters.parcelIDLayerIndex);
		LandCoverDataHelper.updateParcelList(parcelIDLayer);
	}

	/**
	 * For each Model Step, run the following method
	 */
	public void step()
	{
		System.out.println("Started Step Method...");
		
		// Save the intermediate land use type layer
		RasterLayer<?> dataLayer = (RasterLayer<?>)LandCoverDataHelper.rootLayerGroup.getLandscapeLayer(AAGAppParameters.landUseTypeLayerIndex);
		LandCoverDataHelper.saveRasterLayer(dataLayer, true, false);

		System.out.println("Finished Step Method.");
	}
}
