/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.land.exchange;

import sluceii.agent.SimpleAgent;
import sluceii.agent.decisionmaking.DecisionToBuy;
import sluceii.landmarket.LandBidRecord;
import sluceii.landmarket.LandListingRecord;
import sluceii.landmarket.LandMarketListing;
import sluceii.landmarket.LandBidRecord.BidRecordStatus;

/**
 * This class implements some buying actions that are related to the land market.<br> Those related to the buyer agents are not handled here. Each buyer agent should take care of her own resources and properties.
 * @author  Shipeng Sun
 * @version  1.0
 */
public class LandBuyingAction<T extends SimpleAgent & LandBuyingProcess>
{
	/**
	 * A reference to the land market listing
	 */
	protected LandMarketListing landMarket;
	
	/**
	 * The buyer agent
	 */
	private T buyerAgent;

	/**
	 * 
	 */
	public LandBuyingAction()
	{
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param landMarket
	 * @param buyerAgent
	 */
	public LandBuyingAction(LandMarketListing landMarket, T buyerAgent)
	{
		super();
		this.landMarket = landMarket;
		this.buyerAgent = buyerAgent;
	}

	/**
	 * @return  the landMarket
	 */
	public LandMarketListing getLandMarket()
	{
		return landMarket;
	}

	/**
	 * @param landMarket  the landMarket to set
	 */
	public void setLandMarket(LandMarketListing landMarket)
	{
		this.landMarket = landMarket;
	}

	/**
	 * @return  the buyerAgent
	 */
	public SimpleAgent getBuyerAgent()
	{
		return buyerAgent;
	}

	/**
	 * @param buyerAgent  the buyerAgent to set
	 */
	public void setBuyerAgent(T buyerAgent)
	{
		this.buyerAgent = buyerAgent;
	}
	
	/* 
	 * Bid on the specified parcel object.
	 * Depending on the limit on the number of bids on a parcel, this may or may not succeed.
	 */
	public boolean bidOnParcel(LandListingRecord listedParcelObj, DecisionToBuy decisionHelper)
	{
		// The buyer cannot bid on the one that has already got maximum number of bids on it.
		if(landMarket.getOpenBidNumber(listedParcelObj.getParcelID())>= listedParcelObj.getMaxBidsAllowed())
		{
			return false;
		}
		
		float wtp = (float)decisionHelper.calcWTPPrice(listedParcelObj.getParcelObj());
		float bidPrice = (float)decisionHelper.calcBidPrice(listedParcelObj.getParcelObj());
		
		// If the bidding price is lower than the WTA, it is rejected immediately.
		if(bidPrice < listedParcelObj.getWillingToAccept()) return false;
		
		LandBidRecord bidRecord = new LandBidRecord();
		bidRecord.setListingRecObj(listedParcelObj);
		
		bidRecord.setBidPrice(bidPrice);
		bidRecord.setWillingToPay(wtp);
		bidRecord.setRecordStatus(BidRecordStatus.open);
		bidRecord.setBuyer(buyerAgent);
		
		landMarket.addBidRecord(bidRecord);
		
		return true;
	}
	
	/**
	 * The seller has accepted this buyer/household's bid
	 */
	public void bidAccepted(LandBidRecord landBidRecord)
	{
		// Add this property/land parcel to this agent's property list
		buyerAgent.getPropertyList().add(landBidRecord.getParcelObj());
		landBidRecord.setRecordStatus(BidRecordStatus.accepted);
		
		// Notify seller to clean up.
		((LandSellingProcess)landBidRecord.getListingRecord().getSeller()).afterBidAction(landBidRecord);
		
		// Cancel other bids, necessary?
		//bidRecordList.remove(landBidRecord);
		
		buyerAgent.getCaptialResource().subtractAsset(landBidRecord.getBidPrice());
	}
	
	
	/**
	 * @param landBidRecord
	 */
	public void bidRejected(LandBidRecord landBidRecord)
	{
		// TODO Auto-generated method stub
		landBidRecord.setRecordStatus(BidRecordStatus.rejected);
	}
}
