/**
 * SLUCE 2 Project
 * http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.land.exchange;

import java.util.ArrayList;

import sluceii.agent.SimpleAgent;
import sluceii.app.aag.AAGAppParameters;
import sluceii.landmarket.LandBidRecord;
import sluceii.landmarket.LandMarketListing;
import sluceii.landmarket.LandBidRecord.BidRecordStatus;
import sluceii.landscape.Parcel;

/**
 * @author  Shipeng Sun
 * @version  1.0
 */
public class LandSellingAction<T extends SimpleAgent & LandSellingProcess>
{
	
	/**
	 * A reference to the land market listing service object
	 */
	protected LandMarketListing landMarket;
	
	
	/**
	 * The seller who is try to sell his/her properties
	 */
	protected T sellerAgent;
	
	/**
	 * 
	 */
	public LandSellingAction()
	{
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param landMarket
	 * @param sellerAgent
	 */
	public LandSellingAction(LandMarketListing landMarket,T sellerAgent)
	{
		super();
		this.landMarket = landMarket;
		this.sellerAgent = sellerAgent;
	}

	/**
	 * @return  the landMarket
	 */
	public LandMarketListing getLandMarket()
	{
		return landMarket;
	}

	/**
	 * @param landMarket  the landMarket to set
	 */
	public void setLandMarket(LandMarketListing landMarket)
	{
		this.landMarket = landMarket;
	}

	/**
	 * @return  the sellerAgent
	 */
	public SimpleAgent getSellerAgent()
	{
		return sellerAgent;
	}

	/**
	 * @param sellerAgent  the sellerAgent to set
	 */
	public void setSellerAgent(T sellerAgent)
	{
		this.sellerAgent = sellerAgent;
	}

	/**
	 * @param landCover
	 */
	public void putParcelOnMarket(Parcel parcelObj, float askPrice, float wta)
	{
		landMarket.addParcelOnMarket(parcelObj, (T)sellerAgent, askPrice, wta, AAGAppParameters.maxBidNums );
	}
	
	/**
	 * Check and possibly accept offer from the bidder
	 */
	public void checkBidOffers()
	{
		Parcel curParcelObj = null;
		LandBidRecord bidRecObj = null;
		
		float maxBidPrice = -10000.0f;
		int maxBidIndex = -1;
		float wta = 0;
		ArrayList<LandBidRecord> bidRecordList = null;
		
		for(int i = 0; i < sellerAgent.getPropertyList().size(); i++)
		{
			curParcelObj = (Parcel)sellerAgent.getPropertyList().get(i);
			bidRecordList = landMarket.getBidRecordsByParcelID(curParcelObj.getParcelID());
			
			for(int j = 0; j < bidRecordList.size(); j++)
			{
				bidRecObj = bidRecordList.get(j);
				wta = bidRecObj.getWillingToAccept();
				
				if(bidRecObj.getParcelID() == curParcelObj.getParcelID())
				{
					// Check out the highest bid price
					if(bidRecObj.getRecordStatus() == BidRecordStatus.open && bidRecObj.getBidPrice() > maxBidPrice)
					{
						maxBidPrice = bidRecObj.getBidPrice();
						maxBidIndex = j;
					}
				}
			}
			
			// if the maximum bid price is not less than the willingness to accept price, deal
			if(maxBidPrice >= wta)
			{
				this.acceptBidOffer(bidRecordList.get(maxBidIndex));
				
				// Change the status of all other records
				for(int j = 0; j < bidRecordList.size(); j++)
				{
					bidRecObj = bidRecordList.get(j);
					
					if(j == maxBidIndex)
					{
						bidRecObj.setRecordStatus(BidRecordStatus.accepted);
					}
					else
					{
						bidRecObj.setRecordStatus(BidRecordStatus.rejected);
					}
				}

			}
		}		
	}
	
	/**
	 * Remove the parcel from the listing and this agent have to give up the ownership.
	 * The land owner might keep all the bidding records for future price formation.
	 * This owner needs to notify s/he has accepted the offer from one buyer.
	 * @param landBidRecord
	 */
	public void acceptBidOffer(LandBidRecord landBidRecord)
	{
		((LandBuyingProcess)landBidRecord.getBuyer()).bidAccepted(landBidRecord);
	}

}
