/**
 * SLUCE 2 Project http://www.cscs.umich.edu/research/projects/sluce/
 */
package sluceii.landmarket;

import sluceii.agent.SimpleAgent;
import sluceii.land.exchange.LandSellingProcess;
import sluceii.landscape.Parcel;

/**
 * This is the record listed on the market. It contains the parcel, seller, ask
 * price, and willingness to accept information. Buyers search a list of these
 * records and evaluate some of these records to decide which parcels to bid on.
 * 
 * @see LandMarketListing
 * @author Shipeng Sun
 */
public class LandListingRecord
{
	/**
	 * @author Shipeng Sun
	 * The status of the listed parcel/property/house
	 */
	public enum ListingRecordStatus {listed, bidded, done};
	
	/**
	 * 
	 */
	public static int maxListingRecordID;
	
	/**
	 * 
	 */
	protected int listingRecordID;
	
	/**
	 * the status of the listing record
	 */
	protected ListingRecordStatus recordStaus;

	/**
	 * 
	 */
	protected Parcel parcelObj;

	/**
	 * 
	 */
	private SimpleAgent seller;

	/**
	 * ask price from the seller It may equal or higher than WTA
	 */
	protected float askPrice;

	/**
	 * WTA for the seller
	 */
	protected float willingToAccept;
	
	/**
	 * The maximum bids allowed on this parcel
	 */
	protected int maxBidsAllowed;

	/**
	 * 
	 */
	public LandListingRecord()
	{
		// TODO Auto-generated constructor stub
		this.listingRecordID = ++ maxListingRecordID;
		
		// The default status when the listing record is created is listed.
		this.recordStaus = ListingRecordStatus.listed;
	}

	public int getParcelID()
	{
		return parcelObj.getParcelID();
	}

	/**
	 * @return the parcelObj
	 */
	public Parcel getParcelObj()
	{
		return parcelObj;
	}

	/**
	 * @param parcelObj
	 *            the parcelObj to set
	 */
	public void setParcelObj(Parcel parcelObj)
	{
		this.parcelObj = parcelObj;
	}

	/**
	 * @return the askPrice
	 */
	public float getAskPrice()
	{
		return askPrice;
	}

	/**
	 * @param askPrice
	 *            the askPrice to set
	 */
	public void setAskPrice(float askPrice)
	{
		this.askPrice = askPrice;
	}

	/**
	 * @return the willingToAccept
	 */
	public float getWillingToAccept()
	{
		return willingToAccept;
	}

	/**
	 * @param willingToAccept
	 *            the willingToAccept to set
	 */
	public void setWillingToAccept(float willingToAccept)
	{
		this.willingToAccept = willingToAccept;
	}

	/**
	 * @return the seller
	 */
	public SimpleAgent getSeller()
	{
		return seller;
	}

	/**
	 * @param <T>
	 * @param <T>
	 * @param seller the seller to set
	 */
	public <T extends SimpleAgent & LandSellingProcess> void setSeller(T seller)
	{
		this.seller = seller;
	}

	/**
	 * @return the maxBidsAllowed
	 */
	public int getMaxBidsAllowed()
	{
		return maxBidsAllowed;
	}

	/**
	 * @param maxBidsAllowed the maxBidsAllowed to set
	 */
	public void setMaxBidsAllowed(int maxBidsAllowed)
	{
		this.maxBidsAllowed = maxBidsAllowed;
	}

	/**
	 * @return the recordStaus
	 */
	public ListingRecordStatus getRecordStaus()
	{
		return recordStaus;
	}

	/**
	 * @param recordStaus the recordStaus to set
	 */
	public void setRecordStaus(ListingRecordStatus recordStaus)
	{
		this.recordStaus = recordStaus;
	}

}
