package sluceii.params;

import sluceii.app.aag.AAGAppParameters;

/**
 * Log Helper factory,
 * 
 * @author arcabrer
 */
public class LogHelperFactory
{
	public enum LOGGER_TYPE {FILE, MYSQL};

	private static LOGGER_TYPE logger = LOGGER_TYPE.FILE;

	/**
	 * lazy loaded shared among threads; will need to test on the grid
	 */
	private static MySQLDatabaseLogger mySQLinstance = null; 

	/**
	 * @param debugLevel
	 * @return
	 */
	public static LogHelper getLogger(int debugLevel)
	{
		switch (logger)
		{
			case MYSQL:
				if(LogHelperFactory.mySQLinstance == null)
				{
					LogHelperFactory.mySQLinstance = new MySQLDatabaseLogger(); 
				}
				// reuse shared instance of MySQL database connection for better
				// resource usage; may fail on grid / multiple threads
				// other option is to create MySQLDatabaseLogger here.
				mySQLinstance.landscapeDebugLevel = (byte) debugLevel;
				return mySQLinstance;
			case FILE:
			default:
				return new SimpleLogHelper(AAGAppParameters.dataFolderName
						+ AAGAppParameters.logFileName, debugLevel, false);
		}
	}

	/**
	 * @param debugLevel
	 * @return
	 */
	public static LogHelper getTransactionLogger(int debugLevel)
	{
		switch (logger)
		{
			case MYSQL:
				// reuse shared instance of MySQL database connection for better
				// resource usage; may fail on grid / multiple threads
				// other option is to create MySQLDatabaseLogger here.
				mySQLinstance.transactionDebugLevel = (byte) debugLevel;
				return mySQLinstance;
			case FILE:
			default:
				return new SimpleLogHelper(AAGAppParameters.dataFolderName
						+ "/Log/trans.csv", debugLevel, true);
		}
	}
	
	/**
	 * @param debugLevel
	 * @return
	 */
	public static LogHelper createTransactionLogger(int debugLevel, String fileName, boolean addTimeStamp)
	{
		switch (logger)
		{
			case MYSQL:
				// reuse shared instance of MySQL database connection for better
				// resource usage; may fail on grid / multiple threads
				// other option is to create MySQLDatabaseLogger here.
				mySQLinstance.transactionDebugLevel = (byte) debugLevel;
				return mySQLinstance;
			case FILE:
			default:
				return new SimpleLogHelper(fileName, debugLevel, addTimeStamp);
		}
	}
}
