package sluceii.params;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import sluceii.app.aag.AAGAppParameters;

/**
 * A logger writing to a MySQL database. 
 * 
 * Modified by Raymond Cabrera.
 * 
 * @author arcabrer
 * @version 1.0
 */
public class MySQLDatabaseLogger implements LogHelper
{
	private Connection conn;
	private int runID = -1;

	private PreparedStatement stmtLandscape;
	private PreparedStatement stmtTransactions;

	/**
	 * if debugLevel is less than or equal to zero, nothing is done.
	 */
	public byte landscapeDebugLevel = 1;
	public byte transactionDebugLevel = 1;

	public MySQLDatabaseLogger()
	{
		this("jdbc:mysql://localhost/sluceII?user=sluce&password=sluceDBuw");
	}

	public MySQLDatabaseLogger(String url)
	{
		try
		{
			conn = DriverManager.getConnection("jdbc:mysql://localhost/sluceII?user=sluce&password=sluceDBuw");
			conn.setAutoCommit(false);
		}
		catch (SQLException e)
		{
			e.printStackTrace();
		}
	}

	@Override
	public void logRun()
	{
		// create record in tblRun
		try
		{
			// for now, based on parameters provided to Tianyi and those in
			// AAGAppParameters.paramsValueToStringWithoutControl
			PreparedStatement s = conn
					.prepareStatement("INSERT INTO tblRun (worldSize, numBuyer, numSeller, maxBidNums, "
							+ "randomSeed, runNumber, utilityBeta, sdUtilBeta, agResPrice, meanBudget, sdBudget, neighborhoodSize, numSearchParcel) "
							+ "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
			s.setInt(1, AAGAppParameters.worldYSize);
			s.setInt(2, AAGAppParameters.numBuyer);
			s.setInt(3, AAGAppParameters.numSeller);
			s.setInt(4, AAGAppParameters.maxBidNums);
			s.setDouble(5, AAGAppParameters.randomSeed);
			s.setInt(6, AAGAppParameters.runNumber);
			s.setDouble(7, AAGAppParameters.utilityBeta);
			s.setDouble(8, AAGAppParameters.sdUtilBeta);
			s.setDouble(9, AAGAppParameters.agResPrice);
			s.setDouble(10, AAGAppParameters.meanBudget);
			s.setDouble(11, AAGAppParameters.sdBudget);
			s.setInt(12, AAGAppParameters.neighborhoodSize);
			s.setInt(13, AAGAppParameters.numSearchParcel);
			s.execute();

			// get runID from database
			ResultSet rs = s.getGeneratedKeys();
			if (rs.next())
			{
				runID = rs.getInt(1);
			}
			else
			{
				throw new Exception("Could not get runID from database.");
			}
			rs.close();
			rs = null;

			stmtLandscape = conn
					.prepareStatement("INSERT INTO tblLandscapeFiles (runID, tickCount, luFile, tpFile, luFileB, tpFileB) "
							+ "VALUES (?, ?, ?, ?, ?, ?)");
			stmtTransactions = conn
					.prepareStatement("INSERT INTO tblTransactions (runID, tickCount, agentID, bidPrice, "
							+ "preferenceForProximity, dist, osa, tcost, tbudg, util) "
							+ "VALUES (?, ?, ?, ?, ?, ?)");
		}
		catch (SQLException e)
		{
			System.err.println("Error logging the run.");
			e.printStackTrace();
		}
		catch (Exception e)
		{
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
	}

	@Override
	public void logLandscapeData(int debugLvl, double tickCount,
			String fileName, String priceFile, String fileNameB,
			String priceFileB)
	{
		// create record in tblLandscapeFiles
		try
		{
			stmtLandscape.setInt(1, runID);
			stmtLandscape.setDouble(2, tickCount);
			stmtLandscape.setString(3, fileName);
			stmtLandscape.setString(4, priceFile);
			stmtLandscape.setString(5, fileNameB);
			stmtLandscape.setString(6, priceFileB);
			stmtLandscape.execute();
		}
		catch (SQLException e)
		{
			e.printStackTrace();
		}
	}

	@Override
	public void logTransaction(int debugLvl, double tickCount, int agentID,
			double bidPrice, double preferenceForProximity, double dist,
			double osa, double tcost, double tbudg, double util)
	{
		// create record in tblTransactions
		try
		{
			stmtTransactions.setInt(1, runID);
			stmtTransactions.setDouble(2, tickCount);
			stmtTransactions.setInt(3, agentID);
			stmtTransactions.setDouble(4, bidPrice);
			stmtTransactions.setDouble(5, preferenceForProximity);
			stmtTransactions.setDouble(6, dist);
			stmtTransactions.setDouble(7, osa);
			stmtTransactions.setDouble(8, tcost);
			stmtTransactions.setDouble(9, tbudg);
			stmtTransactions.setDouble(10, util);
			stmtTransactions.execute();
		}
		catch (SQLException e)
		{
			e.printStackTrace();
		}
	}

	public void close()
	{
		try
		{
			stmtLandscape.close();
		}
		catch (SQLException e)
		{
		}

		try
		{
			stmtTransactions.close();
		}
		catch (SQLException e)
		{
		}

		try
		{
			conn.commit();
		}
		catch (SQLException e1)
		{
			System.err
					.println("Database error: Transaction could not be committed.");
			e1.printStackTrace();
		}

		try
		{
			conn.close();
		}
		catch (SQLException e)
		{
		}
	}

	@Override
	public LoggerType getLoggerType()
	{
		// TODO Auto-generated method stub
		return LoggerType.mySQLLogger;
	}
}
